C
C
C
C---- DIGS ------------------------------------------------------------M
C
C           ---> DEVICE INDEPENDENT GRAPHICS SUBROUTINES <---
C
C  PURPOSE:  The device independent graphics subroutines write graphic
C            output instructions to a plot file (metafile).  This file
C            can be read by another program which interprets the
C            instructions and sends the appropriate commands to a
C            plotting device, such as a terminal screen, a digital
C            plotter, a dot matrix printer, or a laser jet printer.
C            The DIGS emulate Calcomp's basic FORTRAN software package
C            for electromechanical plotters, which are called by
C            FESWMS-2DH programs to produce graphic output.
C            Plotting instructions are written to a file (referred to
C            as a plot file) in the Hewlette-Packard Graphics Language
C            (HP-GL).  An HP-GL instruction is a two-letter mnemonic
C            followed by its parameter filed, if any, and a terminator.
C            If parameters follow the mnemonic, they must be separated
C            from each other by at least one comma or space, or by a
C            + or - sign which may be preceded by commas or spaces.
C            An instruction is terminated by a semicolon or the next
C            mnemonic.
C
C----------------------------------------------------------------------F
C
C
C
      SUBROUTINE PLOTS(I1,I2,UNITSPEC)
C
C---- PLOTS -----------------------------------------------------------S
C
C  PURPOSE:  Initializes for plotting.
C
C  INPUT ARGUMENT(S):
C    I1,I2    - Dummy arguments.
C    UNITSPEC - Unit specifier for plotter output.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   I1,I2,UNITSPEC
C
C---- Declarations (local)
      INTEGER   IUNIT
C
C---- Common blocks
      COMMON /DIGS/ IUNIT
C
C---- Output Formats
2010  FORMAT('IN;')
C
C---- Take care of dummies
      I1=I1
      I2=I2
C
C---- Put unit specifier in common
      IUNIT=UNITSPEC
C
C---- Initialize for plotting
      WRITE(IUNIT,2010)
C
C---- Select pen number 1
      CALL NEWPEN(1)
C
      RETURN
      END
C
C
C
      SUBROUTINE PLOT(X,Y,IPEN)
C
C---- PLOT ------------------------------------------------------------S
C
C  PURPOSE:
C    PLOTS STRAIGHT LINE BETWEEN TWO POINTS AND DEFINES NEW PLOT
C    ORIGINS.
C
C  INPUT ARGUMENT(S):
C    X,Y -  ARE THE X,Y COORDINATES, IN INCHES FROM THE CURRENT ORIGIN,
C           OF THE POSITION TO WHICH THE PEN IS TO BE MOVED.  AN ORIGIN
C           (WHERE BOTH X AND Y EQUAL ZEO) MAY BE ESTABLISHED ANYWHERE
C           ON (OR OFF) THE PLOTTING SURFACE.
C    IPEN - A SIGNED INTEGER THAT CONTROLS THE PEN STATUS (UP OR DOWN)
C           AND ORIGIN DEFINITION.  IF IPEN = 2, THE PEN IS DOWN DURING
C           MOVEMENT, THUS DRAWING A VISIBLE LINE.  IF IPEN = 3, THE
C           PEN IS UP DURING MOVEMENT.  IF IPEN = -2 OR -3, A NEW
C           ORIGIN IS DEFINED AT THE TERMINAL POSITION OF THE MOVE.
C           IF IPEN = 999, THE EFFECT IS THE SAME AS FOR IPEN = -3
C           EXCEPT THAT THE PLOT FILE IS ALSO CLOSED.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IPEN
      REAL      X,Y
C
C---- Declarations (local)
      INTEGER   IUNIT,NCHARX,NCHARY
      CHARACTER CHARX*12,CHARY*12
C
C---- Common blocks
      COMMON /DIGS/ IUNIT
C
C---- Output Formats
2020  FORMAT('PD',(A),',',(A),';')
2030  FORMAT('PU',(A),',',(A),';')
2050  FORMAT('IP;')
C
C---- Convert floating point coordinates to character strings.  Note
C     that coordinates are converted to HP plotter units (1021 per inch).
      CALL R2CHAR(1021.*X,-1,CHARX,NCHARX)
      CALL R2CHAR(1021.*Y,-1,CHARY,NCHARY)
C
C---- Move with pen down if IPEN = 2
      IF(IABS(IPEN).EQ.2) THEN
        WRITE(IUNIT,2020) CHARX(1:NCHARX),CHARY(1:NCHARY)
C
C---- Move with pen up if IPEN = 3 or 999
      ELSEIF(IABS(IPEN).EQ.3 .OR. IPEN.EQ.999) THEN
        WRITE(IUNIT,2030) CHARX(1:NCHARX),CHARY(1:NCHARY)
      ENDIF
C
C---- Set new scaling points if IPEN < 0
      IF(IPEN.LT.0) WRITE(IUNIT,2050)
C
C---- Store pen and close plot file if IPEN = 999
      IF(IPEN.EQ.999) THEN
        CALL NEWPEN(0)
        CLOSE(IUNIT,STATUS='KEEP')
      ENDIF
C
      RETURN
      END
C
C
C
      SUBROUTINE SYMBOL(X,Y,HEIGHT,STRING,INTEQ,ANGLE,NCHAR)
C
C---- SYMBOL ----------------------------------------------------------S
C
C  PURPOSE:
C    PLOTS ANNOTATION AND SPECIAL SYMBOLS.
C
C  INPUT ARGUMENT(S):
C    X,Y    - COORDINATES, IN INCHES, OF THE LOWER LEFT-HAND CORNER
C             (BEFORE ROTATION) OF THE FIRST CHARACTER TO BE PRODUCED
C             IF NCHAR >= 0 OR OF THE GEMETRIC CENTER OF THE CHARACTER
C             TO BE PRODUCED IF NCHAR < 0.  THE PEN IS UP DURING THE
C             MOVE IF NCHAR >= 0, THE PEN STATUS IS DETERMINED BY NCHAR
C             IF NCHAR < 0.
C    HEIGHT - HEIGHT, IN INCHES, OF THE CHARACTER(S) TO BE PLOTTED.
C             THE WIDTH OF A CHARACTER, INCLUDING SPACING, IS NORMALLY
C             THE SAME AS THE HEIGHT.
C    CHARS  - CHARACTER STRING ARRAY CONTAINING THE CHARACTER(S) TO BE
C             PLOTTED IF NCHAR >= 0.
C    INTEQ  - INTEGER EQUIVALENT OF THE DESIRED SYMBOL IF A SPECIAL
C             SYMBOL IS TO BE PRODUCED (1-14).
C    ANGLE  - ANGLE, IN DEGREES FROM THE X-AXIS, AT WHICH THE
C             CHARACTER(S) ARE TO BE PLOTTED.  IF ANGLE = 0.0, THE
C             CHARACTER(S) WILL BE PLOTTED RIGHT SIDE UP AND PARALLEL
C             TO THE X-AXIS.
C    NCHAR  - >0, THE NUMBER OF CHARACTERS TO BE PLOTTED FROM STRING.
C             -1, A SPECIAL SYMBOL WILL BE PLOTTED, PEN IS UP DURING
C                 THE MOVE.
C             -2, A SPECIAL SYMBOL WILL BE PLOTTED,  PEN IS DOWN DURING
C                 THE MOVE.
C
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   INTEQ,NCHAR
      REAL      X,Y,HEIGHT,ANGLE
      CHARACTER STRING*(*)
C
C---- Declarations (local)
      INTEGER   IUNIT,NC1,NC2,IPEN
      REAL      H,W,A,RISE,RUN,ZIP,DTOR
      CHARACTER STR1*12,STR2*12,CHAR14*14
C
C---- Common blocks
      COMMON /DIGS/ IUNIT
C
C---- Data
      DATA CHAR14/'ABCDEFGHIJKLMN'/,ZIP/0.0004/
      DATA DTOR/0.0174533/
C
C---- Output Formats
2010  FORMAT('SI',(A),',',(A),';')
2020  FORMAT('DI',(A),',',(A),';')
2030  FORMAT('LB',(A))
C
C---- Move to character location
      IPEN=3
      IF(NCHAR.EQ.-2) IPEN=2
      CALL PLOT(X,Y,IPEN)
C
C---- Set absolute character size in centimeters
      H=2.54*HEIGHT
      W=0.695*H
      CALL R2CHAR(H,3,STR1,NC1)
      CALL R2CHAR(W,3,STR2,NC2)
      WRITE(IUNIT,2010) STR2(1:NC2),STR1(1:NC1)
C
C---- Set absolute direction FOR CHARACTER PLOTTING
      A=DTOR*ANGLE
      RISE=SIN(A)
      RUN=COS(A)
      CALL R2CHAR(RISE,3,STR1,NC1)
      CALL R2CHAR(RUN,3,STR2,NC2)
      WRITE(IUNIT,2020) STR2(1:NC2),STR1(1:NC1)
C
C-----PLOT TEXT STRING
      IF(NCHAR.LT.0) THEN
        STRING(1:1)=CHAR14(INTEQ:INTEQ)
        NCHAR=1
      ENDIF
      NCHAR=NCHAR+1
      STRING(NCHAR:NCHAR)=CHAR(3)
      WRITE(IUNIT,2030) STRING(1:NCHAR)
C
      RETURN
      END
C
C
C
      SUBROUTINE NEWPEN(INP)
C
C---- NEWPEN ----------------------------------------------------------S
C
C  PURPOSE:
C    SELECTS PEN ON MULTIPEN PLOTTERS.
C
C  INPUT ARGUMENT(S):
C    INP - NUMBER OF THE PEN SELECTED.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   INP
C
C---- Declarations (local)
      INTEGER   IUNIT
      CHARACTER STRING*8
C
C---- Common blocks
      COMMON /DIGS/ IUNIT
C
C---- Output Formats
2010  FORMAT('SP',(A),';')
C
      CALL R2CHAR(REAL(INP),-1,STRING,NCHAR)
      WRITE(IUNIT,2010) STRING(1:NCHAR)
C
      RETURN
      END
C
C
C
      SUBROUTINE NUMBER(X,Y,HTN,FPN,DEG,NDEC)
C
C---- NUMBER ----------------------------------------------------------S
C
C  PURPOSE:
C    CONVERTS A NUMBER TO A CHARACTER STRING AND PLOTS IT.
C
C  INPUT ARGUMENT(S):
C    X,Y - COORDINATES OF THE LOWER LEFT CORNER OF THE NUMBER.
C    HTN - HEIGHT OF THE NUMBER TO BE PLOTTED.
C    FPN - FLOATING POINT NUMBER TO BE PLOTTED.
C    DEG - ANGLE AT WHICH THE NUMBER IS TO BE PLOTTED, IN DEGREES.
C    NDEC - NUMBER OF DIGITS TO RIGHT OF DECIMAL POINT TO BE INCLUDED.
C
C  SUBPROGRAM(S) CALLED:
C    SYMBOL, R2CHAR
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NDEC
      REAL      X,Y,HTN,FPN,DEG
C
C---- Declarations (local)
      CHARACTER STRING*12
C
      CALL R2CHAR(FPN,NDEC,STRING,NCHAR)
      CALL SYMBOL(X,Y,HTN,STRING,0,DEG,NCHAR)
C
      RETURN
      END
C
C
C
      SUBROUTINE XLINE(XARRAY,YARRAY,NPTS,LINTYP,X0,Y0)
C
C---- XLINE -----------------------------------------------------------S
C
C  PURPOSE:
C    PLOTS A LINE DEFINED BY TWO COORDINATE ARRAYS.
C
C  INPUT ARGUMENT(S):
C    XARRAY - NAME OF ARRAY CONTAINING ABSCISSA OR X VALUES.
C    YARRAY - NAME OF ARRAY CONTAINING ORDINATE OR Y VALUES.
C    NPTS - NUMBER OF POINTS TO BE PLOTTED + 2.
C    LINTYP - XLINE TYPE CODE.
C    X0,Y0 - COORDINATES OF THE PLOT ORIGIN.
C
C  SUBPROGRAM(S) CALLED:
C    PLOT, BOX
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NPTS,LINTYP
      REAL      XARRAY,YARRAY,X0,Y0
C
C---- Declarations (local)
      INTEGER   LOC1,LOCD
      REAL      FIRSTX,DELTAX,FIRSTY,DELTAY,X,Y
C
C---- Dimensions
      DIMENSION XARRAY(NPTS),YARRAY(NPTS)
C
      LOC1=NPTS+1
      LOCD=NPTS+2
      FIRSTX=XARRAY(LOC1)
      DELTAX=XARRAY(LOCD)
      FIRSTY=YARRAY(LOC1)
      DELTAY=YARRAY(LOCD)
      X=X0+(XARRAY(1)-FIRSTX)/DELTAX
      Y=Y0+(YARRAY(1)-FIRSTY)/DELTAY
      CALL PLOT(X,Y,3)
      DO 10 I=2,NPTS
        X=X0+(XARRAY(I)-FIRSTX)/DELTAX
        Y=Y0+(YARRAY(I)-FIRSTY)/DELTAY
        CALL PLOT(X,Y,2)
        IF(LINTYP.GT.0) CALL BOX(X,Y,0.07)
10    CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE AXIS(X0,Y0,LABEL,NCHAR,AXLEN,ANGLE,FIRSTV,DELTAV,NDEC)
C
C---- AXIS ------------------------------------------------------------S
C
C  PURPOSE:
C    PLOTS A GRAPH AXIS INCLUDING TIC MARKS, SCALE VALUES, AND LABEL.
C
C  INPUT ARGUMENT(S):
C    X0,Y0 - COORDINATES OF STARTING POINT OF AXIS, IN INCHES
C    LABEL - AXIS TITLE.
C    NCHAR - NUMBER OF CHARACTERS IN TITLE.
C    AXLEN - FLOATING POINT AXIS LENGTH IN INCHES.
C    ANGLE - ANGLE OF AXIS FROM THE X-DIRECTION, IN DEGREES.
C    FIRSTV - SCALE VALUE AT THE FIRST TIC MARK.
C    DELTAV - CHANGE IN SCALE BETWEEN TIC MARKS ONE INCH APART
C    NDEC - NUMBER OF DIGITS TO BE PLOTTED TO RIGHT OF DECIMAL POINT.
C           IF NDEC=-1, ONLY THE INTEGER PORTION IS PLOTTED.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NCHAR,NDEC
      REAL      X0,Y0,AXLEN,ANGLE,FIRSTV,DELTAV
      CHARACTER LABEL(NCHAR),NBCD*3
C
C---- Declarations (local)
      INTEGER   KN
      REAL      EX,ADX,XVAL
C
      NBCD ='*10'
      KN=NCHAR
      A=1.0
      IF(KN.LT.0) THEN
        A=-A
        KN=-KN
      ENDIF
      EX=0.0
      ADX=ABS(DELTAV)
      IF( ADX.LT.0. .OR. ADX.GT.0. ) THEN
10      IF(ADX.GT.999) THEN
          ADX=ADX/10.
          EX=EX+1.
          GOTO 10
        ENDIF
20      IF(ADX.LT.0.01) THEN
          ADX=ADX*10.
          EX=EX-1.
          GOTO 20
        ENDIF
      ENDIF
      XVAL=FIRSTV*10.0**(-EX)
      ADX=DELTAV*10.0**(-EX)
      STH=ANGLE*0.0174533
      CTH=COS(STH)
      STH=SIN(STH)
      DXB=-0.1
      DYB=0.15*A-0.05
      XN=X0+DXB*CTH-DYB*STH
      YN=Y0+DYB*CTH+DXB*STH
      NTIC=AXLEN+1.0
      NT=NTIC/2
      DO 30 I=1,NTIC
        CALL NUMBER(XN,YN,0.105,XVAL,ANGLE,NDEC)
        XVAL=XVAL+ADX
        XN=XN+CTH
        YN=YN+STH
        IF(NT.EQ.0) THEN
          Z=KN
          IF( EX.LT.0. .OR. EX.GT.0. ) Z=Z+7.
          DXB=-.07*Z+AXLEN*0.5
          DYB=0.325*A-0.075
          XT=X0+DXB*CTH-DYB*STH
          YT=Y0+DYB*CTH+DXB*STH
          CALL SYMBOL(XT,YT,0.14,LABEL,0,ANGLE,KN)
          IF( EX.LT.0. .OR. EX.GT.0. ) THEN
            Z=KN+2
            XT=XT+Z*CTH*0.14
            YT=YT+Z*STH*0.14
            CALL SYMBOL(XT,YT,0.14, NBCD,0,ANGLE,3)
            XT=XT+(3.0*CTH-0.8*STH)*0.14
            YT=YT+(3.0*STH+0.8*CTH)*0.14
            CALL NUMBER(XT,YT,0.07,EX,ANGLE,-1)
          ENDIF
        ENDIF
        NT=NT-1
30    CONTINUE
      CALL PLOT(X0+AXLEN*CTH,Y0+AXLEN*STH,3)
      DXB=-0.07*A*STH
      DYB=+0.07*A*CTH
      A=NTIC-1
      XN=X0+A*CTH
      YN=Y0+A*STH
      DO 40 I=1,NTIC
        CALL PLOT(XN,YN,2)
        CALL PLOT(XN+DXB,YN+DYB,2)
        CALL PLOT(XN,YN,2)
        XN=XN-CTH
        YN=YN-STH
40    CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE SCALE(ARRAY,AXLEN,NPTS)
C
C---- SCALE -----------------------------------------------------------S
C
C  PURPOSE:
C    SCALES VALUES TO BE ALONG AN AXIS.
C
C  INPUT ARGUMENT(S):
C    ARRAY - NAME OF ARRAY CONTAINING VALUES TO BE SCALED.
C    AXLEN - LENGTH IN INCHES OVER WHICH ARRAY IS TO BE SCALED.
C    NPTS - NUMBER OF POINTS TO BE SCALED + 2.
C
C  OUTPUT ARGUMENT(S):
C     ARRAY(NPTS+1) - FIRSTV, THE FIRST VALUE ON THE AXIS.
C     ARRAY(NPTS+2) - DELTAV, THE INCREMENTAL VALUE PER INCH.
C
C----------------------------------------------------------------------F
C
      DIMENSION  ARRAY(NPTS),SAVE(6)
      DATA SAVE/1.,2.,4.,5.,10.,20./
      VMIN=ARRAY(1)
      VMAX=VMIN
      DO 10 I=1,NPTS
        V=ARRAY(I)
        IF(V.LT.VMIN) THEN
          VMIN=V
        ELSEIF(V.GT.VMAX) THEN
          VMAX=V
        ENDIF
10    CONTINUE
      DELTAV=(VMAX-VMIN)/AXLEN
      IF(ABS(DELTAV).LT.1.0E-08) DELTAV=AMAX1(0.1*VMIN,0.1)
      I=ALOG10(DELTAV)
      P=10.**REAL(I)
      DELTAV=DELTAV/P-0.001
      DO 20 I=1,6
        IS=I
        IF(SAVE(I).GT.DELTAV) GOTO 30
20    CONTINUE
30    DELTAV=P*SAVE(IS)
      FIRSTV=DELTAV*AINT(VMIN/DELTAV)
      T=FIRSTV+AXLEN*DELTAV
      IF(T.LT.VMAX) THEN
        IS=IS+1
        IF(IS.GT.6) GOTO 40
        GOTO 30
      ENDIF
40    FIRSTV=FIRSTV-AINT(0.5*(AXLEN-(VMAX-FIRSTV)/DELTAV))*DELTAV
      ARRAY(NPTS+1)=FIRSTV
      ARRAY(NPTS+2)=DELTAV
C
      RETURN
      END
C
C
C
      SUBROUTINE BOX(X,Y,SIDE)
C
C---- BOX -------------------------------------------------------------S
C
C  PURPOSE:
C    DRAWS A BOX CENTERED AROUND A POINT.  THE PEN IS ASSUMED TO BE
C    INITIALLY LOCATED AT THE POINT.
C
C  INPUT ARGUMENT(S):
C    X,Y - COORDINATES OF THE POINT, IN INCHES.
C    SIDE - LENGTH OF THE BOX SIDE, IN INCHES.
C
C  SUBPROGRAM(S) CALLED:
C    PLOT
C
C----------------------------------------------------------------------F
C
      XP=X+0.5*SIDE
      YP=Y+0.5*SIDE
      CALL PLOT(XP,YP,3)
      XP=XP-SIDE
      CALL PLOT(XP,YP,2)
      YP=YP-SIDE
      CALL PLOT(XP,YP,2)
      XP=XP+SIDE
      CALL PLOT(XP,YP,2)
      YP=YP+SIDE
      CALL PLOT(XP,YP,2)
      CALL PLOT(X,Y,3)
      RETURN
      END
C
C
C
      SUBROUTINE SDOT(X,Y,D)
C
C---- SDOT ------------------------------------------------------------S
C
C  PURPOSE:
C    DRAWS A SOLID FILLED DOT.
C
C  INPUT ARGUMENT(S);
C    X,Y - COORDINATES OF THE CENTER OF THE DOT, IN INCHES.
C    D   - DOT DIAMETER, IN INCHES.
C
C----------------------------------------------------------------------F
C
      COMMON /DIGS/ IUNIT
      CHARACTER*12 RCH
C
C---- Output Formats
2000  FORMAT('WG',(A),',0,360;')
C
C---- Move pen to dot center
      CALL PLOT(X,Y,3)
C
C---- Convert floating point numbers to character strings
      R=.5*D
      CALL R2CHAR(R,3,RCH,NRCH)
C
C---- Draw solid filled dot (shaded wedge)
      WRITE(IUNIT,2000) RCH(1:NRCH)
C
      RETURN
      END
