C
C     FLOMOD -- Version 1.10.DG1
C
      PROGRAM FLOMOD
C
C---- FLOMOD ----------------------------------------------------------M
C
C           FINITE ELEMENT SURFACE WATER MODELING SYSTEM FOR
C       TWO-DIMENSIONAL FLOW IN THE HORIZONTAL PLANE *FESWMS-2DH*
C
C             DEPTH-AVERAGED FLOW MODULE - VERSION 1.10.DG1
C               (FOR USE ON DATA GENERAL AViiON SYSTEMS)
C
C                         ----> FLOMOD <----
C
C                            DEVELOPED FOR
C
C                 U.S. DEPARTMENT OF TRANSPORTATION
C                   FEDERAL HIGHWAY ADMINISTRATION
C            OFFICE OF ENGINEERING AND HIGHWAY OPERATIONS
C                      RESEARCH AND DEVELOPMENT
C               6300 GEORGETOWN PIKE, MCLEAN, VA 22101
C
C                                 BY
C
C           U.S. GEOLOGICAL SURVEY, WATER RESOURCES DIVISION
C
C  PURPOSE:
C    THE DEPTH-AVERAGED FLOW MODULE SIMULATES STEADY OR UNSTEADY, TWO
C    DIMENSIONAL, VERTICALLY-AVERAGED SURFACE-WATER FLOW.  THE PROGRAM
C    SOLVES THE DEPTH-AVERAGED EQUATIONS OF MOTION AND CONTINUITY TO
C    OBTAIN HORIZONTAL VELOCITIES AND DEPTHS OF FLOW.  THE EFFECTS OF
C    BOTTOM FRICTION AND TURBULENT STRESSES ARE CONSIDERED AS ARE,
C    OPTIONALLY, SURFACE WIND STRESS AND THE CORIOLIS FORCE.  STRESSES
C    DUE TO TURBULENCE ARE MODELED USING BOUSSINESQ'S EDDY VISCOSITY
C    CONCEPT.
C
C    THE PROGRAM PRODUCES AN OUTPUT FILE CONSISTING OF FLOW-FIELD DATA
C    THAT MAY BE USED BY OTHER FESWMS PROGRAMS.
C
C  SUBPROGRAM(S) CALLED:
C    INPUT1, PRECMP, QCHECK, EQLOAD, NPIDEN, BCODE2, XSECQ,
C    BANGLE, ZFRONT, UPDATE, SOLCOR, OUTPUT, INPUT2, CNORMS
C
C  PROGRAMMER: David C. Froehlich      DATE: January 1, 1988
C
C  MODIFICATIONS:
C  NO.  DATE     PROGRAMMER     DESCRIPTION
C  ==== ======== ============== ========================================
C  1.01 06/03/88 D.C. Froehlich 1. Corrected sub. XSECZ to specify
C                                  level and sloping water-surface
C                                  elevs.
C                               2. Corrected sub. INPUT2 to read
C                                  time-dependent data sets.
C                               3. Corrected sub. WEIRQ to multiply
C                                  dimensionless coefficient by
C                                  SQRT(GRAV).
C                               4. Added sub. OPENF to open temporary
C                                  and needed data files.
C                               5. Corrected subs. RDQSEC and RDZSEC to
C                                  allow up to 10 QSEC or ZSEC cross
C                                  sections to be entered.
C                               6. Corrected error message 403 in sub.
C                                  RFINIT.
C                               7. Corrected main program to properly
C                                  assign initial estimates at the start
C                                  of a new time step.
C                               8. Changed output format headings for
C                                  initial conditions and results.
C                               9. Corrected common block BLKAR by
C                                  by adding WVMIN.
C  1.10 02/24/89 D.C. Froehlich 1. General housecleaning.
C  1.10.DG1
C       05/29/92 K.G.Thibodeaux 1. Added DG Time and Date calls.
C                                  DG system subroutines IDATE and
C                                  TIME.
C                               2. Made modifications in FLOMOD.INC
C                                  file to allow usage on DG equipment.
C                                  (LBMAX set at optimum value.)
C                               3. All INCLUDE statements in the program
C                                  have been changed to be compatible
C                                  with DG and PC compilers.
C                               4. Modified the following sub-programs
C                                  and FLOMOD.INC file to allow the
C                                  user set the maximum number of nodes
C                                  across QSEC and ZSEC cross sections
C                                  (MAXSP); RDQSEC, RDZSEC, XSECQ, and
C                                  XSECZ.
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   LENCHR,IER,IC,IT,ITLO,ITUP,NCYC,NCYL,IDP,N,K,L,ISW
      INTEGER IMON,IDAY,IYEAR
      REAL      DELTS,THETAP
      CHARACTER DATE*8,TIMEBUF*8
      LOGICAL   EXISTS,OPENED
C
C---- Common blocks
      COMMON/CISW/ISW
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(' ***ERROR #400 - FLOMOD INPUT DATA FILE "',(A),
     # '" CANNOT BE FOUND.')
2010  FORMAT(' DATE IS ',I2,'/',I2,'/',I2,' (MM/DD/YY)')
2020  FORMAT(' TIME IS ',A8,' (HH:MM:SS)')
2050  FORMAT(/
     # ' *** FLOMOD RUN TERMINATED BECAUSE OF ',I4,' ERROR(S)!!!')
2060  FORMAT(/
     # ' *** FLOMOD RUN TERMINATED NORMALLY.')
2070  FORMAT(1H+,79(' ')/
     # 1H+,'FLOMOD run terminated because of ',I4,' error(s)!!!')
2080  FORMAT(1H+,79(' ')/
     # 1H+,'FLOMOD run terminated normally.')
2540  FORMAT(1H+,79(' ')/1H+,'Time step ',I3,':  Iteration ',I2,':')
C
C---- Clear the screen
Cx    CALL SETUP
      TITLE = 
     & '$Id: flomod.f,v 2.5 1995/06/07 16:19:02 rsregan Exp rsregan $'
      Title = '@(#)FESWMS - Finite Element Surface Water Model for Two-'
      Title = '@(#)FESWMS - Dimensional Flow in the Horizontal Plane'
      Title = '@(#)FESWMS - FHWA Report HW031.D, D.C Froehlich'
      Title = '@(#)FESWMS - Contact: Janice Fulford, jfulford@usgs.gov'
      Title = '@(#)FESWMS - Version: 1.10.DG1 2.5x 1995/06/07'
      IDP = 0
C
C---- Initialize variables
      ISW=0
      ITDS=0
      ITLO=1
      IER=0
      NCYL=1
C
C---- Read names of FLOMOD data files if FLOMOD.FIL exists
      CALL FLOFIL
C
C---- Open printed output file (UNIT = IO2)
      IF(EXISTS(PRNTFILE)) THEN
        OPEN(IO2,FILE=PRNTFILE,STATUS='OLD')
        CLOSE(IO2,STATUS='DELETE')
      ENDIF
      OPEN(IO2,FILE=PRNTFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     # FORM='FORMATTED')
      CALL FORMFD(IO2)
C
C---- Print date and time of start of run
C     PR1ME DATE
C      CALL DATER(DATE)
C      WRITE(IO2,2010) DATE(1:LENCHR(DATE))
C     DG DATE
      CALL IDATE(IMON,IDAY,IYEAR)
      WRITE(IO2,2010)IMON,IDAY,IYEAR
C     PR1ME TIME
C      CALL TIMER(TIME)
C      WRITE(IO2,2020) TIME(1:LENCHR(TIME))
C     DG TIME
      CALL TIME(TIMEBUF)
      WRITE(IO2,2020)TIMEBUF
C
C---- Open input data file
      IF(EXISTS(DATAFILE)) THEN
        OPEN(IO1,FILE=DATAFILE,STATUS='OLD')
      ELSE
        WRITE(IO2,2000) DATAFILE(1:LENCHR(DATAFILE))
        IER=IER+1
        GOTO 999
      ENDIF
C
C---- Input data for initial solution
      CALL INPUT1(IER)
      IF(IER.GT.0) GOTO 999
C
C---- Perform preliminary computations
      CALL PRECMP(IER)
C
C---- Print element and node data if MOD(IPRNT,4) > 1
      IF(MOD(IPRNT,4).GT.1) CALL PRELNP
C
C---- Print boundary and initial condition data if MOD(IPRNT,8) > 2
      IF(MOD(IPRNT,8).GT.2) CALL PRBDIC
C
C---- Print element assembly sequence if MOD(IPRNT,16) > 4
      IF(MOD(IPRNT,16).GT.4) CALL PRLSEQ
C
C---- Assign node point identification codes
      CALL NPIDEN
C
C---- Assign weir/culvert and ceiling option boundary codes
      CALL BCODE1(IER)
C
C---- Assign slip/no-slip boundary codes
      CALL BCODE2
C
C---- Assign equation numbers for initial solution
      CALL EQLOAD(IER)
C
C---- Compute boundary angles
      CALL BANGLE(IER)
C
C---- Error trap.  Terminate run because of errors if IER > 0.
      IF(IER.GT.0) GOTO 999
      IF(MOD(IDEBUG,2048).GT.1023) GOTO 999
C
C---- Conduct initial flow check
      IF(NCL.GT.1) CALL QCHECK
C
C---- Open temporary and output data files
      CALL OPENF
C
C---- Is this a restart/recovery run?
      IF(IDRUN.GT.0) THEN
        ITLO=NITSV+1
C------ Is this a steady-state or a time-dependent (unsteady) solution?
        IF(DELT.GT.0.) ITDS=INT(TET/DELT+0.5)
        IF(ITDS.LT.1) GOTO 10
        NCYL=ITDS
        GOTO 30
      ENDIF
C
C---- Steady-state solution
C
10    IF(NITS.LT.1) GOTO 30
C
C---- Begin steady-state solution iteration loop
      DO 20 IT=ITLO,NITS
        NITSV=IT
        IF(SCREEN) WRITE(6,2540) ITDS,NITSV
C
C------ Update boundary conditions and equation numbers
        IF(IONOFF.GT.0) THEN
          CALL LONOFF
          CALL NPIDEN
          CALL BCODE2
          CALL EQLOAD(IER)
          CALL BANGLE(IER)
        ENDIF
        IF(NSECQ.GT.0) CALL XSECQ(IER)
        IF(NXSZ.GT.0) CALL XSECZ(IER)
C
C------ Error trap.  Terminate run because of errors if IER > 0.
        IF(IER.GT.0) GOTO 999
C
C------ Compute update iteration number
        IF(IT.GT.NINITS) THEN
          ITUP=MOD(IT-NINITS,NQNITS+1)
        ELSE
          ITUP=0
        ENDIF
C
C------ Set KEY
        IF(ISAVE.GT.0 .OR. NQNITS.GT.0) THEN
          KEY=1
        ELSE
          KEY=0
        ENDIF
C
C------ Solve equations.  Call either ZFRONT or UPDATE.
        IF(ITUP.EQ.0) THEN
          CALL ZFRONT(KEY,IER)
        ELSE
          CALL UPDATE(ITUP,IER)
        ENDIF
C
C------ Error trap.  Terminate run because of errors if IER > 0.
        IF(IER.GT.0) GOTO 999
C
C------ Update solution variables
        CALL SOLCOR(IDP)
C
C------ Output solution results
        IDP=1
        IF(MOD(IT,NITPS).EQ.0) IDP=2
        IF(IT.EQ.NITS) IDP=3
        CALL OUTPUT(IDP)
20    CONTINUE
C
C---- Compute continuity norms
      IF(MOD(INORM,2).EQ.1) CALL CNORMS
C
C---- Conduct flow-checks
      IF(NCL.GT.1) CALL QCHECK
C
C---- Time-dependent (unsteady) solution
C
30    IF(NITD.LT.1) GOTO 999
      NCYC=0
      IF(DELT.GT.0.) NCYC=INT(TMAX/DELT+0.5)
      IF(NCYC.LT.NCYL) GOTO 999
      DELTS=3600.*DELT
      ALPHA=1./(THETA*DELTS)
      THETAP=(1.-THETA)/THETA
C
C---- Begin time-step loop
      DO 100 IC=NCYL,NCYC
        ITDS=IC
        TET=TET+DELT
C
C------ Compute new Betas
        DO 40 N=1,NP
          DO 39 K=1,NDF
            BETAT(K,N)=ALPHA*VEL(K,N)+THETAP*VDOT(K,N)
39        CONTINUE
40      CONTINUE
C
C------ Estimate values at next time level
        DO 60 N=1,NP
          DO 50 K=1,NDF
            VEL(K,N)=VEL(K,N)+DELTS*VDOT(K,N)
50        CONTINUE
60      CONTINUE
C
C------ Update boundary conditions and wind data
        CALL INPUT2(IER)
        IF(IER.GT.0) GOTO 999
C
C------ Begin iteration loop
        NITSV=0
        DO 90 IT=1,NITD
          NITSV=NITSV+1
          IF(SCREEN) WRITE(6,2540) ITDS,NITSV
          IF(IONOFF.GT.0) THEN
            CALL LONOFF
            CALL NPIDEN
            CALL BCODE2
            CALL EQLOAD(IER)
            CALL BANGLE(IER)
          ENDIF
          IF(NSECQ.GT.0) CALL XSECQ(IER)
          IF(NXSZ.GT.0) CALL XSECZ(IER)
C
C-------- Error trap.  Terminate run because of errors if IER > 0.
          IF(IER.GT.0) GOTO 999
C
C-------- Compute update iteration number
          IF(IT.GT.NINITD) THEN
            ITUP=MOD(IT-NINITD,NQNITD+1)
          ELSE
            ITUP=0
          ENDIF
C
C-------- Set KEY
          IF(ISAVE.GT.0 .OR. NQNITD.GT.0) THEN
            KEY=1
          ELSE
            KEY=0
          ENDIF
C
C-------- Solve equations.  Call either ZFRONT or UPDATE.
          IF(ITUP.EQ.0) THEN
            CALL ZFRONT(KEY,IER)
          ELSE
            CALL UPDATE(ITUP,IER)
          ENDIF
C
C-------- Error trap.  Terminate run because of errors if IER > 0.
          IF(IER.GT.0) GOTO 999
C
C-------- Set output control parameter
          IDP=1
          IF(MOD(IC,NITPDT).EQ.0) THEN
            IF(MOD(IT,NITPDI).EQ.0) IDP=2
            IF(IT.EQ.NITD) IDP=3
          ENDIF
C
C-------- Correct solutions
          CALL SOLCOR(IDP)
C
C-------- Output results
          CALL OUTPUT(IDP)
C
C-------- Compute current time derivatives
          DO 80 N=1,NP
            DO 70 K=1,NDF
              VDOT(K,N)=ALPHA*VEL(K,N)-BETAT(K,N)
70          CONTINUE
80        CONTINUE
90      CONTINUE
C
C------ Compute flow checks and continuity norms
        IF(MOD(INORM,4).GT.1) CALL CNORMS
        IF(NCL.GT.1) CALL QCHECK
100   CONTINUE
C
C---- Close boundary & wind data files if opened for unsteady solution
      IF(IBCIN.NE.0) CLOSE(IO95,STATUS='KEEP')
      IF(IWDIN.NE.0) CLOSE(IO96,STATUS='KEEP')
C
C---- Close input data and temporary files
999   INQUIRE(UNIT=IO1,OPENED=OPENED)
      IF(OPENED) CLOSE(IO1,STATUS='KEEP')
      INQUIRE(UNIT=IO90,OPENED=OPENED)
      IF(OPENED) THEN
        CLOSE(IO90,STATUS='DELETE')
        CLOSE(IO91,STATUS='DELETE')
        CLOSE(IO92,STATUS='DELETE')
      ENDIF
      IF(ISAVE.EQ.0) THEN
        IF(EXISTS(LOWER)) THEN
          OPEN(ILCOF,FILE=LOWER)
          CLOSE(ILCOF,STATUS='DELETE')
        ENDIF
        IF(EXISTS(UPPER)) THEN
          OPEN(IUCOF,FILE=UPPER)
          CLOSE(IUCOF,STATUS='DELETE')
        ENDIF
      ENDIF
C
C---- All done
      IF(IER.EQ.0) THEN
        WRITE(IO2,2060)
        IF(SCREEN) WRITE(6,2080)
      ELSE
        WRITE(IO2,2050) IER
        IF(SCREEN) WRITE(6,2070) IER
      ENDIF
C
C---- Print date and time of end of run
      WRITE(IO2,'(/)')
C     PR1ME DATE
C      CALL DATER(DATE)
C      WRITE(IO2,2010) DATE(1:LENCHR(DATE))
C     DG DATE
      CALL IDATE (IMON,IDAY,IYEAR)
      WRITE(IO2,2010) IMON,IDAY,IYEAR
C     PR1ME TIME
C      CALL TIMER(TIME)
C      WRITE(IO2,2020) TIME(1:LENCHR(TIME))
C     DG TIME
      CALL TIME(TIMEBUF)
      WRITE(IO2,2020) TIMEBUF
C
C---- Close printed output file and stop
      CLOSE(IO2,STATUS='KEEP')
C
C---- All done
      END
C
C
C
      SUBROUTINE FLOFIL
C
C---- FLOFIL ----------------------------------------------------------S
C
C  PURPOSE:
C    READS NAMES OF FLOMOD DATA FILES FROM THE FILE 'FLOMOD.FIL'
C
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      LOGICAL   EXISTS
      CHARACTER FILENAME*80
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(A)
C
      IF(EXISTS('FLOMOD.FIL')) THEN
        OPEN(IO1,FILE='FLOMOD.FIL',STATUS='OLD')
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') DATAFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') PRNTFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') GRIDFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') INITFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') BOUNFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') WINDFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') FLOWFILE=FILENAME
        READ(IO1,1000) FILENAME
        IF(FILENAME.NE.' ') RSRCFILE=FILENAME
        CLOSE(IO1)
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE OPENF
C
C---- OPENF -----------------------------------------------------------S
C
C  PURPOSE:
C    OPENS TEMPORARY FILES AND NEEDED OUTPUT FILES
C
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      LOGICAL   EXISTS
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      IF(EXISTS('FM90.TMP')) THEN
        OPEN(IO90,FILE='FM90.TMP')
        CLOSE(IO90,STATUS='DELETE')
      ENDIF
      OPEN(IO90,FILE='FM90.TMP',STATUS='NEW',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
      IF(EXISTS('FM91.TMP')) THEN
        OPEN(IO91,FILE='FM91.TMP')
        CLOSE(IO91,STATUS='DELETE')
      ENDIF
      OPEN(IO91,FILE='FM91.TMP',STATUS='NEW',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
      IF(EXISTS('FM92.TMP')) THEN
        OPEN(IO92,FILE='FM92.TMP')
        CLOSE(IO92,STATUS='DELETE')
      ENDIF
      OPEN(IO92,FILE='FM92.TMP',STATUS='NEW',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
      IF(IRSRC.NE.0) THEN
        IF(EXISTS(RSRCFILE)) THEN
          OPEN(IO97,FILE=RSRCFILE)
          CLOSE(IO97,STATUS='DELETE')
        ENDIF
        IF(IRSRC.LT.0) THEN
          OPEN(IO97,FILE=RSRCFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #    FORM='UNFORMATTED')
        ELSE
          OPEN(IO97,FILE=RSRCFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #    FORM='FORMATTED')
        ENDIF
      ENDIF
      IF(ISOUT.NE.0) THEN
        IF(EXISTS(FLOWFILE)) THEN
          OPEN(IO94,FILE=FLOWFILE)
          CLOSE(IO94,STATUS='DELETE')
        ENDIF
        IF(ISOUT.LT.0) THEN
          OPEN(IO94,FILE=FLOWFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #    FORM='UNFORMATTED')
        ELSE
          OPEN(IO94,FILE=FLOWFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #    FORM='FORMATTED')
        ENDIF
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE TBLOCK(VERSION,WIDE,IO2)
C
C---- TBLOCK ----------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS THE TITLE BLOCK.
C
C  INPUT ARGUMENT(S):
C    VERSION - VERSION NUMBER.
C    WIDE    - PRINT OUTPUT FORMAT CONTROL VARIABLE.
C    IO2     - OUTPUT FILE UNIT NUMBER.
C
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   IO2
      CHARACTER VERSION*8
      LOGICAL   WIDE
C
C---- OUTPUT FORMATS
2000  FORMAT(/
     # 27X,78('=')/27X,'>>>>',70X,'<<<<'/27X,'>>>>',12X,
     # 'FINITE ELEMENT SURFACE WATER MODELING SYSTEM -',
     # 12X,'<<<<'/27X,'>>>>',6X,
     # 'TWO-DIMENSIONAL FLOW IN THE HORIZONTAL PLANE *FESWMS-2DH*',
     # 7X,'<<<<'/27X,'>>>>',70X,'<<<<'/27X,'>>>>',14X,
     # 'DEPTH-AVERAGED FLOW MODULE - VERSION ',A8,11X,'<<<<'/
     # 27X,'>>>>',70X,'<<<<'/27X,'>>>>',28X,'DEVELOPED FOR',29X,'<<<<'/
     # 27X,'>>>>',70X,'<<<<'/27X,'>>>>',18X,
     # 'U.S. DEPARTMENT OF TRANSPORTATION',19X,'<<<<'/27X,'>>>>',20X,
     # 'FEDERAL HIGHWAY ADMINISTRATION',20X,'<<<<'/27X,'>>>>',13X,
     # 'OFFICE OF ENGINEERING AND HIGHWAY OPERATIONS',13X,'<<<<'/
     # 27X,'>>>>',23X,'RESEARCH AND DEVELOPMENT',23X,'<<<<'/
     # 27X,'>>>>',16X,'6300 GEORGETOWN PIKE, MCLEAN, VA 22101',
     # 16X,'<<<<'/27X,'>>>>',70X,'<<<<'/27X,'>>>>',34X,'BY',34X,'<<<<'/
     # 27X,'>>>>',70X,'<<<<'/27X,'>>>>',11X,
     # 'U.S. GEOLOGICAL SURVEY, WATER RESOURCES DIVISION',
     # 11X,'<<<<'/27X,'>>>>',70X,'<<<<'/27X,78('=')/)
2100  FORMAT(/
     # 1X,78('=')/1X,'>>>>',70X,'<<<<'/1X,'>>>>',12X,
     # 'FINITE ELEMENT SURFACE WATER MODELING SYSTEM -',
     # 12X,'<<<<'/1X,'>>>>',6X,
     # 'TWO-DIMENSIONAL FLOW IN THE HORIZONTAL PLANE *FESWMS-2DH*',
     # 7X,'<<<<'/1X,'>>>>',70X,'<<<<'/1X,'>>>>',14X,
     # 'DEPTH-AVERAGED FLOW MODULE - VERSION ',A8,11X,'<<<<'/
     # 1X,'>>>>',70X,'<<<<'/1X,'>>>>',28X,'DEVELOPED FOR',29X,'<<<<'/
     # 1X,'>>>>',70X,'<<<<'/1X,'>>>>',18X,
     # 'U.S. DEPARTMENT OF TRANSPORTATION',19X,'<<<<'/1X,'>>>>',20X,
     # 'FEDERAL HIGHWAY ADMINISTRATION',20X,'<<<<'/1X,'>>>>',13X,
     # 'OFFICE OF ENGINEERING AND HIGHWAY OPERATIONS',13X,'<<<<'/
     # 1X,'>>>>',23X,'RESEARCH AND DEVELOPMENT',23X,'<<<<'/
     # 1X,'>>>>',16X,'6300 GEORGETOWN PIKE, MCLEAN, VA 22101',
     # 16X,'<<<<'/1X,'>>>>',70X,'<<<<'/1X,'>>>>',34X,'BY',34X,'<<<<'/
     # 1X,'>>>>',70X,'<<<<'/1X,'>>>>',11X,
     # 'U.S. GEOLOGICAL SURVEY, WATER RESOURCES DIVISION',
     # 11X,'<<<<'/1X,'>>>>',70X,'<<<<'/1X,78('=')/)
C
C---- PRINT TITLE BLOCK
      IF(WIDE) THEN
        WRITE(IO2,2000) VERSION
      ELSE
        WRITE(IO2,2100) VERSION
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE BANGLE(IER)
C
C---- BANGLE ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES NETWORK BOUNDARY ANGLES.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C--------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,K1,K2,K3,L,N,NCN,NR,N2
      REAL      A1,A2,DXDT,DYDT,EPS,XR,YR,TA,TB
      LOGICAL   SOLID,OPEN
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Data
      DATA EPS/0.001/
C
C---- Output formats
2000  FORMAT(
     # ' ***ERROR #210 - INDETERMINANT BOUNDARY ANGLE AT NODE ',I5,'.')
C
C---- Clear arrays
      DO 15 N=1,NP
        ALFA(1,N)=0.
        ALFA(2,N)=0.
15    CONTINUE
C
C---- Loop over elements
      DO 60 L=1,NE
        IF(LMAT(L).LT.1) GOTO 60
        CALL LNODES(L,NCN)
        IF(NCN.GT.8) NCN=8
        DO 50 K2=2,NCN,2
          N2=IABS(NODE(K2))
C-------- If NPID(N2) = 1, then N2 is a boundary node
          IF(NPID(N2).NE.1) GOTO 50
          SOLID=.FALSE.
          OPEN=.TRUE.
          IF(MOD(NFIX(N2)/1000,10).EQ.5) THEN
            SOLID=.TRUE.
            OPEN=.FALSE.
          ENDIF
          K1=K2-1
          K3=MOD(K2+1,NCN)
          KNODE(1)=K1
          KNODE(2)=K2
          KNODE(3)=K3
C
C-------- COMPUTE LOCAL COORDINATES
          NR=IABS(NODE(KNODE(1)))
          XR=CORD(NR,1)
          YR=CORD(NR,2)
          DO 10 K=2,3
            N=IABS(NODE(KNODE(K)))
            XL(K)=CORD(N,1)-XR
            YL(K)=CORD(N,2)-YR
10        CONTINUE
C
C-------- Loop around integration points
          DO 40 I=1,4
            DYDT=0.
            DXDT=0.
            DO 20 K=2,3
              DYDT=DYDT+DNAL(K,I)*YL(K)
              DXDT=DXDT+DNAL(K,I)*XL(K)
20          CONTINUE
            TA=WAITL(I)*DYDT
            TB=WAITL(I)*DXDT
            DO 30 K=1,3
              N=IABS(NODE(KNODE(K)))
C------------ Do not add open boundary contribution at an interface node
              IF(OPEN .AND. MOD(NFIX(N)/1000,10).EQ.5) GOTO 30
              ALFA(1,N)=ALFA(1,N)+XNL(K,I)*TA
              ALFA(2,N)=ALFA(2,N)+XNL(K,I)*TB
30          CONTINUE
40        CONTINUE
50      CONTINUE
60    CONTINUE
C
C---- Compute boundary angles in radians
      DO 70 N=1,NP
        IF(NPID(N).EQ.1) THEN
          A1=ALFA(1,N)
          A2=ALFA(2,N)
C-------- Check for indeterminant boundary angle
          IF(ABS(A1).GT.0. .OR. ABS(A2).GT.0.) THEN
            ALFA(1,N)=ATAN2(A1,A2)
          ELSE
            IER=IER+1
            WRITE(IO2,2000) N
          ENDIF
        ENDIF
70    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE BCODE1(IER)
C
C---- BCODE1 ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS WEIR/CULVERT, CEILING-OPTION, AND CROSS-SECTION DISCHARGE
C    BOUNDARY CODES AND APPLIES ESSENTIAL BOUNDARY CONDITIONS.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C--------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   ITEMP,K,L,M,N,NCN,NFIX3,N1,N2
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2010  FORMAT(' ***ERROR #211 - WEIR NODE ',I5,
     # ' IS NOT A BOUNDARY NODE.')
2020  FORMAT(' ***ERROR #212 - CULVERT NODE ',I5,
     # ' IS NOT A BOUNDARY NODE.')
2030  FORMAT(' ***ERROR #213 - TOTAL-FLOW CROSS-SECTION NODE ',I5,
     # ' IS NOT A BOUNDARY NODE.')
C
C---- Assign weir/culvert flow boundary condition codes
      IF(NWER.GT.0) THEN
        DO 10 K=1,NWER
          N1=IABS(NOPW(K,1))
          NFIX(N1)=10000*(NFIX(N1)/10000)+5000+MOD(NFIX(N1),1000)
          IF(NPID(N1).NE.1) THEN
            IER=IER+1
            WRITE(IO2,2010) N1
          ENDIF
          N2=IABS(NOPW(K,2))
          IF(N2.GT.0) THEN
            NFIX(N2)=10000*(NFIX(N2)/10000)+5000+MOD(NFIX(N2),1000)
            IF(NPID(N2).NE.1) THEN
              IER=IER+1
              WRITE(IO2,2010) N2
            ENDIF
          ENDIF
10      CONTINUE
      ENDIF
      IF(NCUL.GT.0) THEN
        DO 20 K=1,NCUL
          N1=IABS(NOPC(K,1))
          NFIX(N1)=10000*(NFIX(N1)/10000)+5000+MOD(NFIX(N1),1000)
          IF(NPID(N1).NE.1) THEN
            IER=IER+1
            WRITE(IO2,2020) N1
          ENDIF
          N2=IABS(NOPC(K,2))
          IF(N2.GT.0) THEN
            NFIX(N2)=10000*(NFIX(N2)/10000)+5000+MOD(NFIX(N2),1000)
            IF(NPID(N2).NE.1) THEN
              IER=IER+1
              WRITE(IO2,2020) N2
            ENDIF
          ENDIF
20      CONTINUE
      ENDIF
C
C---- Assign ceiling-option boundary condition codes
      DO 60 L=1,NE
        IF(IABS(LMAT(L)).GT.1000) THEN
          CALL LNODES(L,NCN)
          DO 50 K=1,NCN
            N=IABS(NODE(K))
            NFIX3=MOD(NFIX(N)/100,10)
            IF(NFIX3.EQ.0) NFIX(N)=NFIX(N)+700
50        CONTINUE
        ENDIF
60    CONTINUE
      DO 80 L=1,NE
        M=IABS(LMAT(L))
        IF(M.GT.0 .AND. M.LT.1001) THEN
          CALL LNODES(L,NCN)
          DO 70 K=1,NCN
            N=IABS(NODE(K))
            NFIX3=MOD(NFIX(N)/100,10)
            IF(NFIX3.EQ.7) NFIX(N)=NFIX(N)+100
70        CONTINUE
        ENDIF
80    CONTINUE
C
C---- Assign essential boundary conditions
      DO 100 N=1,NP
        IF(NFIX(N).NE.0) THEN
          DO 90 K=1,NDF
            ITEMP=NFIX(N)/10**(5-K)
            IF(MOD(ITEMP,10).EQ.1) VEL(K,N)=SPEC(N,K)
90        CONTINUE
        ENDIF
100   CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE BCODE2
C
C---- BCODE2 ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS SLIP/NO-SLIP BOUNDARY CODES TO ALL SOLID BOUNADRY NODES
C    TO WHICH CODES HAVE NOT YET BEEN ASSIGNED.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C--------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   K,L,N,NCN,NFX,N1,N2,N3
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- ASSIGN SLIP/NO-SLIP BOUNDARY CONDITION CODES
      IF(ISLIP.EQ.0) NFX=05000
      IF(ISLIP.EQ.1) NFX=11000
      DO 20 L=1,NE
        IF(LMAT(L).LT.1) GOTO 20
        CALL LNODES(L,NCN)
        IF(NCN.GT.8) NCN=8
        DO 10 K=2,NCN,2
          N2=IABS(NODE(K))
C-------- CHECK FOR BOUNDARY SIDE
          IF(NPID(N2).EQ.1) THEN
C-------- ASSIGN CODES
            N1=IABS(NODE(K-1))
            N3=MOD(K+1,NCN)
            N3=IABS(NODE(N3))
            IF(NFIX(N1).EQ.0) NFIX(N1)=NFX
            IF(NFIX(N2).EQ.0) NFIX(N2)=NFX
            IF(NFIX(N3).EQ.0) NFIX(N3)=NFX
          ENDIF
10      CONTINUE
20    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE BINTEG(L,NCN,KEY)
C
C---- BINTEG ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES BOUNDARY INTEGRALS AND INSERTS TERMS IN THE ELEMENT
C    COEFFICIENT MATRIX AND LOAD VECTOR.  IT ALSO COMPUTES BOUNDARY
C    FLOW CONSTRAINT EQUATION COEFFICIENTS USED IN BSPECS.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER.
C    NCN - NUMBER OF ELEMENT NODES.
C    KEY - EQUATION SOLUTION KEY (SEE ZFRONT FOR EXPLANATION).
C
C  SUBPROGRAM(S) CALLED:
C    FRCFCT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,NCN,KEY
C
C---- Declarations (local)
      INTEGER   I,J,JJ,K,KK,K1,K2,K3,M,N,NFIX1,NFIX2,NFIX3,N1,N2,N3,
     #          IU,IUU,IUV,IUH,IV,IVU,IVV,IVH,ISW
      REAL      U,V,H,P,ZB,ZC,DCX,DCY,DXDT,DYDT,DSDT,WEIGHT,BETA,
     #          FFACT,DFDH,FUN,FVN,TU,TUU,TUV,TUH,TV,TVU,TVV,TVH,THM,
     #          DFUUN,DFUVN,DFUHN,DFVUN,DFVVN,DFVHN
      LOGICAL   SOLID,OPEN,PRESS
C
C---- Common blocks
      COMMON/CISW/ISW
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2077  FORMAT(' ***ERROR #??? - DEPTH <= 0 IN ELEMENT ',I5,':'/
     # '     K  NODE     DEPTH')
2078  FORMAT(2I6,F10.3)
C
C---- Loop over element sides
      DO 130 K2=2,NCN,2
        N2=IABS(NODE(K2))
        K1=K2-1
        IF(NCN.EQ.6) K3=MOD(K2+1,6)
        IF(NCN.GT.6) K3=MOD(K2+1,8)
        N1=IABS(NODE(K1))
        N3=IABS(NODE(K3))
        KNODE(1)=K1
        KNODE(2)=K2
        KNODE(3)=K3
C
C------ Determine if element side forms a solid or an open boundary
        IF(NPID(N2).EQ.1) THEN
          SOLID=.FALSE.
          OPEN=.TRUE.
        ELSE
          SOLID=.FALSE.
          OPEN=.FALSE.
        ENDIF
        IF(MOD(NFIX(N2)/1000,10).EQ.5) THEN
          SOLID=.TRUE.
          OPEN=.FALSE.
        ENDIF
C
C------ Insert natural depth specs
        DO 10 K=1,3
          KK=KNODE(K)
          N=IABS(NODE(KK))
          NFIX3=MOD(NFIX(N)/100,10)
          IF(NFIX3.EQ.2) PL(KK)=SPEC(N,3)
          IF(LMAT(L).GT.1000.AND.PL(KK).GT.CL(KK)-GL(KK)) THEN
            HL(KK)=CL(KK)-GL(KK)
          ELSE
            HL(KK)=PL(KK)
          ENDIF
10      CONTINUE
C
C------ Lopp around integration points
        DO 120 I=1,4
          DYDT=0.0
          DXDT=0.0
          DO 20 K=1,3
            DYDT=DYDT+DNAL(K,I)*YL(KNODE(K))
            DXDT=DXDT-DNAL(K,I)*XL(KNODE(K))
20        CONTINUE
          DSDT=SQRT(DXDT*DXDT+DYDT*DYDT)
          WEIGHT=WAITL(I)*DSDT
          DCY=DYDT/DSDT
          DCX=DXDT/DSDT
          U=0.0
          V=0.0
          H=0.0
          P=0.
          ZB=0.0
          ZC=0.
          DO 30 K=1,3
            U=U+XNL(K,I)*UL(KNODE(K))
            V=V+XNL(K,I)*VL(KNODE(K))
30        CONTINUE
          DO 40 K=1,3,2
            H=H+XML(K,I)*HL(KNODE(K))
            P=P+XML(K,I)*PL(KNODE(K))
            ZB=ZB+XML(K,I)*GL(KNODE(K))
            ZC=ZC+XML(K,I)*CL(KNODE(K))
40        CONTINUE
C
C-------- Check for depth <= 0
          IF(H.LT.ZIP) THEN
            IF (ISW.NE.L) THEN
            CALL LNODES(L,NCN)
            WRITE(IO2,2077) L
            ISW=L
            DO 45 K=1,NCN
              N=IABS(NODE(K))
              WRITE(IO2,2078) K,N,VEL(3,N)
45          CONTINUE
          ENDIF
          ENDIF
C
C-------- CHECK FOR PRESSURE FLOW
          PRESS=.FALSE.
          IF(LMAT(L).GT.1000.AND.P.GT.ZC-ZB) PRESS=.TRUE.
C
C-------- COMPUTE COEFFICIENT OF BOTTOM FRICTION
          M=MOD(LMAT(L),100)
          CALL FRCFCT(H,M,FFACT,DFDH)
C
C-------- COMPUTE MOMENTUM CORRECTION COEFFICIENT
          BETA=BETA0+CBETA*FFACT
C
C-------- MOMENTUM EQUATION RESIDUALS
          FUN=H*(BETA*U*U+GRAV*(P-0.5*H))*DCY+H*BETA*U*V*DCX
          FVN=H*BETA*U*V*DCY+H*(BETA*V*V+GRAV*(P-0.5*H))*DCX
          DO 50 K=1,3
            IU=1+NDF*(KNODE(K)-1)
            IV=IU+1
            F(IU)=F(IU)-WEIGHT*XNL(K,I)*FUN
            F(IV)=F(IV)-WEIGHT*XNL(K,I)*FVN
50        CONTINUE
C
C-------- MOMENTUM EQUATION DERIVATIVES  ( IF KEY < 3 )
          IF(KEY.LT.3) THEN
            TUU=BETA*H*(2.*U*DYDT+V*DXDT)*WAITL(I)
            TUV=BETA*H*U*DXDT*WAITL(I)
            TVU=BETA*H*V*DYDT*WAITL(I)
            TVV=BETA*H*(U*DYDT+2.*V*DXDT)*WAITL(I)
            IF(PRESS) THEN
              TUH=GRAV*H*DYDT*WAITL(I)
              TVH=GRAV*H*DXDT*WAITL(I)
            ELSE
              TUH=((BETA*U*U+GRAV*H)*DYDT+BETA*U*V*DXDT)*WAITL(I)
              TVH=(BETA*V*U*DYDT+(BETA*V*V+GRAV*H)*DXDT)*WAITL(I)
            ENDIF
            DO 70 K=1,3
              IUU=1+NDF*(KNODE(K)-1)
              IUV=IUU+1
              IUH=IUU+2
              N=IABS(NODE(KNODE(K)))
              NFIX1=NFIX(N)/10000
              NFIX2=MOD(NFIX(N)/1000,10)
              NFIX3=MOD(NFIX(N)/100,10)
              DFUUN=XNL(K,I)*TUU
              DFUVN=XNL(K,I)*TUV
              DFUHN=XML(K,I)*TUH
              DFVUN=XNL(K,I)*TVU
              DFVVN=XNL(K,I)*TVV
              DFVHN=XML(K,I)*TVH
              DO 60 J=1,3
                IU=1+NDF*(KNODE(J)-1)
                IV=IU+1
C
C-------------- WITH RESPECT TO U
                ECOEFM(IU,IUU)=ECOEFM(IU,IUU)+XNL(J,I)*DFUUN
                ECOEFM(IV,IUU)=ECOEFM(IV,IUU)+XNL(J,I)*DFVUN
C
C-------------- WITH RESPECT TO V
                ECOEFM(IU,IUV)=ECOEFM(IU,IUV)+XNL(J,I)*DFUVN
                ECOEFM(IV,IUV)=ECOEFM(IV,IUV)+XNL(J,I)*DFVVN
C
C-------------- WITH RESPECT TO H (IF NFIX3 = 0)
                IF(NFIX3.EQ.0) THEN
                  ECOEFM(IU,IUH)=ECOEFM(IU,IUH)+XNL(J,I)*DFUHN
                  ECOEFM(IV,IUH)=ECOEFM(IV,IUH)+XNL(J,I)*DFVHN
                ENDIF
60            CONTINUE
70          CONTINUE
          ENDIF
C
C-------- COMPUTE SOLID BOUNDARY FLOW CONSTRAINT EQUATION COEFFICIENTS
          IF(SOLID) THEN
            TU=WAITL(I)*H*DYDT
            TV=WAITL(I)*H*DXDT
            DO 90 K=1,3
              N=IABS(NODE(KNODE(K)))
              KK=KNODE(K)
              UCOFS(KK)=UCOFS(KK)+XNL(K,I)*TU
              VCOFS(KK)=VCOFS(KK)+XNL(K,I)*TV
              THM=WAITL(I)*XNL(K,I)*(VEL(1,N)*DYDT+VEL(2,N)*DXDT)
              DO 80 J=1,3,2
                JJ=KNODE(J)
                HCOFS(JJ,KK)=HCOFS(JJ,KK)+XML(J,I)*THM
80            CONTINUE
90          CONTINUE
          ENDIF
C
C-------- COMPUTE OPEN BOUNDARY FLOW CONSTRAINT EQUATION COEFFICIENTS
          IF(OPEN) THEN
            TU=WAITL(I)*H*DYDT
            TV=WAITL(I)*H*DXDT
            DO 110 K=1,3
              N=IABS(NODE(KNODE(K)))
              KK=KNODE(K)
              UCOFO(KK)=UCOFO(KK)+XNL(K,I)*TU
              VCOFO(KK)=VCOFO(KK)+XNL(K,I)*TV
              THM=WAITL(I)*XNL(K,I)*(VEL(1,N)*DYDT+VEL(2,N)*DXDT)
              DO 100 J=1,3,2
                JJ=KNODE(J)
                HCOFO(JJ,KK)=HCOFO(JJ,KK)+XML(J,I)*THM
100           CONTINUE
110         CONTINUE
          ENDIF
120     CONTINUE
130   CONTINUE
C     WRITE(IO2,'(/I5,5X,9I5)') L,(NODE(K),K=1,9)
C     WRITE(IO2,'(9F10.2)') (UCOFS(J),J=1,9)
C     WRITE(IO2,'(9F10.2)') (VCOFS(J),J=1,9)
C     WRITE(IO2,'(9F10.2)') (UCOFO(J),J=1,9)
C     WRITE(IO2,'(9F10.2)') (VCOFO(J),J=1,9)
      RETURN
      END
C
C
C
      BLOCK DATA
C
C---- BLOCK DATA -----------------------------------------------------S
C
C  PURPOSE:
C    INITIALIZES VARIABLES IN LABELED COMMON BLOCK. THIS SUBROUTINE
C    IS NOT CALLED, ITS PRESENCE SUFFICES TO PROVIDE INITIAL DATA
C    VALUES FOR REFERENECS IN THE MAIN PROGRAM AND SUBPROGRAMS TO
C    LABELED (NAMED) COMMON BLOCKS.
C
C---------------------------------------------------------------------F
C
C---- Declarations
      INTEGER
     # IDEBUG,IDRUN,IPRNT,IUNIT,IWIND,IFRIC,ISLIP,IXINT,
     # IGRID,INITC,ISOUT,IRSRC,IBCIN,IWDIN,ISVIS,ISAVE,
     # NITS,NINITS,NQNITS,NANITS,NITP,NITPS,
     # NITD,NINITD,NQNITD,NANITD,NITPD,NITPDT,NITPDI,
     # ITDS,NITSV,NBUFF,INORM,IONOFF,
     # NP,NE,NWER,NCUL,NCL,NSECQ,NXSZ,NMAT,NSZB,NPVMIN,NUPV,
     # MEMORY,INX,
     # IO1,IO2,IO90,IO91,IO92,IO93,IO94,IO95,IO96,IO97,
     # IO98,IO99,ILCOF,IUCOF
      REAL
     # TSTRT,TMAX,DELT,THETA,
     # WSEL,OMEGA,ROWAT,BETA0,CBETA,CFLAG,DEPTOL,RELAX,
     # WVEL,WDIR,ROAIR,CSURF1,CSURF2,WVMIN,
     # VOID,VDX,GRAV,ZIP,RTOD,DTOR,
     # TET,TSNEW,ALPHA,XIN,SR,SF
      REAL
     # XNL,DNAL,WAITL,XML,
     # XNR,DNAR,DNBR,WAITR,XMR,DMAR,DMBR,
     # XNRH,DNARH,DNBRH,WAITRH,XMRH,DMARH,DMBRH,
     # XNQ,DNAQ,DNBQ,
     # XNQH,DNAQH,DNBQH,
     # XNT,DNAT,DNBT,WAITT,XMT,DMAT,DMBT,
     # XNTH,DNATH,DNBTH,WAITTH,XMTH,DMATH,DMBTH,
     # XNRH1,XNRH2,DNARH1,DNARH2,
     # DNBRH1,DNBRH2,XNQH1,XNQH2,
     # DNAQH1,DNAQH2,DNBQH1,DNBQH2
      CHARACTER*63 DATAFILE,PRNTFILE,GRIDFILE,FLOWFILE,BOUNFILE,
     # INITFILE,RSRCFILE,WINDFILE,LOWER,UPPER
      CHARACTER TITLE*80,FLAG*1,ID*4,FORM*11,VERSION*8
C
C---- Common blocks
      COMMON /BLKAC/
     # DATAFILE,PRNTFILE,GRIDFILE,FLOWFILE,BOUNFILE,
     # INITFILE,RSRCFILE,WINDFILE,LOWER,UPPER,
     # TITLE,FLAG(5),ID(20),FORM
C
      COMMON /BLKAI/
     # IDEBUG,IDRUN,IPRNT,IUNIT,IWIND,IFRIC,ISLIP,IXINT,
     # IGRID,INITC,ISOUT,IRSRC,IBCIN,IWDIN,ISVIS,ISAVE,
     # NITS,NINITS,NQNITS,NANITS,NITP,NITPS,
     # NITD,NINITD,NQNITD,NANITD,NITPD,NITPDT,NITPDI,
     # ITDS,NITSV,NBUFF,INORM,IONOFF,
     # NP,NE,NWER,NCUL,NCL,NSECQ,NXSZ,NMAT,NSZB,NPVMIN,NUPV,
     # MEMORY,INX(16),
     # IO1,IO2,IO90,IO91,IO92,IO93,IO94,IO95,IO96,IO97,
     # IO98,IO99,ILCOF,IUCOF
C
      COMMON /BLKAR/
     # TSTRT,TMAX,DELT,THETA,
     # WSEL,OMEGA,ROWAT,BETA0,CBETA,CFLAG,DEPTOL,RELAX,
     # WVEL,WDIR,ROAIR,CSURF1,CSURF2,WVMIN,
     # VOID,VDX,GRAV,ZIP,RTOD,DTOR,VERSION,
     # TET,TSNEW,ALPHA,XIN(16),SR(12),SF(12)
C
      COMMON /BLKC/
     # XNL(3,4),DNAL(3,4),WAITL(4),XML(3,4)
C
      COMMON /BLKR/
     # XNR(8,9),DNAR(8,9),DNBR(8,9),WAITR(9),
     # XMR(9,9),DMAR(9,9),DMBR(9,9)
C
      COMMON /BLKRH/
     # XNRH(8,16),DNARH(8,16),DNBRH(8,16),WAITRH(16),
     # XMRH(8,16),DMARH(8,16),DMBRH(8,16)
C
      COMMON /BLKQ/
     # XNQ(9,9),DNAQ(9,9),DNBQ(9,9)
C
      COMMON /BLKQH/
     # XNQH(9,16),DNAQH(9,16),DNBQH(9,16)
C
      COMMON /BLKT/
     # XNT(6,7),DNAT(6,7),DNBT(6,7),WAITT(7),
     # XMT(6,7),DMAT(6,7),DMBT(6,7)
C
      COMMON /BLKTH/
     # XNTH(6,16),DNATH(6,16),DNBTH(6,16),WAITTH(16),
     # XMTH(6,16),DMATH(6,16),DMBTH(6,16)
C
      DIMENSION
     # XNRH1(8,8),XNRH2(8,8),DNARH1(8,8),DNARH2(8,8),
     # DNBRH1(8,8),DNBRH2(8,8),XNQH1(9,8),XNQH2(9,8),
     # DNAQH1(9,8),DNAQH2(9,8),DNBQH1(9,8),DNBQH2(9,8)
C
      EQUIVALENCE
     # (XNRH1(1,1),XNRH(1,1)),(XNRH2(1,1),XNRH(1,9)),
     # (DNARH1(1,1),DNARH(1,1)),(DNARH2(1,1),DNARH(1,9)),
     # (DNBRH1(1,1),DNBRH(1,1)),(DNBRH2(1,1),DNBRH(1,9)),
     # (XNQH1(1,1),XNQH(1,1)),(XNQH2(1,1),XNQH(1,9)),
     # (DNAQH1(1,1),DNAQH(1,1)),(DNAQH2(1,1),DNAQH(1,9)),
     # (DNBQH1(1,1),DNBQH(1,1)),(DNBQH2(1,1),DNBQH(1,9))
C
C---- DATA
      DATA VERSION/'1.10.DG1'/
      DATA
     # IO1,IO2,IO90,IO91,IO92,IO93,IO94,IO95,IO96,IO97,IO98,IO99
     #/  5,  8,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99/
      DATA
     # DATAFILE/'FLOMOD.DAT'/,PRNTFILE/'FLOMOD.PRT'/
     # GRIDFILE/'GRID.DAT'/,FLOWFILE/'FLOW.DAT'/,
     # BOUNFILE/'BOUN.DAT'/,RSRCFILE/'RSRC.DAT'/,
     # INITFILE/'INIT.DAT'/,WINDFILE/'WIND.DAT'/,
     # LOWER/'LOWER.DAT'/,UPPER/'UPPER.DAT'/
      DATA
     # VOID/-1.E+35/,VDX/-1.E+30/,DTOR/.01745329/,RTOD/57.295780/,
     # GRAV/32.174/,ZIP/1.0E-08/
      DATA ID/
     # 'SWMS','ELEM','NODE','PROP','INIT','BOUN','WIND','FLUX','WEIR',
     # 'CULV','QSEC','ZSEC','TIME','LAST','WIER','    ','    ','    ',
     # '    ','    '/
      DATA SR/0.75,0.80,0.84,0.86,0.88,0.90,0.92,0.94,0.96,0.98,0.99,1./
      DATA SF/1.00,.995,.987,.975,.960,.930,.885,.820,.710,.575,.450,0./
C
C---- ONE-DIMENSIONAL SHAPE FUNCTIONS
      DATA WAITL/0.17392743,2*0.32607258,0.17392743/
      DATA XML/
     #  0.93056816, 0., 0.06943184,
     #  0.66999052, 0., 0.33000948,
     #  0.33000948, 0., 0.66999052,
     #  0.06943184, 0., 0.93056816/
      DATA XNL/
     #  0.80134603, 0.25844425, -0.05979028,
     #  0.22778408, 0.88441289, -0.11219697,
     # -0.11219697, 0.88441289,  0.22778408,
     # -0.05979028, 0.25844425,  0.80134603/
      DATA DNAL/
     # -2.72227262, 3.44454525, -0.72227262,
     # -1.67996209, 1.35992418,  0.32003791,
     # -0.32003791,-1.35992418,  1.67996209,
     #  0.72227262,-3.44454525,  2.72227262/
C
C---- TRIANGULAR ELEMENT WEIGHTS AND SHAPE FUNCTIONS
      DATA WAITT/0.1125,3*0.066197075,3*0.06296959/
      DATA XMT/
     # 0.33333333,0., 0.33333333,0., 0.33333333,0.,
     # 0.05971587,0., 0.47014206,0., 0.47014206,0.,
     # 0.47014206,0., 0.05971587,0., 0.47014206,0.,
     # 0.47014206,0., 0.47014206,0., 0.05971587,0.,
     # 0.79742699,0., 0.10128651,0., 0.10128651,0.,
     # 0.10128651,0., 0.79742699,0., 0.10128651,0.,
     # 0.10128651,0., 0.10128651,0., 0.79742699,0./
      DATA DMAT/
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0.,
     # 1.,0.,0.,0.,-1.,0./
      DATA DMBT/
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0.,
     # 0.,0.,1.,0.,-1.,0./
      DATA XNT/
     #-.11111111, .44444444,-.11111111, .44444444,-.11111111, .44444444,
     #-.05258390, .11229977,-.02807495, .88413422,-.02807495, .11229977,
     #-.02807495, .11229977,-.05258390, .11229977,-.02807495, .88413422,
     #-.02807495, .88413422,-.02807495, .11229977,-.05258390, .11229977,
     # .47435262, .32307439,-.08076860, .04103583,-.08076860, .32307439,
     #-.08076860, .32307439, .47435262, .32307439,-.08076860, .04103583,
     #-.08076860, .04103583,-.08076860, .32307439, .47435262, .32307439/
      DATA DNAT/
     # 0.33333333, 1.33333333, 0.,-1.33333333,-0.33333333, 0.00000000,
     #-0.76113653, 1.88056824, 0.,-1.88056824,-0.88056824, 1.64170477,
     # 0.88056824, 0.23886348, 0.,-0.23886348,-0.88056824, 0.00000000,
     # 0.88056824, 1.88056824, 0.,-1.88056824, 0.76113653,-1.64170477,
     # 2.18970796, 0.40514604, 0.,-0.40514604, 0.59485397,-2.78456193,
     #-0.59485397, 3.18970796, 0.,-3.18970796, 0.59485397, 0.00000000,
     #-0.59485397, 0.40514604, 0.,-0.40514604,-2.18970796, 2.78456193/
      DATA DNBT/
     # 0., 1.33333333, .33333333, .0000000,-.33333333,-1.33333333,
     # 0., 0.23886348, .88056824, .00000000,-.88056824,-.23886348,
     # 0., 1.88056824,-.76113653,1.64170477,-.88056824,-1.88056824,
     # 0., 1.88056824,.88056824,-1.64170477,.76113653,-1.88056824,
     # 0., 3.18970796,-.59485397, .00000000,.59485397,-3.18970796,
     # 0., 0.40514604,2.18970796,-2.78456193, .59485397,-.40514604,
     # 0., 0.40514604,-.59485397,2.78456193,-2.18970796,-.40514604/
      DATA WAITTH/
     # 0.010846452,2*0.020334519,0.010846452,
     # 0.045168099,2*0.084679449,0.045168099,
     # 0.070776136,2*0.13268843,0.070776136,
     # 0.047136736,2*0.088370177,0.047136736/
      DATA XMTH/
     # .8602401, 0., .0097038, 0., .1300561, 0.,
     # .8602401, 0., .0461221, 0., .0936378, 0.,
     # .8602401, 0., .0936378, 0., .0461221, 0.,
     # .8602401, 0., .1300561, 0., .0097038, 0.,
     # .5835904, 0., .0289121, 0., .3874975, 0.,
     # .5835904, 0., .1374191, 0., .2789905, 0.,
     # .5835904, 0., .2789905, 0., .1374191, 0.,
     # .5835904, 0., .3874975, 0., .0289121, 0.,
     # .2768430, 0., .0502101, 0., .6729469, 0.,
     # .2768430, 0., .2386487, 0., .4845083, 0.,
     # .2768430, 0., .4845083, 0., .2386487, 0.,
     # .2768430, 0., .6729469, 0., .0502101, 0.,
     # .0571042, 0., .0654670, 0., .8774288, 0.,
     # .0571042, 0., .3111646, 0., .6317312, 0.,
     # .0571042, 0., .6317312, 0., .3111646, 0.,
     # .0571042, 0., .8774288, 0., .0654670, 0./
      DATA DMATH/
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0.,
     #-1., 0., .5, 0., .5, 0./
      DATA DMBTH/
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0.,
     # 0., 0., -.5, 0., .5, 0./
      DATA XNTH/
     #  .6197861,  .0333903, -.0095155,  .0050481, -.0962269,  .4475178,
     #  .6197861,  .1587043, -.0418676,  .0172751, -.0761017,  .3222039,
     #  .6197861,  .3222039, -.0761017,  .0172751, -.0418676,  .1587043,
     #  .6197861,  .4475178, -.0962269,  .0050481, -.0095155,  .0333903,
     #  .0975652,  .0674913, -.0272403,  .0448134, -.0871889,  .9045593,
     #  .0975652,  .3207859, -.0996511,  .1533545, -.1233191,  .6512647,
     #  .0975652,  .6512647, -.1233191,  .1533545, -.0996511,  .3207859,
     #  .0975652,  .9045593, -.0871889,  .0448134, -.0272403,  .0674913,
     # -.1235589,  .0556013, -.0451680,  .1351550,  .2327681,  .7452025,
     # -.1235589,  .2642729, -.1247423,  .4625091, -.0150117,  .5365310,
     # -.1235589,  .5365310, -.0150117,  .4625091, -.1247423,  .2642729,
     # -.1235589,  .7452025,  .2327681,  .1351550, -.0451680,  .0556013,
     # -.0505824,  .0149538, -.0568951,  .2297705,  .6623338,  .2004195,
     # -.0505824,  .0710752, -.1175178,  .7862895,  .1664375,  .1442980,
     # -.0505824,  .1442980,  .1664375,  .7862895, -.1175178,  .0710752,
     # -.0505824,  .2004195,  .6623338,  .2297705, -.0568951,  .0149538/
      DATA DNATH/
     #-2.4409606, 1.6816652, -.4805924,  .2795197, -.2398878, 1.2002560,
     #-2.4409606, 1.5359920, -.4077558,  .2795197, -.3127244, 1.3459292,
     #-2.4409606, 1.3459292, -.3127244,  .2795197, -.4077558, 1.5359920,
     #-2.4409606, 1.2002560, -.2398878,  .2795197, -.4805924, 1.6816652,
     #-1.3343617, 1.0515325, -.4421758,  .8328191,  .2749950, -.3828091,
     #-1.3343617,  .6175044, -.2251618,  .8328191,  .0579809,  .0512190,
     #-1.3343617,  .0512190,  .0579809,  .8328191, -.2251618,  .6175044,
     #-1.3343617, -.3828091,  .2749950,  .8328191, -.4421758, 1.0515325,
     # -.1073720,  .3528455, -.3995797, 1.4463140,  .8458937,-2.1381014,
     # -.1073720, -.4009086, -.0227027, 1.4463140,  .4690167,-1.3843473,
     # -.1073720,-1.3843473,  .4690167, 1.4463140, -.0227027, -.4009086,
     # -.1073720,-2.1381014,  .8458937, 1.4463140, -.3995797,  .3528455,
     #  .7715832, -.1476596, -.3690660, 1.8857916, 1.2548576,-3.3955068,
     #  .7715832,-1.1304498,  .1223291, 1.8857916,  .7634625,-2.4127166,
     #  .7715832,-2.4127166,  .7634625, 1.8857916,  .1223291,-1.1304498,
     #  .7715832,-3.3955068, 1.2548576, 1.8857916, -.3690660, -.1476596/
      DATA DNBTH/
     # 0.,-1.7204803,  .4805924, -.2407046, -.2398878, 1.7204803,
     # 0.,-1.7204803,  .4077558, -.0950314, -.3127244, 1.7204803,
     # 0.,-1.7204803,  .3127244,  .0950314, -.4077558, 1.7204803,
     # 0.,-1.7204803,  .2398878,  .2407046, -.4805924, 1.7204803,
     # 0.,-1.1671809,  .4421758, -.7171708,  .2749950, 1.1671809,
     # 0.,-1.1671809,  .2251618, -.2831427,  .0579809, 1.1671809,
     # 0.,-1.1671809, -.0579809,  .2831427, -.2251618, 1.1671809,
     # 0.,-1.1671809, -.2749950,  .7171708, -.4421758, 1.1671809,
     # 0., -.5536860,  .3995797,-1.2454735,  .8458937,  .5536860,
     # 0., -.5536860,  .0227027, -.4917193,  .4690167,  .5536860,
     # 0., -.5536860, -.4690167,  .4917193, -.0227027,  .5536860,
     # 0., -.5536860, -.8458937, 1.2454735, -.3995797,  .5536860,
     # 0., -.1142084,  .3690660,-1.6239236, 1.2548576,  .1142084,
     # 0., -.1142084, -.1223291, -.6411334,  .7634625,  .1142084,
     # 0., -.1142084, -.7634625,  .6411334,  .1223291,  .1142084,
     # 0., -.1142084,-1.2548576, 1.6239236, -.3690660,  .1142084/
C
C---- SERENDIPITY QUADRILATERAL ELEMENT WEIGHTS AND SHAPE FUNCTIONS
      DATA WAITR/
     # 0.308641975,0.493827161,0.308641975,
     # 0.493827161,0.790123457,0.493827161,
     # 0.308641975,0.493827161,0.308641975/
      DATA XMR/
     # .1       ,0., .01270167,0., .1       ,0., .78729833,0.,0.,
     # .05635083,0., .05635083,0., .44364917,0., .44364917,0.,0.,
     # .01270167,0., .1       ,0., .78729833,0., .1       ,0.,0.,
     # .44364917,0., .05635083,0., .05635083,0., .44364917,0.,0.,
     # .25      ,0., .25      ,0., .25      ,0., .25      ,0.,0.,
     # .05635083,0., .44364917,0., .44364917,0., .05635083,0.,0.,
     # .78729833,0., .1       ,0., .01270167,0., .1       ,0.,0.,
     # .44364917,0., .44364917,0., .05635083,0., .05635083,0.,0.,
     # .1       ,0., .78729833,0.,.1        ,0., .01270167,0.,0./
      DATA DMAR/
     # -.05635083,0.,  .05635083,0.,  .44364917,0., -.44364917,0.,0.,
     # -.05635083,0.,  .05635083,0.,  .44364917,0., -.44364917,0.,0.,
     # -.05635083,0.,  .05635083,0.,  .44364917,0., -.44364917,0.,0.,
     # -.25      ,0.,  .25      ,0.,  .25      ,0., -.25      ,0.,0.,
     # -.25      ,0.,  .25      ,0.,  .25      ,0., -.25      ,0.,0.,
     # -.25      ,0.,  .25      ,0.,  .25      ,0., -.25      ,0.,0.,
     # -.44364917,0.,  .44364917,0.,  .05635083,0., -.05635083,0.,0.,
     # -.44364917,0.,  .44364917,0.,  .05635083,0., -.05635083,0.,0.,
     # -.44364917,0.,  .44364917,0.,  .05635083,0., -.05635083,0.,0./
      DATA DMBR/
     # -.44364917,0., -.05635083,0.,  .05635083,0.,  .44364917,0.,0.,
     # -.25     ,0., -.25      ,0.,  .25      ,0.,  .25      ,0.,0.,
     # -.05635083,0., -.44364917,0.,  .44364917,0.,  .05635083,0.,0.,
     # -.44364917,0., -.05635083,0.,  .05635083,0.,  .44364917,0.,0.,
     # -.25     ,0., -.25      ,0.,  .25      ,0.,  .25      ,0.,0.,
     # -.05635083,0., -.44364917,0.,  .44364917,0.,  .05635083,0.,0.,
     # -.44364917,0., -.05635083,0.,  .05635083,0.,  .44364917,0.,0.,
     # -.25     ,0., -.25      ,0.,  .25      ,0.,  .25      ,0.,0.,
     # -.05635083,0., -.44364917,0.,  .44364917,0.,  .05635083,0.,0./
      DATA XNR/
     #-.10000000, .04508066,-.03237900, .04508066,-.10000000, .35491933,
     # .43237899, .35491933,
     #-.10000000, .11270166,-.10000000, .20000000,-.10000000, .88729833,
     #-.10000000, .20000000,
     #-.03237900, .04508066,-.10000000, .35491933, .43237899, .35491933,
     #-.10000000, .04508066,
     #-.10000000, .20000000,-.10000000, .11270166,-.10000000, .20000000,
     #-.10000000, .88729833,
     #-.25000000, .50000000,-.25000000, .50000000,-.25000000, .50000000,
     #-.25000000, .50000000,
     #-.10000000, .20000000,-.10000000, .88729833,-.10000000, .20000000,
     #-.10000000, .11270166,
     # .43237899, .35491933,-.10000000, .04508066,-.03237900, .04508066,
     #-.10000000, .35491933,
     #-.10000000, .88729833,-.10000000, .20000000,-.10000000, .11270166,
     #-.10000000, .20000000,
     #-.10000000, .35491933, .43237899, .35491933,-.10000000, .04508066,
     #-.03237900, .04508066/
      DATA DNAR/
     #-.04364916, .17459667,-.13094750, .20000000,-.34364916,1.37459665,
     #-1.03094747,-.20000000,
     # .04364916,0.0,-.04364916, .20000000, .34364916,0.0,
     #-.34364916,-.20000000,
     # .13094750,-.17459667, .04364916, .2000000,1.03094747,-1.37459665,
     # .34364916,-.20000000,
     #-.38729833, .77459666,-.38729833, .50000000,-.38729833, .77459666,
     #-.38729833,-.50000000,
     # 0., 0., 0., .5, 0., 0.,
     # 0.,-.5,
     # .38729833,-.77459666, .38729833, .50000000, .38729833,-.77459666,
     # .38729833,-.50000000,
     #-1.03094747,1.37459666,-.34364916, .2000000,-.13094750, .17459667,
     #-.04364916,-.20000000,
     #-.34364916,0.0, .34364916, .20000000,-.04364916,0.0,
     # .04364916,-.20000000,
     # .34364916,-1.37459665,1.03094747, .2000000, .04364916,-.17459667,
     # .13094750,-.20000000/
      DATA DNBR/
     # .34364916,-.20000000, .13094750,-.17459667, .04364916, .20000000,
     #1.03094747,-1.37459665,
     # .38729833,-.50000000, .38729833,-.77459666, .38729833, .50000000,
     # .38729833,-.77459666,
     # .13094750,-.20000000, .34364916,-1.37459665,1.03094747, .2000000,
     # .04364916,-.17459667,
     #-.34364916,-.20000000, .04364916,0.0,-.04364916, .20000000,
     # .34364916,0.0,
     # 0.,-.5, 0., 0., 0., .5,
     # 0., 0.,
     # .04364916,-.20000000,-.34364916,0.0, .34364916, .20000000,
     #-.04364916,0.0,
     #-1.03094747,-.2000000,-.04364916, .17459667,-.13094750, .20000000,
     #-.34364916,1.37459665,
     #-.38729833,-.50000000,-.38729833, .77459666,-.38729833, .50000000,
     #-.38729833, .77459666,
     #-.04364916,-.2000000,-1.03094747,1.37459665,-.34364916, .20000000,
     #-.13094750, .17459667/
      DATA WAITRH/
     #  0.12100300,2*0.22685186,0.12100300,
     #  0.22685186,2*0.42529330,2*0.22685186,2*0.42529330,
     #  0.22685186,0.12100300,2*0.22685186,0.12100300/
      DATA XMRH/
     # 0.8659571, 0., 0.0646111, 0., 0.0048208, 0., 0.0646111, 0.,
     # 0.6234718, 0., 0.0465187, 0., 0.0229132, 0., 0.3070963, 0.,
     # 0.3070963, 0., 0.0229132, 0., 0.0465187, 0., 0.6234718, 0.,
     # 0.0646111, 0., 0.0048208, 0., 0.0646111, 0., 0.8659571, 0.,
     # 0.6234718, 0., 0.3070963, 0., 0.0229132, 0., 0.0465187, 0.,
     # 0.4488873, 0., 0.2211032, 0., 0.1089063, 0., 0.2211032, 0.,
     # 0.2211032, 0., 0.1089063, 0., 0.2211032, 0., 0.4488873, 0.,
     # 0.0465187, 0., 0.0229132, 0., 0.3070963, 0., 0.6234718, 0.,
     # 0.3070963, 0., 0.6234718, 0., 0.0465187, 0., 0.0229132, 0.,
     # 0.2211032, 0., 0.4488873, 0., 0.2211032, 0., 0.1089063, 0.,
     # 0.1089063, 0., 0.2211032, 0., 0.4488873, 0., 0.2211032, 0.,
     # 0.0229132, 0., 0.0465187, 0., 0.6234718, 0., 0.3070963, 0.,
     # 0.0646111, 0., 0.8659571, 0., 0.0646111, 0., 0.0048208, 0.,
     # 0.0465187, 0., 0.6234718, 0., 0.3070963, 0., 0.0229132, 0.,
     # 0.0229132, 0., 0.3070963, 0., 0.6234718, 0., 0.0465187, 0.,
     # 0.0048208, 0., 0.0646111, 0., 0.8659571, 0., 0.0646111, 0./
      DATA DMARH/
     # -.4652841, 0.,  .4652841, 0.,  .0347159, 0., -.0347159, 0.,
     # -.3349953, 0.,  .3349953, 0.,  .1650047, 0., -.1650047, 0.,
     # -.1650047, 0.,  .1650047, 0.,  .3349953, 0., -.3349953, 0.,
     # -.0347159, 0.,  .0347159, 0.,  .4652841, 0., -.4652841, 0.,
     # -.4652841, 0.,  .4652841, 0.,  .0347159, 0., -.0347159, 0.,
     # -.3349953, 0.,  .3349953, 0.,  .1650047, 0., -.1650047, 0.,
     # -.1650047, 0.,  .1650047, 0.,  .3349953, 0., -.3349953, 0.,
     # -.0347159, 0.,  .0347159, 0.,  .4652841, 0., -.4652841, 0.,
     # -.4652841, 0.,  .4652841, 0.,  .0347159, 0., -.0347159, 0.,
     # -.3349953, 0.,  .3349953, 0.,  .1650047, 0., -.1650047, 0.,
     # -.1650047, 0.,  .1650047, 0.,  .3349953, 0., -.3349953, 0.,
     # -.0347159, 0.,  .0347159, 0.,  .4652841, 0., -.4652841, 0.,
     # -.4652841, 0.,  .4652841, 0.,  .0347159, 0., -.0347159, 0.,
     # -.3349953, 0.,  .3349953, 0.,  .1650047, 0., -.1650047, 0.,
     # -.1650047, 0.,  .1650047, 0.,  .3349953, 0., -.3349953, 0.,
     # -.0347159, 0.,  .0347159, 0.,  .4652841, 0., -.4652841, 0./
      DATA DMBRH/
     # -.4652841, 0., -.0347159, 0.,  .0347159, 0.,  .4652841, 0.,
     # -.4652841, 0., -.0347159, 0.,  .0347159, 0.,  .4652841, 0.,
     # -.4652841, 0., -.0347159, 0.,  .0347159, 0.,  .4652841, 0.,
     # -.4652841, 0., -.0347159, 0.,  .0347159, 0.,  .4652841, 0.,
     # -.3349953, 0., -.1650047, 0.,  .1650047, 0.,  .3349953, 0.,
     # -.3349953, 0., -.1650047, 0.,  .1650047, 0.,  .3349953, 0.,
     # -.3349953, 0., -.1650047, 0.,  .1650047, 0.,  .3349953, 0.,
     # -.3349953, 0., -.1650047, 0.,  .1650047, 0.,  .3349953, 0.,
     # -.1650047, 0., -.3349953, 0.,  .3349953, 0.,  .1650047, 0.,
     # -.1650047, 0., -.3349953, 0.,  .3349953, 0.,  .1650047, 0.,
     # -.1650047, 0., -.3349953, 0.,  .3349953, 0.,  .1650047, 0.,
     # -.1650047, 0., -.3349953, 0.,  .3349953, 0.,  .1650047, 0.,
     # -.0347159, 0., -.4652841, 0.,  .4652841, 0.,  .0347159, 0.,
     # -.0347159, 0., -.4652841, 0.,  .4652841, 0.,  .0347159, 0.,
     # -.0347159, 0., -.4652841, 0.,  .4652841, 0.,  .0347159, 0.,
     # -.0347159, 0., -.4652841, 0.,  .4652841, 0.,  .0347159, 0./
      DATA XNRH1/
     #  .6254571,  .2405000, -.0646111,  .0179443, -.0131235,  .0179443,
     # -.0646111,  .2405000,
     #  .1253910,  .1731552, -.0707621,  .0614064, -.0504346,  .0852891,
     # -.1470514,  .8230065,
     # -.1470514,  .0852891, -.0504346,  .0614064, -.0707621,  .1731552,
     #  .1253910,  .8230065,
     # -.0646111,  .0179443, -.0131235,  .0179443, -.0646111,  .2405000,
     #  .6254571,  .2405000,
     #  .1253910,  .8230065, -.1470514,  .0852891, -.0504346,  .0614064,
     # -.0707621,  .1731552,
     # -.1436610,  .5925483, -.2211032,  .2918646, -.1829584,  .2918646,
     # -.2211032,  .5925483,
     # -.2211032,  .2918646, -.1829584,  .2918646, -.2211032,  .5925483,
     # -.1436610,  .5925483,
     # -.0707621,  .0614064, -.0504346,  .0852891, -.1470514,  .8230065,
     #  .1253910,  .1731552/
      DATA XNRH2/
     # -.1470514,  .8230065,  .1253910,  .1731552, -.0707621,  .0614064,
     # -.0504346,  .0852891,
     # -.2211032,  .5925483, -.1436610,  .5925483, -.2211032,  .2918646,
     # -.1829584,  .2918646,
     # -.1829584,  .2918646, -.2211032,  .5925483, -.1436610,  .5925483,
     # -.2211032,  .2918646,
     # -.0504346,  .0614064, -.0707621,  .1731552,  .1253910,  .8230065,
     # -.1470514,  .0852891,
     # -.0646111,  .2405000,  .6254571,  .2405000, -.0646111,  .0179443,
     # -.0131235,  .0179443,
     # -.0707621,  .1731552,  .1253910,  .8230065, -.1470514,  .0852891,
     # -.0504346,  .0614064,
     # -.0504346,  .0852891, -.1470514,  .8230065,  .1253910,  .1731552,
     # -.0707621,  .0614064,
     # -.0131235,  .0179443, -.0646111,  .2405000,  .6254571,  .2405000,
     # -.0646111,  .0179443/
      DATA DNARH1/
     #-1.2020190, 1.6026920, -.4006730,  .1292221, -.0896854,  .1195806,
     # -.0298951, -.1292221,
     # -.6908452, 1.1539063, -.4630611,  .4422064, -.3402816,  .5683663,
     # -.2280847, -.4422064,
     # -.2280847,  .5683663, -.3402816,  .4422064, -.4630611, 1.1539063,
     # -.6908452, -.4422064,
     # -.0298951,  .1195806, -.0896854,  .1292221, -.4006730, 1.6026920,
     #-1.2020190, -.1292221,
     # -.7170485,  .6327511,  .0842975,  .1292221, -.0535007,  .0472110,
     #  .0062896, -.1292221,
     # -.3416761,  .4555681, -.1138920,  .4422064, -.1682955,  .2243939,
     # -.0560985, -.4422064,
     # -.0560985,  .2243939, -.1682955,  .4422064, -.1138920,  .4555681,
     # -.3416761, -.4422064,
     #  .0062896,  .0472110, -.0535007,  .1292221,  .0842975,  .6327511,
     # -.7170485, -.1292221/
      DATA DNARH2/
     # -.0842975, -.6327511,  .7170485,  .1292221, -.0062896, -.0472110,
     #  .0535007, -.1292221,
     #  .1138920, -.4555681,  .3416761,  .4422064,  .0560985, -.2243939,
     #  .1682955, -.4422064,
     #  .1682955, -.2243939,  .0560985,  .4422064,  .3416761, -.4555681,
     #  .1138920, -.4422064,
     #  .0535007, -.0472110, -.0062896,  .1292221,  .7170485, -.6327511,
     # -.0842975, -.1292221,
     #  .4006730,-1.6026920, 1.2020190,  .1292221,  .0298951, -.1195806,
     #  .0896854, -.1292221,
     #  .4630611,-1.1539063,  .6908452,  .4422064,  .2280847, -.5683663,
     #  .3402816, -.4422064,
     #  .3402816, -.5683663,  .2280847,  .4422064,  .6908452,-1.1539063,
     #  .4630611, -.4422064,
     #  .0896854, -.1195806,  .0298951,  .1292221, 1.2020190,-1.6026920,
     #  .4006730, -.1292221/
      DATA DNBRH1/
     #-1.2020190, -.1292221, -.0298951,  .1195806, -.0896854,  .1292221,
     # -.4006730, 1.6026920,
     # -.7170485, -.1292221,  .0062896,  .0472110, -.0535007,  .1292221,
     #  .0842975,  .6327511,
     # -.0842975, -.1292221,  .0535007, -.0472110, -.0062896,  .1292221,
     #  .7170485, -.6327511,
     #  .4006730, -.1292221,  .0896854, -.1195806,  .0298951,  .1292221,
     # 1.2020190,-1.6026920,
     # -.6908452, -.4422064, -.2280847,  .5683663, -.3402816,  .4422064,
     # -.4630611, 1.1539063,
     # -.3416761, -.4422064, -.0560985,  .2243939, -.1682955,  .4422064,
     # -.1138920,  .4555681,
     #  .1138920, -.4422064,  .1682955, -.2243939,  .0560985,  .4422064,
     #  .3416761, -.4555681,
     #  .4630611, -.4422064,  .3402816, -.5683663,  .2280847,  .4422064,
     #  .6908452,-1.1539063/
      DATA DNBRH2/
     # -.2280847, -.4422064, -.6908452, 1.1539063, -.4630611,  .4422064,
     # -.3402816,  .5683663,
     # -.0560985, -.4422064, -.3416761,  .4555681, -.1138920,  .4422064,
     # -.1682955,  .2243939,
     #  .1682955, -.4422064,  .1138920, -.4555681,  .3416761,  .4422064,
     #  .0560985, -.2243939,
     #  .3402816, -.4422064,  .4630611,-1.1539063,  .6908452,  .4422064,
     #  .2280847, -.5683663,
     # -.0298951, -.1292221,-1.2020190, 1.6026920, -.4006730,  .1292221,
     # -.0896854,  .1195806,
     #  .0062896, -.1292221, -.7170485,  .6327511,  .0842975,  .1292221,
     # -.0535007,  .0472110,
     #  .0535007, -.1292221, -.0842975, -.6327511,  .7170485,  .1292221,
     # -.0062896, -.0472110,
     #  .0896854, -.1292221,  .4006730,-1.6026920, 1.2020190,  .1292221,
     #  .0298951, -.1195806/
C
C---- LAGRANGIAN QUADRILATERAL ELEMENT WEIGHTS AND SHAPE FUNCTIONS
      DATA XNQ/
     #-.06000000,-.03491933, .00762100,-.03491933,-.06000000, .27491933,
     # .47237900, .27491933, .16000000,
     # .0       ,-.08729833, .0       , .0       , .0       , .68729833,
     # .0       , .0       , .40000000,
     # .00762100,-.03491933,-.06000000, .27491933, .47237900, .27491933,
     #-.06000000,-.03491933, .16000000,
     # .0       , .0       , .0       ,-.08729833, .0       , .0       ,
     # .0       , .68729833, .40000000,
     # .0       , .0       , .0       , .0       , .0       , .0       ,
     # .0       , .0       ,1.00000000,
     # .0       , .0       , .0       , .68729833, .0       , .0       ,
     # .0       ,-.08729833, .40000000,
     # .47237900, .27491933,-.06000000,-.03491933, .00762100,-.03491933,
     #-.06000000, .27491933, .16000000,
     # .0       , .68729833, .0       , .0       , .0       ,-.08729833,
     # .0       , .0       , .40000000,
     #-.06000000, .27491933, .47237900, .27491933,-.06000000,-.03491933,
     # .00762100,-.03491933, .16000000/
      DATA DNAQ/
     # .11127017,-.13524200, .02397183,-.10983867,-.18872983,1.06475800,
     #-.87602817,-.50983867, .61967734,
     # .04364917, .0       ,-.04364917, .20000000, .34364917, .0       ,
     #-.34364917,-.20000000, .0       ,
     #-.02397183, .13524200,-.11127017, .50983867, .87602817,-1.0647580,
     # .18872983, .10983867,-.61967734,
     # .0       , .0       , .0       ,-.27459667, .0       , .0       ,
     # .0      ,-1.27459667,1.54919334,
     # .0       , .0       , .0       , .50000000, .0       , .0       ,
     # .0       ,-.50000000, .0       ,
     # .0       , .0       , .0       ,1.27459667, .0       , .0       ,
     # .0       ,.27459667,-1.54919334,
     #-.87602817,1.06475800,-.18872983,-.10983867, .02397183,-.13524200,
     # .11127017,-.50983867, .61967734,
     #-.34364917, .0       , .34364917, .20000000,-.04364917, .0       ,
     # .04364917,-.20000000, .0       ,
     # .18872983,-1.0647580, .87602817, .50983867,-.11127017, .13524200,
     #-.02397183, .10983867,-.61967734/
      DATA DNBQ/
     # .18872983, .10983867,-.02397183, .13524200,-.11127017, .50983867,
     # .87602817,-1.0647580,-.61967734,
     # .0       , .27459667, .0       , .0       , .0       ,1.27459667,
     # .0       , .0      ,-1.54919334,
     #-.02397183, .10983867, .18872983,-1.0647580, .87602817, .50983867,
     #-.11127017, .13524200,-.61967734,
     #-.34364917,-.20000000, .04364917, .0       ,-.04364917, .20000000,
     # .34364917, .0       , .0       ,
     # .0       ,-.50000000, .0       , .0       , .0       , .50000000,
     # .0       , .0       , .0       ,
     # .04364917,-.20000000,-.34364917, .0       , .34364917, .20000000,
     #-.04364917, .0       , .0       ,
     #-.87602817,-.50983867, .11127017,-.13524200, .02397183,-.10983867,
     #-.18872983,1.06475800, .61967734,
     # .0      ,-1.27459667, .0       , .0       , .0       ,-.27459667,
     # .0       , .0       ,1.54919334,
     # .11127017,-.50983867,-.87602817,1.06475800,-.18872983,-.10983867,
     # .02397183,-.13524200, .61967734/
      DATA XNQH1/
     # 0.6421553, 0.2071033,-0.0479127,-0.0154525, 0.0035749,-0.0154525,
     #-0.0479127, 0.2071033, 0.0667934,
     # 0.1825338, 0.0588695,-0.0136193,-0.0528793, 0.0067083,-0.0289967,
     #-0.0899086, 0.7087207, 0.2285714,
     #-0.0899086,-0.0289967, 0.0067083,-0.0528793,-0.0136193, 0.0588695,
     # 0.1825339, 0.7087207, 0.2285714,
     #-0.0479127,-0.0154525, 0.0035749,-0.0154525,-0.0479127, 0.2071032,
     # 0.6421555, 0.2071033, 0.0667934,
     # 0.1825338, 0.7087207,-0.0899086,-0.0289967, 0.0067083,-0.0528793,
     #-0.0136193, 0.0588695, 0.2285714,
     # 0.0518856, 0.2014551,-0.0255567,-0.0992285, 0.0125882,-0.0992285,
     #-0.0255567, 0.2014551, 0.7821862,
     #-0.0255567,-0.0992284, 0.0125882,-0.0992285,-0.0255567, 0.2014551,
     # 0.0518856, 0.2014551, 0.7821862,
     #-0.0136193,-0.0528793, 0.0067083,-0.0289967,-0.0899086, 0.7087207,
     # 0.1825339, 0.0588695, 0.2285714/
      DATA XNQH2/
     #-0.0899086, 0.7087207, 0.1825339, 0.0588695,-0.0136193,-0.0528793,
     # 0.0067083,-0.0289967, 0.2285714,
     #-0.0255567, 0.2014551, 0.0518856, 0.2014551,-0.0255567,-0.0992285,
     # 0.0125882,-0.0992284, 0.7821862,
     # 0.0125882,-0.0992284,-0.0255567, 0.2014551, 0.0518856, 0.2014551,
     #-0.0255567,-0.0992284, 0.7821862,
     # 0.0067083,-0.0528793,-0.0136193, 0.0588695, 0.1825338, 0.7087207,
     #-0.0899086,-0.0289967, 0.2285714,
     #-0.0479127, 0.2071033, 0.6421555, 0.2071032,-0.0479127,-0.0154525,
     # 0.0035749,-0.0154525, 0.0667934,
     #-0.0136193, 0.0588695, 0.1825339, 0.7087207,-0.0899086,-0.0289967,
     # 0.0067083,-0.0528793, 0.2285714,
     # 0.0067083,-0.0289967,-0.0899086, 0.7087207, 0.1825338, 0.0588695,
     #-0.0136193,-0.0528793, 0.2285714,
     # 0.0035749,-0.0154525,-0.0479127, 0.2071032, 0.6421553, 0.2071032,
     #-0.0479127,-0.0154525, 0.0667934/
      DATA DNAQH1/
     #-1.0907414, 1.3801360,-0.2893952,-0.0933336, 0.0215924,-0.1029752,
     # 0.0813827,-0.3517778, 0.4451114,
     #-0.3100452, 0.3923061,-0.0822611,-0.3193936, 0.0405184,-0.1932338,
     # 0.1527154,-1.2038066, 1.5232000,
     # 0.1527153,-0.1932338, 0.0405184,-0.3193936,-0.0822611, 0.3923063,
     #-0.3100452,-1.2038066, 1.5232000,
     # 0.0813827,-0.1029752, 0.0215924,-0.0933336,-0.2893952, 1.3801362,
     #-1.0907412,-0.3517778, 0.4451114,
     #-0.6731155, 0.5448848, 0.1282306, 0.0413560,-0.0095676,-0.0406551,
     # 0.0502227,-0.2170882, 0.1757323,
     #-0.1913343, 0.1548845, 0.0364498, 0.1415228,-0.0179536,-0.0762897,
     # 0.0942433,-0.7428901, 0.6013671,
     # 0.0942433,-0.0762897,-0.0179536, 0.1415228, 0.0364498, 0.1548845,
     #-0.1913343,-0.7428901, 0.6013671,
     # 0.0502227,-0.0406551,-0.0095676, 0.0413560, 0.1282305, 0.5448849,
     #-0.6731155,-0.2170882, 0.1757323/
      DATA DNAQH2/
     #-0.1282306,-0.5448850, 0.6731154, 0.2170882,-0.0502227, 0.0406551,
     # 0.0095676,-0.0413560,-0.1757323,
     #-0.0364498,-0.1548845, 0.1913343, 0.7428900,-0.0942433, 0.0762897,
     # 0.0179536,-0.1415229,-0.6013672,
     # 0.0179536, 0.0762897,-0.0942433, 0.7428900, 0.1913342,-0.1548845,
     #-0.0364498,-0.1415229,-0.6013672,
     # 0.0095676, 0.0406551,-0.0502227, 0.2170882, 0.6731154,-0.5448849,
     #-0.1282306,-0.0413560,-0.1757323,
     # 0.2893952,-1.3801365, 1.0907412, 0.3517777,-0.0813827, 0.1029752,
     #-0.0215924, 0.0933336,-0.4451115,
     # 0.0822611,-0.3923063, 0.3100451, 1.2038064,-0.1527154, 0.1932338,
     #-0.0405184, 0.3193936,-1.5232003,
     #-0.0405184, 0.1932337,-0.1527154, 1.2038064, 0.3100451,-0.3923062,
     # 0.0822611, 0.3193936,-1.5232003,
     #-0.0215924, 0.1029751,-0.0813827, 0.3517777, 1.0907409,-1.3801362,
     # 0.2893951, 0.0933336,-0.4451115/
      DATA DNBQH1/
     #-1.0907414,-0.3517778, 0.0813827,-0.1029752, 0.0215924,-0.0933336,
     #-0.2893952, 1.3801360, 0.4451114,
     #-0.6731155,-0.2170882, 0.0502227,-0.0406551,-0.0095676, 0.0413560,
     # 0.1282306, 0.5448848, 0.1757323,
     #-0.1282306,-0.0413560, 0.0095676, 0.0406551,-0.0502227, 0.2170882,
     # 0.6731154,-0.5448850,-0.1757323,
     # 0.2893952, 0.0933336,-0.0215924, 0.1029752,-0.0813827, 0.3517777,
     # 1.0907412,-1.3801365,-0.4451115,
     #-0.3100452,-1.2038066, 0.1527154,-0.1932338, 0.0405184,-0.3193936,
     #-0.0822611, 0.3923061, 1.5232000,
     #-0.1913343,-0.7428901, 0.0942433,-0.0762897,-0.0179536, 0.1415228,
     # 0.0364498, 0.1548845, 0.6013671,
     #-0.0364498,-0.1415229, 0.0179536, 0.0762897,-0.0942433, 0.7428900,
     # 0.1913343,-0.1548845,-0.6013672,
     # 0.0822611, 0.3193936,-0.0405184, 0.1932338,-0.1527154, 1.2038064,
     # 0.3100451,-0.3923063,-1.5232003/
      DATA DNBQH2/
     # 0.1527153,-1.2038066,-0.3100452, 0.3923063,-0.0822611,-0.3193936,
     # 0.0405184,-0.1932338, 1.5232000,
     # 0.0942433,-0.7428901,-0.1913343, 0.1548845, 0.0364498, 0.1415228,
     #-0.0179536,-0.0762897, 0.6013671,
     # 0.0179536,-0.1415229,-0.0364498,-0.1548845, 0.1913342, 0.7428900,
     #-0.0942433, 0.0762897,-0.6013672,
     #-0.0405184, 0.3193936, 0.0822611,-0.3923062, 0.3100451, 1.2038064,
     #-0.1527154, 0.1932337,-1.5232003,
     # 0.0813827,-0.3517778,-1.0907412, 1.3801362,-0.2893952,-0.0933336,
     # 0.0215924,-0.1029752, 0.4451114,
     # 0.0502227,-0.2170882,-0.6731155, 0.5448849, 0.1282305, 0.0413560,
     #-0.0095676,-0.0406551, 0.1757323,
     # 0.0095676,-0.0413560,-0.1282306,-0.5448849, 0.6731154, 0.2170882,
     #-0.0502227, 0.0406551,-0.1757323,
     #-0.0215924, 0.0933336, 0.2893951,-1.3801362, 1.0907409, 0.3517777,
     #-0.0813827, 0.1029751,-0.4451115/
      END
C
C
C
      SUBROUTINE BSPECS(L,NCN)
C
C---- BSPECS ---------------------------------------------------------S
C
C  PURPOSE:
C    INSERTS CONSTRAINT EQUATIONS AND BOUNDARY SPECIFICATIONS INTO
C    THE COEFFICIENT MATRIX AND RESIDUAL VECTOR.  ALONG ALL SOLID
C    BOUNDARIES, THE X AND Y MOMENTUM EQUATIONS ARE TRANSFORMED TO
C    THE TANGENTIAL AND NORMAL MOMENTUM EQUATIONS, RESPECTIVELY,
C    UNLESS OTHERWISE SPECIFIED.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER
C    NCN - NUMBER OF NODES IN ELEMENT CONNECTIVITY LIST
C
C  SUBPROGRAM(S) CALLED:
C    WEIRQ, CULVQ, XSECQ
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,NCN
C
C---- Declarations (local)
      INTEGER   IA,IB,IC,IH,IH1,IH2,IJ,J,JJ,K,N,NCNX,
     #          NFIX1,NFIX2,NFIX3,N1,N2
      REAL      U,V,H,SGN,CSN,TEMP,QWER,QCUL,DQDH,WS1,WS2,CF
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      NCNX=NCN/2
C
C---- LOOP OVER NODE POINTS
      DO 100 K=1,NCN
        N=IABS(NOP(L,K))
        IF(NFIX(N).EQ.0) GOTO 100
        NFIX1=NFIX(N)/10000
        NFIX2=MOD(NFIX(N)/1000,10)
        NFIX3=MOD(NFIX(N)/100,10)
        U=VEL(1,N)
        V=VEL(2,N)
        H=VEL(3,N)
        IA=1+NDF*(K-1)
        IB=IA+1
        IC=IA+2
        IF(MOD(K,2).EQ.0) THEN
          IH1=NDF*(K-2)+3
          IF(NCN.EQ.6) IH2=MOD(K+1,6)
          IF(NCN.GT.6) IH2=MOD(K+1,8)
          IH2=NDF*(IH2-1)+3
        ELSE
          IH1=IC
          IH2=IH1
        ENDIF
        SGN=SIN(ALFA(1,N))
        CSN=COS(ALFA(1,N))
C
C------ FORM TANGENTIAL/NORMAL MOMENTUM EQUATIONS ON BOUNDARIES
        IF(NFIX1.GT.2 .OR. NFIX2.GT.2) THEN
          DO 20 J=1,NCN
            IJ=NDF*(J-1)
            DO 10 JJ=1,NDF
              IJ=IJ+1
              TEMP=ECOEFM(IA,IJ)*CSN+ECOEFM(IB,IJ)*SGN
              ECOEFM(IB,IJ)=-ECOEFM(IA,IJ)*SGN+ECOEFM(IB,IJ)*CSN
              ECOEFM(IA,IJ)=TEMP
10          CONTINUE
20        CONTINUE
          TEMP=F(IA)*CSN+F(IB)*SGN
          F(IB)=-F(IA)*SGN+F(IB)*CSN
          F(IA)=TEMP
        ENDIF
C
C------ ZERO OUT X/TANGENTIAL FLOW EQUATION IF NFIX1 > 0
        IF(NFIX1.GT.0) THEN
          DO 30 J=1,27
            ECOEFM(IA,J)=0.0
30        CONTINUE
          F(IA)=0.0
        ENDIF
C
C------ ZERO OUT Y/NORMAL FLOW EQUATION IF NFIX2 > 0
        IF(NFIX2.GT.0) THEN
          DO 40 J=1,27
            ECOEFM(IB,J)=0.0
40        CONTINUE
          F(IB)=0.0
        ENDIF
C
C------ ZERO OUT DEPTH EQUATION IF NFIX3 = 1
        IF(NFIX3.EQ.1) THEN
          DO 50 J=1,27
            ECOEFM(IC,J)=0.0
50        CONTINUE
          F(IC)=0.0
        ENDIF
C
C------ INSERT OPEN BOUNDARY TOTAL FLOW EQUATION IF NFIX1 = 5
        IF(NFIX1.EQ.5) THEN
          ECOEFM(IA,IA)=UCOFO(K)
          ECOEFM(IA,IB)=VCOFO(K)
          IH=3-2*NDF
          JJ=-1
          DO 60 J=1,NCNX
            JJ=JJ+2
            IH=IH+2*NDF
            ECOEFM(IA,IH)=HCOFO(JJ,K)
60        CONTINUE
          F(IA)=-U*UCOFO(K)-V*VCOFO(K)
        ENDIF
C
C------ INSERT SOLID BOUNDARY TOTAL FLOW EQUATION IF NFIX2 = 5
        IF(NFIX2.EQ.5) THEN
          ECOEFM(IB,IA)=UCOFS(K)
          ECOEFM(IB,IB)=VCOFS(K)
          IH=3-2*NDF
          JJ=-1
          DO 70 J=1,NCNX
            JJ=JJ+2
            IH=IH+2*NDF
            ECOEFM(IB,IH)=HCOFS(JJ,K)
70        CONTINUE
          F(IB)=-U*UCOFS(K)-V*VCOFS(K)
        ENDIF
C
C------ INSERT BOUNDARY SPECS IF N IS MAKING ITS LAST APPEARANCE
        IF(NOP(L,K).GT.0) GOTO 100
C
C------ X VELOCITY IF NFIX1 = 1
        IF(NFIX1.EQ.1) THEN
          ECOEFM(IA,IA)=1.0
          F(IA)=SPEC(N,1)-U
C
C------ X UNIT DISCHARGE IF NFIX1 = 2
        ELSEIF(NFIX1.EQ.2) THEN
          ECOEFM(IA,IA)=H
          TEMP=0.5*U
          ECOEFM(IA,IH1)=TEMP
          ECOEFM(IA,IH2)=ECOEFM(IA,IH2)+TEMP
          F(IA)=SPEC(N,1)-U*H
C
C------ TANGENTIAL VELOCITY IF NFIX1 = 3
        ELSEIF(NFIX1.EQ.3) THEN
          ECOEFM(IA,IA)=CSN
          ECOEFM(IA,IB)=SGN
          F(IA)=SPEC(N,1)-U*CSN-V*SGN
C
C------ TANGENTIAL UNIT DISCHARGE IF NFIX1 = 4
        ELSEIF(NFIX1.EQ.4) THEN
          ECOEFM(IA,IA)=CSN*H
          ECOEFM(IA,IB)=SGN*H
          TEMP=0.5*(U*CSN+V*SGN)
          ECOEFM(IA,IH1)=TEMP
          ECOEFM(IA,IH2)=ECOEFM(IA,IH2)+TEMP
          F(IA)=SPEC(N,1)-U*H*CSN-V*H*SGN
C
C------ TOTAL FLOW NORMAL TO OPEN BOUNDARY IF NFIX1 = 5
        ELSEIF(NFIX1.EQ.5) THEN
          F(IA)=F(IA)-SPEC(N,1)
        ENDIF
C
C------ Y VELOCITY IF NFIX2 = 1
        IF(NFIX2.EQ.1) THEN
          ECOEFM(IB,IB)=1.0
          F(IB)=SPEC(N,2)-V
C
C------ Y UNIT DISCHARGE IF NFIX2 = 2
        ELSEIF(NFIX2.EQ.2) THEN
          ECOEFM(IB,IB)=H
          TEMP=0.5*V
          ECOEFM(IB,IH1)=TEMP
          ECOEFM(IB,IH2)=ECOEFM(IB,IH2)+TEMP
          F(IB)=SPEC(N,2)-V*H
C
C------ NORMAL VELOCITY IF NFIX2 = 3
        ELSEIF(NFIX2.EQ.3) THEN
          ECOEFM(IB,IA)=-SGN
          ECOEFM(IB,IB)=CSN
          F(IB)=SPEC(N,2)+U*SGN-V*CSN
C
C------ NORMAL UNIT DISCHARGE IF NFIX2 = 4
        ELSEIF(NFIX2.EQ.4) THEN
          ECOEFM(IB,IA)=-SGN*H
          ECOEFM(IB,IB)=CSN*H
          TEMP=0.5*(V*CSN-U*SGN)
          ECOEFM(IB,IH1)=TEMP
          ECOEFM(IB,IH2)=ECOEFM(IB,IH2)+TEMP
          F(IB)=SPEC(N,2)+U*H*SGN-V*H*CSN
C
C------ TOTAL FLOW NORMAL TO SOLID BOUNDARY IF NFIX2 = 5
        ELSEIF(NFIX2.EQ.5) THEN
          F(IB)=F(IB)-SPEC(N,2)
C
C-------- ADD WEIR FLOW IF N IS A WEIR NODE
          IF(NWER.GT.0) THEN
            DO 80 J=1,NWER
              N1=IABS(NOPW(J,1))
              N2=IABS(NOPW(J,2))
              IF(N.EQ.N1 .OR. N.EQ.N2) THEN
C
C-------------- COMPUTE WEIR FLOW
                WS1=VEL(3,N1)+GRND(N1)
                WS2=0.0
                IF(N2.GT.0) WS2=VEL(3,N2)+GRND(N2)
                CALL WEIRQ(J,WS1,WS2,QWER,DQDH,CF)
                IF(N.EQ.N2) THEN
                  QWER=-QWER
                  DQDH=-DQDH
                ENDIF
                IF(DQDH.LT.0.) DQDH=0.0
C
C-------------- ADD TO CONSTRAINT EQUATION COEFFICIENTS
                TEMP=0.5*DQDH
                ECOEFM(IB,IH1)=ECOEFM(IB,IH1)-TEMP
                ECOEFM(IB,IH2)=ECOEFM(IB,IH2)-TEMP
                F(IB)=F(IB)+QWER
              ENDIF
80          CONTINUE
          ENDIF
C
C-------- ADD CULVERT FLOW IF N IS A CULVERT NODE
          IF(NCUL.GT.0) THEN
            DO 90 J=1,NCUL
              N1=IABS(NOPC(J,1))
              N2=IABS(NOPC(J,2))
              IF(N.EQ.N1 .OR. N.EQ.N2) THEN
C
C-------------- COMPUTE CULVERT FLOW
                WS1=VEL(3,N1)+GRND(N1)
                WS2=CELV(J)
                IF(N2.GT.0) WS2=VEL(3,N2)+GRND(N2)
                CALL CULVQ(J,WS1,WS2,QCUL,DQDH)
                IF(N.EQ.N2) THEN
                  QCUL=-QCUL
                  DQDH=-DQDH
                ENDIF
C
C-------------- ADD TO CONSTRAINT EQUATION COEFFICIENTS
                TEMP=0.5*DQDH
                ECOEFM(IB,IH1)=ECOEFM(IB,IH1)-TEMP
                ECOEFM(IB,IH2)=ECOEFM(IB,IH2)-TEMP
                F(IB)=F(IB)+QCUL
              ENDIF
90          CONTINUE
          ENDIF
C
C------ TANGENTIAL VELOCITY IF NFIX2 = 6
        ELSEIF(NFIX2.EQ.6) THEN
          ECOEFM(IB,IA)=CSN
          ECOEFM(IB,IB)=SGN
          F(IB)=SPEC(N,2)-U*CSN-V*SGN
        ENDIF
C
C------ DEPTH IF NFIX3 = 1
        IF(NFIX3.EQ.1) THEN
          ECOEFM(IC,IC)=1.0
          F(IC)=SPEC(N,3)-H
        ENDIF
C
C------ If NFIX3 = 2, depth is specified as a "natural" boundary
C       boundary condition in subroutine BINTEG.
C
C------ If NFIX3 = 3, depth is not specified because supercritical
C       flow occurs at the outflow boundary.  Setting NFIX = 00300
C       keeps the open boundary node from being perceived as a solid
C       boundary node.
C
100   CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE CNORMS
C
C---- CNORMS ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES CONTINUITY NORMS (THAT IS, THE MEAN SQUARE CONTINUITY
C    EQUATION RESIDUAL OVER EACH ELEMENT).  NORMS GREATER THAN A
C    USER-DEFINED VALUE ARE FLAGGED WITH AN ASTERISK.
C
C  SUBPROGRAM(S) CALLED:
C    SFLOAD, LVALUE, SHPFNC, QVALUE, FORMFD
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   I,L,LLO,LSTEP,NCN,NGP
      REAL      AREA,AMW,DETJ,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     #          H,DHDX,DHDY,DHDT,P,DPDX,DPDY,DPDT,ZB,DZBDX,DZBDY,
     #          ZC,DZCDX,DZCDY,TAUSX,TAUSY,FFACT,DFDH,UMEAN,TFRIC,
     #          USTAR,TNU,BETA,R,LAREA
      LOGICAL   PRESS
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(1H+,79(' ')/1H+,'Computing (continuity norms)...')
2020  FORMAT(54X,'*** CONTINUITY NORMS ***'//
     # 2X,4(3X,'ELMT',8X,'NORM',9X,'AREA'))
2001  FORMAT(2X,4(4X,'NO.',6X,'(FT/S)',8X,'(FT2)')/)
2002  FORMAT(2X,4(4X,'NO.',7X,'(M/S)',9X,'(M2)')/)
2010  FORMAT(2X,4(I7,E12.3,A1,F12.2))
2030  FORMAT(/' NOTE -- AN ASTERISK (*) INDICATES A CONTINUITY NORM > ',
     # E10.3)
2100  FORMAT(28X,'*** CONTINUITY NORMS ***'//
     # 7X,2(4X,'ELMT',8X,'NORM',9X,'AREA'))
2101  FORMAT(7X,2(5X,'NO.',6X,'(FT/S)',8X,'(FT2)')/)
2102  FORMAT(7X,2(5X,'NO.',7X,'(M/S)',9X,'(M2)')/)
2110  FORMAT(7X,2(I8,E12.3,A1,F12.2))
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2000)
C
C---- LOOP OVER ELEMENTS
      DO 100 L=1,NE
        B(L)=0.
        IF(LMAT(L).LT.1) GOTO 100
        AREA=0.
C
C-------LOAD PROPER SHAPE FUNCTIONS
        CALL SFLOAD(L,NCN,NGP,0)
C
C------ FILL LOCAL VALUE ARRAYS
        CALL LVALUE(L,NCN)
C
C------ LOOP OVER INTEGRATION POINTS
        DO 50 I=1,NGP
          CALL SHPFNC(NCN,I,DETJ)
          AMW=WAITX(I)*DETJ
          AREA=AREA+AMW
C
C-------- COMPUTE VARIABLES, THEIR DERIVATIVES, AND OTHER PARAMETERS
          CALL QVALUE(L,NCN,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,H,DHDX,
     #    DHDY,DHDT,P,DPDX,DPDY,DPDT,ZB,DZBDX,DZBDY,ZC,DZCDX,DZCDY,
     #    TAUSX,TAUSY,FFACT,DFDH,UMEAN,TFRIC,USTAR,TNU,BETA,PRESS)
C
C-------- COMPUTE CONTINUTIY EQUATION RESIDUAL AND NORM
          R=DHDT+U*DHDX+V*DHDY+H*(DUDX+DVDY)
          B(L)=B(L)+AMW*R*R
50      CONTINUE
        IF(AREA.GT.0.) THEN
          B(L)=SQRT(B(L))/AREA
        ELSE
          B(L)=-1.
        ENDIF
100   CONTINUE
C
C---- OUTPUT NORMS
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2020)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2001)
        ELSE
          WRITE(IO2,2002)
        ENDIF
        LSTEP=(NE+3)/4
      ELSE
        WRITE(IO2,2100)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2101)
        ELSE
          WRITE(IO2,2102)
        ENDIF
        LSTEP=(NE+1)/2
      ENDIF
      DO 120 LLO=1,LSTEP
        L=LLO-LSTEP
C
C------ SET FLAG
        DO 110 I=1,4
          L=L+LSTEP
          IF(L.GT.NE) GOTO 110
          IF(B(L).GT.CFLAG) THEN
            FLAG(I)='*'
          ELSE
            FLAG(I)=' '
          ENDIF
110     CONTINUE
C
C------ PRINT VALUES
        IF(WIDE) THEN
          WRITE(IO2,2010) (L,B(L),FLAG(1+(L-1)/LSTEP),LAREA(L),
     #    L=LLO,NE,LSTEP)
        ELSE
          WRITE(IO2,2110) (L,B(L),FLAG(1+(L-1)/LSTEP),LAREA(L),
     #    L=LLO,NE,LSTEP)
        ENDIF
120   CONTINUE
      WRITE(IO2,2030) CFLAG
      RETURN
      END
C
C
C
      SUBROUTINE CULVQ(J,WS1,WS2,Q,DQDH)
C
C---- CULVQ ----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES FLOW THROUGH A CULVERT.  FLOW IS ASSUMED TO LEAVE THE
C    SYSTEM AT ONE BOUNDARY NODE AND RE-ENTER THE SYSTEM AT ANOTHER
C    BOUNDARY NODE OR TO SIMPLY EXIT THE SYSTEM FROM A SINGLE BOUNDARY
C    NODE AND NOT RETURN.  TO SIMPLIFY COMPUTATIONS, CULVERTS ARE
C    TREATED AS FLOWING FULL WITH BOTH ENDS SUBMERGED (TYPE 4 FLOW) OR
C    AS FLOWING UNDER INLET CONTROL (TYPE 5 FLOW).  TYPE 5 FLOW IS
C    SPECIFIED BY ENTERING A NON-ZERO VALUE FOR THE INVERT ELEVATION.
C    CULVERTS IN WHICH REVERSE FLOW IS NOT PERMITTED (I.E. FLOW FROM
C    NODE2 TO NODE1) ARE IDENTIFIED BY A NEGATIVE VALUE FOR NODE1.
C    CULVERTS AT BOUNDARY NODES WHICH ALLOW FLOW TO LEAVE THE SYSTEM
C    ARE IDENTIFIED BY A ZERO VALUE FOR NODE2.  ONLY TYPE 5 FLOW IS
C    COMPUTED AT THESE CULVERTS.  TYPE 4 FLOW MAY BE SIMULATED, HOWEVER,
C    BY ASSIGNING AN APPROPRIATE VALUE TO THE DISCHARGE COEFFICIENT AND
C    THE INVERT ELEVATION.
C
C  INPUT ARGUMENT(S):
C    J      - CULVERT NUMBER
C    WS1    - WATER SURFACE ELEVATION AT NODE1
C    WS2    - WATER SURFACE ELEVATION AT NODE2
C
C  OUTPUT ARGUMENT(S):
C    Q      - COMPUTED DISCHARGE
C    DQDH   - DERIVATIVE OF DISCHARGER WITH RESPECT TO DEPTH
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   J
      REAL      WS1,WS2,Q,DQDH
C
C---- Declarations (local)
      REAL      DELH,DELH1,DELH2,H,TEMP
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INITIALIZE VALUES AND DETERMINE TYPE OF FLOW
      Q=0.0
      DQDH=0.0
      IF(CELV(J).LT.0. .OR. CELV(J).GT.0. .OR. NOPC(J,2).EQ.0) THEN
C
C------ TYPE 5 FLOW
        DELH1=WS1-CELV(J)
        DELH2=0.0
        IF(NOPC(J,2).GT.0) DELH2=WS2-CELV(J)
        IF(DELH1.GE.DELH2 .OR. NOPC(J,1).GE.0) THEN
          H=DELH1
          IF(DELH2.GT.DELH) H=DELH2
          IF(H.GT.0.) THEN
            Q=CCOF(J)*CARE(J)*SQRT(64.4*H)
            IF(DELH1.LT.DELH2) Q=-Q
            DQDH=0.5*Q/H
          ENDIF
        ENDIF
      ELSE
C
C------ TYPE 4 FLOW
        DELH=WS1-WS2
        IF(DELH.GT.0. .OR. NOPC(J,1).GE.0) THEN
          TEMP=1.0
          IF(CHYR(J).GT.0.) TEMP=1.0+29.17*CCOF(J)*CCOF(J)*CMAN(J)*
     #    CMAN(J)*CLEN(J)/CHYR(J)**(4./3.)
          H=ABS(DELH)
          Q=CCOF(J)*CARE(J)*SQRT(64.4*H/TEMP)
          IF(WS1.LT.WS2) Q=-Q
          IF(H.GT.0.0) DQDH=0.5*Q/H
        ENDIF
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE EQLOAD(IER)
C
C---- EQLOAD ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS AN EQUATION NUMBER TO EACH ACTIVE DEGREE-OF-FREEDOM AND
C    SETS THE LAST APPEARANCE OF EACH NODE NEGATIVE IN THE ELEMENT
C    CONNECTIVITY LISTS.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    LNODES, FORMFD
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,J,JINT,K,L,LKNT,LL,M,N,NCN,
     #          NFRST,NLAST,NWAVE,NWMAX,NWSSQ,NWSUM
      REAL      TEMP
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(1H+,79(' ')/1H+,'Computing...')
2001  FORMAT(/
     # 52X,'*** SOLUTION PARAMETERS ***'//
     # 52X,'NO. OF ELEMENTS.....:',I6/
     # 52X,'NO. OF EQUATIONS....:',I6/
     # 52X,'MAX FRONTWIDTH......:',I6/
     # 52X,'RMS FRONTWIDTH......:',I6/
     # 52X,'AVG FRONTWIDTH......:',I6)
2010  FORMAT(' ***ERROR #470 - MAXIMUM ALLOWABLE NUMBER OF ',
     # 'EQUATIONS HAS BEEN EXCEEDED.')
2030  FORMAT(' ***ERROR #480 - FRONTWIDTH DOES NOT VANISH WITH ',
     # 'LAST ELEMENT.')
2040  FORMAT(' ***ERROR #490 - FRONTWIDTH EXCEEDS MAXIMUM ',
     # 'ALLOWABLE FRONTWIDTH.')
2090  FORMAT(/51X,'*** DEGREE-OF-FREEDOM ARRAY ***'//
     # 6X,4('     N  NFIX NBC(N,K)         '))
2101  FORMAT(/
     # 26X,'*** SOLUTION PARAMETERS ***'//
     # 26X,'NO. OF ELEMENTS.....:',I6/
     # 26X,'NO. OF EQUATIONS....:',I6/
     # 26X,'MAX FRONTWIDTH......:',I6/
     # 26X,'RMS FRONTWIDTH......:',I6/
     # 26X,'AVG FRONTWIDTH......:',I6)
2190  FORMAT(/25X,'*** DEGREE-OF-FREEDOM ARRAY ***'//
     # 6X,2('     N  NFIX NBC(N,K)         '))
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2000)
C
C---- INITIALIZE VARIABLES
      NSZB=0
      LKNT=0
      NLAST=0
      NWAVE=0
      NWMAX=0
      NWSSQ=0
      NWSUM=0
      DO 20 N=1,NP
        DO 10 M=1,NDF
          NBC(N,M)=-1
10      CONTINUE
20    CONTINUE
      DO 22 L=1,NE
        IF(LMAT(L).EQ.0) GOTO 22
        DO 21 K=1,9
          NOP(L,K)=IABS(NOP(L,K))
21      CONTINUE
22    CONTINUE
C
C---- SET LAST APPEARANCE OF EACH NODE IN CONNECTIVITY LISTS NEGATIVE
C     AND SET EQUATION NUMBERS OF ACTIVE DEGREES-OF-FREEDOM TO ZERO
      DO 50 LL=NE,1,-1
        L=ISEQ(LL)
        IF(L.LT.1) GOTO 50
        IF(LMAT(L).LT.1) GOTO 50
        CALL LNODES(L,NCN)
        DO 40 K=1,NCN
          N=NODE(K)
          IF(NBC(N,1).LT.0) THEN
            NOP(L,K)=-N
            DO 30 M=1,NDF
              NBC(N,M)=0
30          CONTINUE
            IF(MOD(K,2).EQ.0) NBC(N,3)=-1
            IF(K.EQ.9) NBC(N,3)=-1
          ENDIF
40      CONTINUE
50    CONTINUE
C
C---- COMPUTE FRONTWIDTH AND ASSIGN EQUATION NUMBERS
      DO 100 LL=1,NE
        L=ISEQ(LL)
        IF(L.LT.1) GOTO 100
        CALL LNODES(L,NCN)
C
C------ FIND NUMBER OF DOF MAKING THEIR FIRST APPEARANCE
        NFRST=0
        DO 70 K=1,NCN
          N=IABS(NODE(K))
          DO 60 M=1,NDF
            IF(NBC(N,M).EQ.0) THEN
              NFRST=NFRST+1
              NBC(N,M)=1
            ENDIF
60        CONTINUE
70      CONTINUE
C
C------ COMPUTE FRONTWIDTH AND SEQUENCE SUM
        LKNT=LKNT+1
        NWAVE=NWAVE+NFRST-NLAST
        NWSSQ=NWSSQ+NWAVE*NWAVE
        NWSUM=NWSUM+NWAVE
        NWMAX=MAX0(NWAVE,NWMAX)
C
C------ FIND NUMBER OF DOF MAKING THEIR LAST APPEARANCE AND ASSIGN
C       EQUATION NUMBERS
        NLAST=0
        DO 90 K=1,NCN
          N=NODE(K)
          IF(N.LT.0) THEN
            N=IABS(N)
            DO 80 M=1,NDF
              IF(NBC(N,M).GT.0) THEN
                NLAST=NLAST+1
                NSZB=NSZB+1
                NBC(N,M)=NSZB
              ENDIF
80          CONTINUE
          ENDIF
90      CONTINUE
100   CONTINUE
C
C---- COMPUTE RMS AND AVG FRONTWIDTH
      TEMP=REAL(NWSSQ)/REAL(LKNT)
      NWSSQ=INT(SQRT(TEMP))
      TEMP=REAL(NWSUM)/REAL(LKNT)
      NWSUM=INT(TEMP)
C
C---- PRINT SOLUTION PARAMETERS
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2001) LKNT,NSZB,NWMAX,NWSSQ,NWSUM
      ELSE
        WRITE(IO2,2101) LKNT,NSZB,NWMAX,NWSSQ,NWSUM
      ENDIF
C
C---- CHECK NUMBER OF SYSTEM EQUATIONS FORMED
      IF(NSZB.GT.MAXQ) THEN
        IER=IER+1
        WRITE(IO2,2010)
      ENDIF
C
C---- CHECK ON FINAL FRONTWIDTH
      IF(NWAVE.NE.NLAST) THEN
        IER=IER+1
        WRITE(IO2,2030)
      ENDIF
C
C---- CHECK ON MAXIMUM FRONTWIDTH
      IF(NWMAX.GT.MAXF) THEN
        IER=IER+1
        WRITE(IO2,2040)
      ENDIF
C
C---- SET EQUATION NUMBERS FOR INACTIVE DOF TO ZERO
      DO 120 N=1,NP
        DO 110 M=1,NDF
          IF(NBC(N,M).LT.0) THEN
            NBC(N,M)=0
          ENDIF
110     CONTINUE
120   CONTINUE
C
C---- PRINT DEGREE-OF-FREEDOM ARRAY IF MOD(IPRNT,32) > 15
      IF(MOD(IPRNT,32).GT.15) THEN
        CALL FORMFD(IO2)
        IF(WIDE) THEN
          WRITE(IO2,2090)
          JINT=(NP+3)/4
          DO 130 J=1,JINT
            WRITE(IO2,'(6X,20I6)') (N,NFIX(N),(NBC(N,K),K=1,NDF),
     #      N=J,NP,JINT)
130       CONTINUE
        ELSE
          WRITE(IO2,2190)
          JINT=(NP+1)/2
          DO 140 J=1,JINT
            WRITE(IO2,'(6X,12I6)') (N,NFIX(N),(NBC(N,K),K=1,NDF),
     #      N=J,NP,JINT)
140       CONTINUE
        ENDIF
      ENDIF
999   RETURN
      END
C
C
C
      SUBROUTINE FORDAR(IER)
C
C---- FORDAR ---------------------------------------------------------S
C
C  PURPOSE:
C    FILLS THE FRONTAL ORDER ARRAY.  THE FRONTAL ORDER ARRAY CONTAINS
C    A LIST OF ELEMENTS IN THE ORDER IN WHICH THEY ARE TO BE ASSEMBLED
C    BY THE FRONTAL SOLVER (SUBROUTINE ZFRONT).  THE ELEMENT SEQUENCE
C    ARRAY IS SCANNED AND THE ELEMENT NUMBERS ARE ENTERED  IN THE
C    APPROPRIATE POSITIONS IN ARRAY ISEQ.  ELEMENTS NOT HAVING ASSIGNED
C    ASSEMBLY ORDERS ARE PROCESSED AFTER THOSE WITH SPECIFIED ORDERS.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,KOUNT,L
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C--- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #450 - ASSEMBLY SEQUENCE',I5,' IS ASSIGNED ',
     # 'TO ELEMENTS ',I5,' AND ',I5,'.')
C
C---- FIND MAXIMUM SPECIFIED SEQUENCE
      KOUNT=0
      DO 10 L=1,NE
        ISEQ(L)=0
        KOUNT=MAX0(KOUNT,LSEQ(L))
10    CONTINUE
C
C---- ASSIGN SEQUENCE TO ELEMENTS WITHOUT ONE
      DO 20 L=1,NE
        IF(LMAT(L).NE.0 .AND. LSEQ(L).LE.0) THEN
          KOUNT=KOUNT+1
          LSEQ(L)=KOUNT
        ENDIF
20    CONTINUE
C
C---- FILL FRONTAL ORDER ARRAY (ISEQ) WITH APPROPRIATE ELEMENT NUMBERS
      DO 30 L=1,NE
        IF(LMAT(L).NE.0) THEN
          I=LSEQ(L)
          IF(ISEQ(I).NE.0) THEN
            IER=IER+1
            WRITE(IO2,2000) I,ISEQ(I),L
          ENDIF
          ISEQ(I)=L
        ENDIF
30    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE FRCFCT(H,M,FFACT,DFDH)
C
C---- FRCFCT ---------------------------------------------------------S
C
C  PURPOSE:
C    CALCULATES THE BOTTOM FRICTION FACTOR AND ITS DERIVATIVE WITH
C    RESPECT TO DEPTH GIVEN THE FLOW DEPTH AND THE ELEMENT TYPE CODE.
C
C  INPUT ARGUMENT(S):
C    H - FLOW DEPTH.
C    M - ELEMENT TYPE CODE.
C
C  OUTPUT ARGUMENT(S):
C    FFACT - BOTTOM FRICTION FACTOR.
C    DFDH - DERIVATIVE OF FFACT WITH RESPECT TO DEPTH.
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   M
      REAL      H,FFACT,DFDH
C
C---- Declarations (local)
      REAL      RC,TEMP,T1,T2,DNDH,DFDN
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Initialize values
      FFACT=0.0
      DFDH=0.0
C
C---- CHECK FOR DEPTH <= 0
      IF(H.LT.ZIP) RETURN
      IF(IFRIC.EQ.0) THEN
C
C------ Compute friction factor using Manning coefficient
        DNDH=0.0
        IF(PROP(M,4).LE.PROP(M,2) .OR. H.LE.PROP(M,2)) THEN
          RC=PROP(M,1)
        ELSE IF(H.GE.PROP(M,4)) THEN
          RC=PROP(M,3)
        ELSE
          TEMP=PROP(M,4)-PROP(M,2)
          T1=(PROP(M,4)-H)/TEMP
          T2=(H-PROP(M,2))/TEMP
          RC=T1*PROP(M,1)+T2*PROP(M,3)
          DNDH=(PROP(M,3)-PROP(M,1))/TEMP
        ENDIF
C------ CALCULATE FRICTION FACTOR AND DERIVATIVES
        FFACT=GRAV*RC*RC/H**(1./3.)
        DFDN=2.*GRAV*RC/H**(1./3.)
        DFDH=-GRAV*RC*RC/(3.*H**(4./3.))
        DFDH=DFDH+DFDN*DNDH
C------ CORRECT FOR UNITS
        IF(IUNIT.EQ.0) THEN
          FFACT=FFACT/2.208
          DFDH=0.453*DFDH
        ENDIF
C
C---- Compute friction factor using Chezy coefficient
      ELSE
        FFACT=GRAV/(PROP(M,5)*PROP(M,5))
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE HIGHOI
C
C---- HIGHOI ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS HIGHER ORDER INTEGRATION TO ALL ELEMENTS OR JUST TO
C    ELEMENTS WITH CURVED SIDES.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   K,L,N1,N2,N3,NCN
      REAL      EPS,XBAR,YBAR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Data
      DATA EPS/0.001/
C
      DO 30 L=1,NE
        IF(LMAT(L).LT.1) GOTO 30
        IF(LMAT(L).GT.100) GOTO 30
        IF(IXINT.GT.1) GOTO 20
        CALL LNODES(L,NCN)
        IF(NCN.EQ.9) NCN=8
        DO 10 K=2,NCN,2
          N1=NODE(K-1)
          N2=NODE(K)
          N3=K+1
          IF(N3.GT.NCN) N3=1
          N3=NODE(N3)
          XBAR=0.5*(CORD(N1,1)+CORD(N3,1))
          IF(ABS(XBAR-CORD(N2,1)).GT.EPS) GOTO 20
          YBAR=0.5*(CORD(N1,2)+CORD(N3,2))
          IF(ABS(YBAR-CORD(N2,2)).GT.EPS) GOTO 20
10      CONTINUE
        GOTO 30
20      LMAT(L)=LMAT(L)+100
30    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE INPUT1(IER)
C
C---- INPUT1 ---------------------------------------------------------S
C
C  PURPOSE:
C    READS PROGRAM CONTROL DATA, NETWORK DATA, AND INITIAL BOUNDARY
C    CONDITION, WIND, AND FLOW FIELD DATA FOR THE FINITE ELEMENT
C    NETWORK.  DATA ARE READ IN GROUPS OF ONE OR MORE RECORDS WHICH
C    CONTAIN RELATED INFORMATION.  THESE DATA SETS ARE IDENTIFIED BY
C    A RECORD WITH A CHARACTER CODE IN THE FIRST FOUR COLUMNS.  THE
C    DATA SETS MUST IMMEDIATELY FOLLOW THE IDENTIFICATION RECORDS.
C    THOSE SATA SETS WHICH CONSIST OF A VARIABLE NUMBER OF RECORDS ARE
C    TERMINATED WITH ONE OR MORE BLANK RECORDS.  ANY DATA READ FROM
C    RECORDS WILL OVERRIDE THOSE WHICH HAVE BEEN READ FROM SPECIFIED
C    INPUT FILES.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    TBLOCK, RDSWMS, RDELEM, RDNODE, RDPROP, RDINIT, RDBOUN, RDWIND,
C    RDFLUX, RDWEIR, RDCULV, RDTIME, RDQSEC, RFGRID, RFRSRC, RFINIT,
C    RFBOUN, RFWIND, RDZSEC
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   L,N,K,LENCHR
      REAL      TSOLD
      CHARACTER IDS*4
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(A4,6X,7E10.0)
C
C---- OUTPUT FORMATS
2000  FORMAT(/' ===> FESWMS-2DH -- FLOMOD Ver. ',A8,' <==='/)
2010  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
2020  FORMAT(/' ***',I5,' ERROR(S) ENCOUNTERED DURING DATA INPUT'/)
C
C---- ASSIGN INITIAL VALUES
      NP=0
      NE=0
      NMAT=0
      NWER=0
      NCUL=0
      NCL=0
      NSECQ=0
      NXSZ=0
      NITSV=0
      ITDS=0
      TET=0.
      TSNEW=-1.0
      TSOLD=TSNEW
      WIDE=.TRUE.
      ECHO=.FALSE.
      SCREEN=.FALSE.
      DO 21 N=1,MAXP
        CORD(N,1)=VOID
        CORD(N,2)=VOID
        GRND(N)=0.
        CEIL(N)=0.
        SIGMA(N,1)=VOID
        SIGMA(N,2)=VOID
        NFIX(N)=0
        DO 20 K=1,NDF
          VEL(K,N)=VOID
          VDOT(K,N)=VOID
          SPEC(N,K)=0.
20      CONTINUE
21    CONTINUE
      DO 23 L=1,MAXE
        LMAT(L)=0
        LSEQ(L)=0
        DO 22 K=1,9
          NOP(L,K)=0
22      CONTINUE
23    CONTINUE
      DO 24 K=1,MAXS
        XSQ(K)=0.
        XSZ1(K)=0.
        XSZ2(K)=0.
24    CONTINUE
      DO 25 K=1,MAXSN
        LXSQN(K)=0
        LXSZN(K)=0
25    CONTINUE
C
C---- Look for 'SWMS' ID record to begin data input
1     READ(IO1,1000) IDS,(XIN(K),K=1,3)
      IF(IDS.EQ.ID(12)) STOP
      IF(IDS.NE.'SWMS') GOTO 1
      IF(XIN(1).GT.0) WIDE=.FALSE.
      IF(XIN(2).GT.0.) SCREEN=.TRUE.
CX    IDEBUG=INT(XIN(3))
      IDEBUG=0
C
C---- Screen acknowledgement
      IF(SCREEN) THEN
        WRITE(6,2000) VERSION
        WRITE(6,2010) DATAFILE(1:LENCHR(DATAFILE))
      ENDIF
C
C---- Print title block
      CALL TBLOCK(VERSION,WIDE,IO2)
C
C---- Read program control (SWMS) data set
      CALL RDSWMS(IDS,IER)
C
C---- Read netwrok data file if IGRID <> 0
      IF(IGRID.NE.0) CALL RFGRID(IER)
C
C---- Read restart/recovery file if IDRUN <> 0
      IF(IDRUN.NE.0) CALL RFRSRC(IER)
C
C---- Read initial condition file if IDRUN <> 0 and INITC <> 0
      IF(IDRUN.EQ.0 .AND. INITC.NE.0) CALL RFINIT(IER)
C
C---- Read boundary condition file if IBCIN <> 0
      IF(IBCIN.NE.0) CALL RFBOUN(IER)
C
C---- Read wind condition file if IWDIN <> 0
      IF(IWDIN.NE.0) CALL RFWIND(IER)
C
C---- READ THE NEXT DATA SET IDENTIFICATION CARD
100   READ(IO1,1000) IDS,(XIN(K),K=1,7)
      DO 102 K=2,15
        IF(IDS.EQ.ID(K)) GOTO 104
102   CONTINUE
      GOTO 100
104   GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,9,100,100,100,100,100),K
C
C---- READ ELEMENT (ELEM) DATA SET
2     CALL RDELEM(IDS,IER)
      GOTO 100
C
C---- READ NODE DATA
3     CALL RDNODE(IDS,IER)
      GOTO 100
C
C---- READ ELEMENT PROPERTY DATA
4     CALL RDPROP(IDS,IER)
      GOTO 100
C
C---- READ INITIAL CONDITION DATA
5     CALL RDINIT(IDS,IER)
      GOTO 100
C
C---- READ BOUNDARY CONDITION DATA
6     CALL RDBOUN(IDS,IER)
      GOTO 100
C
C---- READ WIND DATA
7     CALL RDWIND(IDS,IER)
      GOTO 100
C
C---- READ FLUX LINE DATA
8     CALL RDFLUX(IDS,IER)
      GOTO 100
C
C---- READ WEIR DATA
9     CALL RDWEIR(IDS,IER)
      GOTO 100
C
C---- READ CULVERT DATA
10    CALL RDCULV(IDS,IER)
      GOTO 100
C
C---- READ CROSS-SECTION TOTAL FLOW DATA
11    CALL RDQSEC(IDS,IER)
      GOTO 100
C
C---- READ CROSS-SECTION WATER-SURFACE ELEVATION DATA
12    CALL RDZSEC(IDS,IER)
      GOTO 100
C
C---- READ TIME-DEPENDENT DATA
13    CALL RDTIME(IDS,IER)
      IF(TSNEW.LE.TSTRT) GOTO 100
      GOTO 200
C
C---- END OF DATA
14    CALL RDLAST
C
C---- PRINT NUMBER OF ERRORS FOUND
200   IF(IER.GT.0) WRITE(IO2,2020) IER
      RETURN
      END
C
C
C
      SUBROUTINE INPUT2(IER)
C
C---- INPUT2 ---------------------------------------------------------S
C
C  PURPOSE:
C    READS UPDATED BOUNDARY CONDITION AND WIND DATA FROM RECORDS OR
C    THE APPROPRIATE FILES.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    RFBOUN, RFWIND, RDBOUN, RDWIND, RDTIME, RDLAST, RDQSEC
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   K
      CHARACTER IDS*4
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
2010  FORMAT(' ***ERROR #395 - TIME-DEPENDENT DATA SETS ARE ',
     # 'OUT OF ORDER.')
2030  FORMAT(//39X,'*** BOUNDARY CONDITION UPDATE AT ',F8.3,' HRS ***'/)
2035  FORMAT(//46X,'*** WIND DATA UPDATE AT ',F8.3,' HRS ***'/)
2055  FORMAT(//
     # 36X,'*** CROSS-SECTION TOTAL FLOW DATA UPDATE AT ',F8.3,
     # ' HRS ***'/)
2060  FORMAT(//
     # 30X,'*** CROSS-SECTION WATER-SURFACE ELEVATION DATA UPDATE AT ',
     # F8.3,' HRS ***'/)
2130  FORMAT(//13X,'*** BOUNDARY CONDITION UPDATE AT ',F8.3,' HRS ***'/)
2135  FORMAT(//20X,'*** WIND DATA UPDATE AT ',F8.3,' HRS ***'/)
2155  FORMAT(//
     # 10X,'*** CROSS-SECTION TOTAL FLOW DATA UPDATE AT ',F8.3,
     # ' HRS ***'/)
2160  FORMAT(//
     # 4X, '*** CROSS-SECTION WATER-SURFACE ELEVATION DATA UPDATE AT ',
     # F8.3,' HRS ***'/)
2100  FORMAT(/' ***',I5,' ERROR(S) ENCOUNTERED DURING DATA INPUT'/)
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) DATAFILE(1:LENCHR(DATAFILE))
C
C---- Read boundary condition file if IBCIN > 0
      IF(IBCIN.NE.0) CALL RFBOUN(IER)
C
C---- Read wind data file if IWDIN > 0
      IF(IWDIN.NE.0) CALL RFWIND(IER)
C
C---- Read time-dependent data
      IF((TET+ZIP).LT.TSNEW) GOTO 999
100   READ(IO1,'(A4,6X,7E10.0)') IDS
      DO 102 K=2,14
        IF(IDS.EQ.ID(K)) GOTO 104
102   CONTINUE
104   GOTO (1,1,1,1,1,6,7,1,1,1,11,12,13,14,1,100,100,100,100,100),K
C
C---- Input data sets are out of order
1     WRITE(IO2,2010)
      IER=IER+1
      GOTO 999
C
C---- Read boundary condition data
6     IF(WIDE) THEN
        WRITE(IO2,2030) TET
      ELSE
        WRITE(IO2,2130) TET
      ENDIF
      CALL RDBOUN(IDS,IER)
      GOTO 100
C
C---- Read wind data
7     IF(WIDE) THEN
        WRITE(IO2,2035) TET
      ELSE
        WRITE(IO2,2135) TET
      ENDIF
      CALL RDWIND(IDS,IER)
      GOTO 100
C
C---- Read cross section flow data
11    IF(WIDE) THEN
        WRITE(IO2,2055) TET
      ELSE
        WRITE(IO2,2155) TET
      ENDIF
      CALL RDQSEC(IDS,IER)
      GOTO 100
C
C---- Read cross section water-surface elevation data
12    IF(WIDE) THEN
        WRITE(IO2,2060) TET
      ELSE
        WRITE(IO2,2160) TET
      ENDIF
      CALL RDZSEC(IDS,IER)
      GOTO 100
C
C---- Read time-dependent data
13    CALL RDTIME(IDS,IER)
      IF(TSNEW.LT.(TET+ZIP)) GOTO 100
      GOTO 999
C
C---- End of data
14    CALL RDLAST
C
C---- Print number of errors encountered during data input
999   IF(IER.GT.0) WRITE(IO2,2100) IER
      RETURN
      END
C
C
C
      SUBROUTINE LCHECK(L,IDS,MAXE,IO2,IER,ERR)
C
C---- LCHECK ---------------------------------------------------------S
C
C  PURPOSE:
C    CHECKS FOR A VALID ELEMENT NUMBER.  THE ELEMENT NUMBER MUST BE
C    GREATER THAN ZERO AND LESS THAN OR EQUAL TO THE MAXIMUM ELEMENT
C    DIMENSION, MAXE.  IF AN ELEMENT NUMBER IS FOUND TO BE OUTSIDE
C    THIS RANGE, THE ERROR COUNT IS INCREMENTED BY ONE AND A MESSAGE .
C    IS PRINTED.
C
C  INPUT ARGUMENT(S):
C    L      - ELEMENT NUMBER TO BE CHECKED
C    IDS    - DATA SET FROM WHICH THE ELEMENT NUMBER IS READ
C    MAXE   - MAXIMUM ELEMENT NUMBER (DIMENSION)
C    IO2    - UNIT NUMBER OF LISTING FILE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C    ERR    - LOGICAL ERROR INDICATOR
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,MAXE,IO2,IER
      CHARACTER IDS*(*)
      LOGICAL   ERR
C
C---- Declarations (local)
      INTEGER   LENCHR,NCN,NGP
C
C---- OUTPUT FORMATS
 2000 FORMAT(' ***ERROR #220 - ELEMENT NUMBER ',I5,' IN THE ',(A),
     #       ' DATA SET IS OUT OF RANGE.')
C
      ERR=.FALSE.
      IF(L.LT.1) ERR=.TRUE.
      IF(L.GT.MAXE) ERR=.TRUE.
      IF(ERR) THEN
        IER=IER+1
        WRITE(IO2,2000) L,IDS(1:LENCHR(IDS))
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE LCOEFS(L,KEY)
C
C---- LCOEFS ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES THE ELEMENT COEFFICENT MATRIX AND RESIDUAL VECTOR FOR A
C    NEWTON SOLUTION OF THE SHALLOW WATER EQUATIONS.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER.
C    KEY - OPERATIONAL CONTROL KEY.
C
C                  1 - PERFORM LU FACTORIZATION OF A AND SOLVE AX = B
C            KEY = 2 - PERFORM LU FACTORIZATION OF A ONLY
C                  3 - REDUCE LOAD VECTOR B AND SOLVE FOR X
C
C  SUBPROGRAM(S) CALLED:
C    SFLOAD, LVALUE, SHPFNC, QVALUE, QRESID, QCOEFS, BINTEG, BSPECS
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,KEY
C
C---- Declarations (local)
      INTEGER   I,J,K,KODE,LENCHR,NBN,NCM,NCN,NGP
      REAL      AREA,WEIGHT,DETJ,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     #          H,DHDX,DHDY,DHDT,P,DPDX,DPDY,DPDT,ZB,DZBDX,DZBDY,
     #          ZC,DZCDX,DZCDY,TAUSX,TAUSY,FFACT,DFDH,UMEAN,TFRIC,
     #          USTAR,TNU,BETA
      LOGICAL   PRESS
C
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2030  FORMAT(1X,'RESIDUALS'/1X,1P10E13.6/1X,1P10E13.6/1X,1P7E13.6)
2032  FORMAT(1X,'COEFFICIENTS'/(1X,1P10E13.6/1X,1P10E13.6/1X,1P7E13.6))
2100  FORMAT(' ***WARNING #130 - DETERMINANT <= 0 IN ELEMENT ',I5,
     # ' AT INTEG. POINT ',I2)
C
C---- PRINT ELEMENT NUMBER AND CONNECTIVITY LIST IF MOD(IDEBUG,4) > 1
      IF(MOD(IDEBUG,4).GT.1) THEN
        WRITE(IO2,'(1X,''ELEMENT '',I5,'' IS BEING PROCESSED''/10I6)')
     #  L,(NOP(L,K),K=1,9)
      ENDIF
C
C---- LOAD PROPER SHAPE FUNCTIONS
      KODE=0
CX    IF(LMAT(L).GT.100) KODE=1
      CALL SFLOAD(L,NCN,NGP,KODE)
      NCM=NCN/2
      NBN=NCN*NDF
C
C---- FILL LOCAL VALUE ARRAYS
      CALL LVALUE(L,NCN)
C
C---- INITIALIZE MATRICES AND VARIABLES
      AREA=0.
      DO 20 I=1,NBN
        DO 10 J=1,NBN
          ECOEFM(I,J) = 0.0
10      CONTINUE
        F(I) = 0.0
20    CONTINUE
      DO 40 I=1,9
        UCOFS(I)=0.
        VCOFS(I)=0.
        UCOFO(I)=0.
        VCOFO(I)=0.
        DO 30 J=1,9
          HCOFS(J,I)=0.
          HCOFO(J,I)=0.
30      CONTINUE
40    CONTINUE
C
C---- ELEMENT EQUATIONS
      DO 50 I=1,NGP
C
C---- COMPUTE JACOBIAN AND ASSIGN SHAPE FUNCTIONS AT ITH GAUSS POINT
      CALL SHPFNC(NCN,I,DETJ)
C
C---- CHECK FOR VANISHING DETERMINANT
      IF(DETJ.LE.0.) WRITE(IO2,2100) L,I
C
C---- COMPUTE AREA CONTRIBUTION AND SUM
      WEIGHT=WAITX(I)*DETJ
      AREA=AREA+WEIGHT
C
C---- PRINT DETERMINANT AND WEIGHTING FUNCTIONS IF MOD(IDEBUG,4) > 1
      IF(MOD(IDEBUG,4).GT.1) THEN
        WRITE(IO2,'(1X,I14,1P6E14.7)') I,WEIGHT,DETJ
        WRITE(IO2,'(1X,1P9E14.7/1X,1P9E14.7/1X,1P9E14.7)')
     #  (XN(J),J=1,9),(DNX(J),J=1,9),(DNY(J),J=1,9)
        WRITE(IO2,'(1X,1P4E14.7/1X,1P4E14.7/1X,1P4E14.7)')
     #  (XM(J),J=1,4),(DMX(J),J=1,4),(DMY(J),J=1,4)
      ENDIF
C
C---- COMPUTE VARIABLES, THEIR DERIVATIVES, AND OTHER PARAMETERS
      CALL QVALUE(L,NCN,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     # H,DHDX,DHDY,DHDT,P,DPDX,DPDY,DPDT,ZB,DZBDX,DZBDY,ZC,DZCDX,DZCDY,
     # TAUSX,TAUSY,FFACT,DFDH,UMEAN,TFRIC,USTAR,TNU,BETA,PRESS)
C
C---- PRINT VALUES IF MOD(IDEBUG,4) > 1
      IF(MOD(IDEBUG,4).GT.1) THEN
        WRITE(IO2,'(1X,1P6E14.7/1X,1P6E14.7)')U,DUDX,DUDY,V,DVDX,DVDY,
     #  H,DHDX,DHDY,ZB,DZBDX,DZBDY,
     #  FFACT,UMEAN,USTAR,TFRIC
      ENDIF
C
C---- COMPUTE EQUATION RESIDUALS
      CALL QRESID(NCN,WEIGHT,
     # U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     # H,DHDX,DHDY,DHDT,P,DZBDX,DZBDY,DZCDX,DZCDY,
     # TAUSX,TAUSY,FFACT,UMEAN,TNU,BETA,PRESS)
C
C---- COMPUTE EQUATION COEFFICIENTS IF KEY < 3
      IF(KEY.LT.3) THEN
        CALL QCOEFS(NCN,WEIGHT,
     #  U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,H,DHDX,
     #  DHDY,DHDT,DZBDX,DZBDY,DZCDX,DZCDY,
     #  DFDH,UMEAN,TFRIC,TNU,BETA,PRESS)
      ENDIF
50    CONTINUE
C
C---- COMPUTE BOUNDARY INTEGRALS AND INSERT
      CALL BINTEG(L,NCN,KEY)
C
C---- PRINT RESIDUALS AND COEFFICIENT MATRIX IF MOD(IDEBUG,8) > 3
      IF(MOD(IDEBUG,8).GT.3) THEN
        WRITE(IO2,2030) (F(J),J=1,27)
        WRITE(IO2,2032) ((ECOEFM(J,K),K=1,27),J=1,27)
      ENDIF
C
C---- INSERT CONTRAINT EQUATIONS AND BOUNDARY SPECS
      CALL BSPECS(L,NCN)
C
C---- PRINT ELEMENT RESIDUALS AND COEFFICIENTS IF MOD(IDEBUG,8) > 3
      IF(MOD(IDEBUG,8).GT.3) THEN
        WRITE(IO2,2030) (F(J),J=1,27)
        WRITE(IO2,2032) ((ECOEFM(J,K),K=1,27),J=1,27)
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE LNODES(L,NCN)
C
C---- LNODES ---------------------------------------------------------S
C
C  PURPOSE:
C    DETERMINES THE TOTAL NUMBER OF NODES IN AN ELEMENT AND FILLS
C    THE ARRAY NODE WITH THE CONNECTIVITY LIST.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER.
C
C  OUTPUT ARGUMENT(S):
C    NCN - NUMBER OF NODES IN CONNECTIVITY LIST.
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,NCN
C
C---- Declarations (local)
      INTEGER   K
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      NCN=6
      IF(NOP(L,8).NE.0) NCN=8
      IF(NOP(L,9).NE.0) NCN=9
      DO 10 K=1,9
        NODE(K)=0
10    CONTINUE
      DO 20 K=1,NCN
        NODE(K)=NOP(L,K)
20    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE LONOFF
C
C---- LONOFF ---------------------------------------------------------S
C
C  PURPOSE:
C    DETERMINES WHETHER AN ELEMENT SHOULD BE TURNED ON (IF IT IS
C    CURRENTLY OFF) OR OFF (IF IT IS CURRENTLY ON) BASED ON THE STATUS
C    ON THE ELEMENT'S NODES (THAT IS, WHETHER THEY ARE WET OR DRY).
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C--------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   I,IDLAST,K,KODE,L,N,N1,N2,NCN,M
      REAL      GR,GRMAX,WS,WSMIN
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C-----LOOP AROUND ELEMENTS
      DO 60 L=1,NE
        M=LMAT(L)
        IF(M.EQ.0) GOTO 60
        CALL LNODES(L,NCN)
        KODE=0
C
C------ IF ELEMENT IS ON ARE THERE ANY DRY NODES?
        IF(M.GT.0) THEN
          DO 20 K=1,NCN
            N=IABS(NODE(K))
            IF(VEL(3,N).LT.ZIP) KODE=1
20        CONTINUE
C
C------ If element is turned off, set values at boundary nodes to zero.
        IF(KODE.EQ.1) THEN
          K=MIN(8,NCN)
          N=IABS(NODE(K))
          IDLAST=NPID(N)
          DO 26 K=2,NCN,2
            N1=IABS(NODE(K-1))
            N2=IABS(NODE(K))
            IF(NPID(N2).EQ.1 .AND. IDLAST.EQ.1) THEN
              IDLAST=NPID(N2)
              NPID(N1)=0
              NPID(N2)=0
              N1=IABS(NODE(K-1))
              DO 22 I=1,NDF
                VEL(I,N1)=0.
                VEL(I,N2)=0.
22            CONTINUE
            ELSEIF(NPID(N2).EQ.1) THEN
              IDLAST=NPID(N2)
              NPID(N2)=0
              DO 24 I=1,NDF
                VEL(I,N2)=0.
24            CONTINUE
            ELSE
              IDLAST=NPID(N2)
              NPID(N1)=1
              NPID(N2)=1
            ENDIF
26        CONTINUE
C
C-------- Take care of center node
          IF(NCN.EQ.9) THEN
            N=IABS(NODE(NCN))
            DO 28 I=1,NDF
              VEL(I,N)=0.
28          CONTINUE
          ENDIF
        ENDIF
C
C------ IF ELEMENT IS OFF SHOULD IT BE TURNED BACK ON?
        ELSE
          WSMIN=-VOID
          GRMAX=VOID
          DO 30 K=1,NCN
            N=IABS(NODE(K))
            IF(VEL(3,N).GT.ZIP) THEN
              WS=VEL(3,N)+GRND(N)
              IF(WS.LT.WSMIN) WSMIN=WS
            ELSE
              GR=GRND(N)
              IF(GR.GT.GRMAX) GRMAX=GR
            ENDIF
30        CONTINUE
          IF(WSMIN.LT.-VOID .AND. WSMIN.GT.GRMAX+DEPTOL) THEN
            KODE=1
            DO 40 K=1,NCN
              N=IABS(NODE(K))
              WS=VEL(3,N)+GRND(N)
              IF(WS.LT.WSMIN) VEL(3,N)=WSMIN-GRND(N)
40          CONTINUE
          ENDIF
C
C-------- Set all dry node depths to WSMIN-GRND(N) if element is to be
C         turned back on.
          IF(KODE.EQ.1) THEN
            DO 42 K=1,NCN
              N=IABS(NODE(K))
              VEL(3,N)=WSMIN-GRND(N)
42          CONTINUE
          ENDIF
C
C-------- Update node point identification codes
        ENDIF
C
C------ IF ELEMENT HAS SWITCHED THEN SET BC CODES TO ZERO
        IF(KODE.EQ.1) THEN
          LMAT(L)=-M
          DO 50 K=1,NCN
            N=IABS(NODE(K))
            NFIX(N)=0
50        CONTINUE
        ENDIF
60    CONTINUE
      RETURN
      END
C
C
C
      FUNCTION LAREA(L)
C
C---- LAREA ----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES THE AREA OF A 2-DIMENSIONAL ELEMENT.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER
C
C  SUBPROGRAM(S) CALLED:
C    SFLOAD, SHPFNC
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   I,K,L,N,NCN,NGP,NR
      REAL      LAREA,DETJ,XR,YR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      LAREA = 0.
      IF(LMAT(L).EQ.0) RETURN
C
C---- LOAD PROPER SHAPE FUNCTIONS
      CALL SFLOAD(L,NCN,NGP,0)
C
C---- FILL LOCAL (ELEMENT) COORDINATE ARRAYS
      NR=IABS(NOP(L,1))
      XR=CORD(NR,1)
      YR=CORD(NR,2)
      DO 10 K=2,NCN
        N=IABS(NOP(L,K))
        XL(K)=CORD(N,1)-XR
        YL(K)=CORD(N,2)-YR
10    CONTINUE
C
C---- LOOP AROUND GAUSS POINTS
      DO 20 I=1,NGP
        CALL SHPFNC(NCN,I,DETJ)
        LAREA=LAREA+WAITX(I)*DETJ
20    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE LVALUE(L,NCN)
C
C---- LVALUE ---------------------------------------------------------S
C
C  PURPOSE:
C    FILLS LOCAL (ELEMENT) VALUE ARRAYS.
C
C  INPUT ARGUMENT(S):
C    L - ELEMENT NUMBER.
C    NCN - TOTAL NUMBER OF ELEMENT NODES.
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,NCN
C
C---- Declarations (local)
      INTEGER   K,N,NFIX3,NR
      REAL      FAC,XR,YR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      NR=IABS(NOP(L,1))
      XR=CORD(NR,1)
      YR=CORD(NR,2)
      DO 10 K=1,NCN
        N=IABS(NOP(L,K))
        UL(K)=VEL(1,N)
        VL(K)=VEL(2,N)
        PL(K)=VEL(3,N)
        HL(K)=PL(K)
        UDOTL(K)=VDOT(1,N)
        VDOTL(K)=VDOT(2,N)
        HDOTL(K)=VDOT(3,N)
        GL(K)=GRND(N)
        CL(K)=CEIL(N)
        XL(K)=CORD(N,1)-XR
        YL(K)=CORD(N,2)-YR
C
C------ CHECK FOR PRESSURE FLOW
        IF(LMAT(L).GT.1000 .AND. PL(K).GT.CL(K)-GL(K)) THEN
          HL(K)=CL(K)-GL(K)
          HDOTL(K)=0.
          NFIX3=MOD(NFIX(N)/100,10)
          IF(NFIX3.EQ.8) THEN
            FAC=PL(K)/HL(K)
            UL(K)=FAC*UL(K)
            VL(K)=FAC*VL(K)
          ENDIF
        ENDIF
10    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE MCHECK(M,IDS,MAXM,IO2,IER,ERR)
C
C---- MCHECK ---------------------------------------------------------S
C
C  PURPOSE:
C    CHECKS FOR A VALID PROPERTY TYPE NUMBER.  THE PROPERTY TYPE
C    NUMBER MUST BE GREATER THAN ZERO AND LESS THAN OR EQUAL TO THE
C    MAXIMUM PROPERTY TYPE DIMENSION, MAXM.  IF AN INVALID PROPERTY
C    TYPE NUMBER IS FOUND, THE ERROR COUNT IS INCREMENTED BY ONE
C    AND AN ERROR MESSAGE IS PRINTED.
C
C  INPUT ARGUMENT(S):
C    M    - PROPERTY TYPE NUMBER TO BE CHECKED
C    IDS    - DATA SET FROM WHICH THE NODE NUMBER WAS READ
C    IO2    - UNIT NUMBER OF LISTING FILE
C    MAXM   - MAXIMUM PROPERTY TYPE DIMENSION (NUMBER)
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C    ERR    - LOGICAL ERROR INDICATOR
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   M,MAXM,IO2,IER
      CHARACTER IDS*(*)
      LOGICAL   ERR
C
C---- Declarations (local)
      INTEGER   LENCHR
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #235 - PROPERTY SET NUMBER ',I5,' IN THE ',
     # (A),' DATA SET IS OUT OF RANGE.')
C
      ERR=.FALSE.
      IF(M.LT.1) ERR=.TRUE.
      IF(M.GT.MAXM) ERR=.TRUE.
      IF(ERR) THEN
        IER=IER+1
        WRITE(IO2,2000) M,IDS(1:LENCHR(IDS))
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE MIDSID
C
C---- MIDSID ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS MISSING GROUND SURFACE AND CEILING ELEVATIONS TO MIDSIDE
C    AND CENTER NODES.  THESE VALUES ARE LINEARLY INTERPOLATED FROM THE
C    ELEMENT CORNER NODES.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   K,L,N,NC,NCN,NM,N1,N2,N3
      REAL      G,C,X,Y
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C-----SET NPID ARRAY TO ZERO
      DO 10 N=1,NP
        NPID(N)=0
10    CONTINUE
C
C---- LOOP AROUND ELEMENT SIDES
      DO 40 L=1,NE
        IF(LMAT(L).LT.1) GOTO 40
        CALL LNODES(L,NCN)
        IF(NCN.GT.8) NCN=8
C------ MIDSIDE NODES
        DO 20 K=2,NCN,2
          N2=NODE(K)
          IF(NPID(N2).EQ.0) THEN
            NPID(N2)=1
            N1=NODE(K-1)
            N3=MOD(K+1,NCN)
            N3=NODE(N3)
            GRND(N2)=0.5*(GRND(N1)+GRND(N3))
            CEIL(N2)=0.5*(CEIL(N1)+CEIL(N3))
            IF(CORD(N2,1).LT.VDX) THEN
              CORD(N2,1)=0.5*(CORD(N1,1)+CORD(N3,1))
              CORD(N2,2)=0.5*(CORD(N1,2)+CORD(N3,2))
            ENDIF
          ENDIF
20      CONTINUE
C------ CENTER NODE FOR LAGRANGIAN QUADRILATERAL
        IF(NODE(9).NE.0) THEN
          G=0.
          C=0.
          X=0.
          Y=0.
          DO 30 K=2,NCN,2
            NC=NODE(K-1)
            NM=NODE(K)
            X=X-0.25*CORD(NC,1)+0.50*CORD(NM,1)
            Y=Y-0.25*CORD(NC,2)+0.50*CORD(NM,2)
            G=G+0.25*GRND(NC)
            C=C+0.25*CEIL(NC)
30        CONTINUE
          N=NODE(9)
          CORD(N,1)=X
          CORD(N,2)=Y
          GRND(N)=G
          CEIL(N)=C
        ENDIF
40    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE NCHECK(N,IDS,MAXP,IO2,IER,ERR)
C
C---- NCHECK ---------------------------------------------------------S
C
C  PURPOSE:
C    CHECKS FOR A VALID NODE NUMBER.  THE NODE NUMBER MUST BE GREATER
C    THAN ZERO AND LESS THAN OR EQUAL TO THE MAXIMUM NODAL ARRAY
C    DIMENSION, MAXP.  IF AN INVALID NODE NUMBER IS FOUND TO BE
C    OUTSIDE THIS RANGE, THE ERROR COUNT IS INCREMENTED BY ONE AND A
C    MESSAGE IS PRINTED.
C
C  INPUT ARGUMENT(S):
C    N      - NODE NUMBER TO BE CHECKED
C    IDS    - DATA SET FROM WHICH THE NODE NUMBER WAS READ
C    MAXP   - MAXIMUM NODE DIMENSION (NUMBER)
C    IO2    - UNIT NUMBER OF LISTING FILE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C    ERR    - LOGICAL ERROR INDICATOR
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   N,MAXP,IO2,IER
      CHARACTER IDS*(*)
      LOGICAL   ERR
C
C---- Declarations (local)
      INTEGER   LENCHR
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #230 - NODE NUMBER ',I5,' IN THE ',(A),
     # ' DATA SET IS OUT OF RANGE.')
C
      ERR=.FALSE.
      IF(N.LT.1) ERR=.TRUE.
      IF(N.GT.MAXP) ERR=.TRUE.
      IF(ERR) THEN
        IER=IER+1
        WRITE(IO2,2000) N,IDS(1:LENCHR(IDS))
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE NPIDEN
C
C---- NPIDEN ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS NODE POINT IDENTIFICATION VALUES.  A VALUE OF 1
C    INDICATES A BOUNDARY NODE.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES
C
C--------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   K,KHI,L,N,NCN,N1,N2,N3
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- CLEAR NPID ARRAY
      DO 10 N=1,NP
        NPID(N)=0
10    CONTINUE
C
C---- COMPUTE NODAL IDENTIFICATION ARRAY VALUES
      DO 30 L=1,NE
        IF(LMAT(L).GT.0) THEN
          CALL LNODES(L,NCN)
          KHI=MIN(NCN,8)
          DO 20 K=1,KHI
            N=IABS(NODE(K))
            NPID(N)=NPID(N)+1
20        CONTINUE
        ENDIF
30    CONTINUE
      DO 50 L=1,NE
        IF(LMAT(L).GT.0) THEN
          CALL LNODES(L,NCN)
          KHI=MIN(NCN,8)
          DO 40 K=2,KHI,2
            N2=IABS(NODE(K))
            IF(NPID(N2).EQ.1) THEN
              N1=IABS(NODE(K-1))
              N3=MOD(K+1,KHI)
              N3=IABS(NODE(N3))
              NPID(N1)=1
              NPID(N3)=1
            ENDIF
40        CONTINUE
        ENDIF
50    CONTINUE
      DO 32 L=1,NE
        IF(LMAT(L).GT.0) THEN
          CALL LNODES(L,NCN)
          DO 31 K=1,NCN
            N=IABS(NODE(K))
            IF(NPID(N).NE.1) NPID(N)=0
31        CONTINUE
        ENDIF
32    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE OUTPUT(IDP)
C
C---- OUTPUT ---------------------------------------------------------S
C
C  PURPOSE:
C    OUTPUTS COMPUTED RESULTS.  RESULTS ARE WRITTEN TO THE
C    RESTART/RECOVERY FILE, IF ONE IS USED, AFTER EVERY ITERATION.
C    RESULTS ARE WRITTEN TO THE SOLUTION OUTPUT FILE AT THE END OF EVERY
C    COMPUTATION INTERVAL.  RESULTS ARE PRINTED AFTER SELECTED ITER-
C    ATIONS AND AT THE END OF EVERY COMPUTATION INTERVAL.
C
C  INPUT ARGUMENT(S):
C    IDP - OUTPUT CODE
C
C                 1 - WRITE TO RSTART/RECOVERY FILE ONLY.
C      IDP =      2 - PRINT INTERMEDIATE RESULTS.
C                 3 - PRINT FINAL RESULTS.
C
C  SUBPROGRAM(S) CALLED:
C    WEIRQ, CULVQ
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IDP
C
C---- Declarations (local)
      INTEGER   K,KK,J,JINT,N,N1,N2
      REAL      DEPTH,WS,WS1,WS2,QWER,DQDH,CF,QCUL,XOUT
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      DIMENSION XOUT(5,5)
C
C---- OUTPUT FORMATS
2000  FORMAT(1H+,79(' ')/1H+,'Writing (',(A),')...')
2010  FORMAT(//3(4X,'NODE',3X,'X-VEL',3X,'Y-VEL',3X,'DEPTH',6X,'WSEL '))
2011  FORMAT(3(10X,'(FT/S)',2X,'(FT/S)',4X,'(FT)',6X,'(FT) ')/)
2012  FORMAT(3(11X,'(M/S)',3X,'(M/S)',5X,'(M)',7X,'(M) ')/)
2040  FORMAT(/58X,'*** WEIR FLOW ***'//
     # 34X,'WEIR',5X,'NODE1',5X,'NODE2',5X,'WSEL1',5X,'WSEL2',7X,'Q',
     # 9X,'SUB'/
     # 35X,'NO.',26X,'(FT)',6X,'(FT)',3X,'(FT3/S)',6X,'FACT'/)
2050  FORMAT(/56X,'*** CULVERT FLOW ***'//
     # 39X,'CULV',5X,'NODE1',5X,'NODE2',5X,'WSEL1',5X,'WSEL2',7X,'Q'/
     # 40X,'NO.',26X,'(FT)',6X,'(FT)',3X,'(FT3/S)'/)
2060  FORMAT(/' NOTE -- AN ASTERISK (*) FOLLOWING WSEL INDICATES ',
     # 'PRESSURE FLOW.')
2110  FORMAT(//2(1X,'NODE',3X,'X-VEL',3X,'Y-VEL',3X,'DEPTH',6X,'WSEL '))
2111  FORMAT(2(7X,'(FT/S)',2X,'(FT/S)',4X,'(FT)',6X,'(FT) ')/)
2112  FORMAT(2(8X,'(M/S)',3X,'(M/S)',5X,'(M)',7X,'(M) ')/)
2140  FORMAT(/32X,'*** WEIR FLOW ***'//
     # 8X,'WEIR',5X,'NODE1',5X,'NODE2',5X,'WSEL1',5X,'WSEL2',7X,'Q',
     # 9X,'SUB'/
     # 9X,'NO.',26X,'(FT)',6X,'(FT)',3X,'(FT3/S)',6X,'FACT'/)
2150  FORMAT(/30X,'*** CULVERT FLOW ***'//
     # 13X,'CULV',5X,'NODE1',5X,'NODE2',5X,'WSEL1',5X,'WSEL2',7X,'Q'/
     # 14X,'NO.',26X,'(FT)',6X,'(FT)',3X,'(FT3/S)'/)
2200  FORMAT(F10.3,2I10)
2201  FORMAT(I10,3F10.3,3G10.3)
2210  FORMAT(3(3X,I5,3F8.3,F10.3,A1))
2211  FORMAT(2(I5,3F8.3,F10.3,A1))
2220  FORMAT(28X,3I10,4F10.3)
2221  FORMAT(2X,3I10,4F10.3)
2230  FORMAT(33X,3I10,3F10.3)
2231  FORMAT(7X,3I10,3F10.3)
2240  FORMAT(F10.3,I10)
2241  FORMAT(I10,3F10.3,3G10.3)
C
C---- Write results to the restart/recovery file if IRSRC <> 0
      IF(IRSRC.NE.0) CALL RSRC
      IF(IDP.EQ.1) GOTO 999
C
C---- PRINT RESULTS OF CURRENT ITERATION
      IF(SCREEN) WRITE(6,2000) PRNTFILE(1:LENCHR(PRNTFILE))
      IF(WIDE) THEN
        JINT=(NP+2)/3
        WRITE(IO2,2010)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2011)
        ELSE
          WRITE(IO2,2012)
        ENDIF
      ELSE
        JINT=(NP+1)/2
        WRITE(IO2,2110)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2111)
        ELSE
          WRITE(IO2,2112)
        ENDIF
      ENDIF
      DO 20 J=1,JINT
        KK=0
        DO 10 N=J,NP,JINT
          KK=KK+1
          DEPTH=VEL(3,N)
          XOUT(1,KK)=REAL(N)
          IF(DEPTH.GT.0.) THEN
            XOUT(2,KK)=VEL(1,N)
            XOUT(3,KK)=VEL(2,N)
            XOUT(4,KK)=DEPTH
            WS=DEPTH+GRND(N)
            XOUT(5,KK)=WS
            FLAG(KK)=' '
            IF(ABS(CEIL(N)).GT.0. .AND. WS.GT.CEIL(N)) FLAG(KK)='*'
          ELSE
            XOUT(2,KK)=0.
            XOUT(3,KK)=0.
            XOUT(4,KK)=0.
            XOUT(5,KK)=0.
            FLAG(KK)=' '
          ENDIF
10      CONTINUE
        IF(WIDE) THEN
          WRITE(IO2,2210) (INT(XOUT(1,K)),
     #    XOUT(2,K),XOUT(3,K),XOUT(4,K),XOUT(5,K),FLAG(K),K=1,KK)
        ELSE
          WRITE(IO2,2211) (INT(XOUT(1,K)),
     #    XOUT(2,K),XOUT(3,K),XOUT(4,K),XOUT(5,K),FLAG(K),K=1,KK)
        ENDIF
20    CONTINUE
      WRITE(IO2,2060)
      IF(IDP.EQ.2) GOTO 999
C
C---- PRINT WEIR AND CULVERT FLOW RESULTS
      IF(NWER.GT.0) THEN
        IF(WIDE) THEN
          WRITE(IO2,2040)
        ELSE
          WRITE(IO2,2140)
        ENDIF
        DO 30 K=1,NWER
          QWER=0.0
          N1=IABS(NOPW(K,1))
          N2=IABS(NOPW(K,2))
          WS1=VEL(3,N1)+GRND(N1)
          WS2=0.0
          IF(N2.GT.0) WS2=VEL(3,N2)+GRND(N2)
          CALL WEIRQ(K,WS1,WS2,QWER,DQDH,CF)
          IF(WIDE) THEN
            WRITE(IO2,2220) K,N1,N2,WS1,WS2,QWER,CF
          ELSE
            WRITE(IO2,2221) K,N1,N2,WS1,WS2,QWER,CF
          ENDIF
30      CONTINUE
      ENDIF
      IF(NCUL.GT.0) THEN
        IF(WIDE) THEN
          WRITE(IO2,2050)
        ELSE
          WRITE(IO2,2150)
        ENDIF
        DO 40 K=1,NCUL
          N1=IABS(NOPC(K,1))
          N2=IABS(NOPC(K,2))
          WS1=VEL(3,N1)+GRND(N1)
          WS2=CELV(K)
          IF(N2.GT.0) WS2=VEL(3,N2)+GRND(N2)
          CALL CULVQ(K,WS1,WS2,QCUL,DQDH)
          IF(WIDE) THEN
            WRITE(IO2,2230) K,N1,N2,WS1,WS2,QCUL
          ELSE
            WRITE(IO2,2231) K,N1,N2,WS1,WS2,QCUL
          ENDIF
40      CONTINUE
      ENDIF
C
C---- WRITE RESULTS OF CURRENT TIME STEP TO THE SOLUTION OUTPUT FILE
      IF(SCREEN) WRITE(6,2000) FLOWFILE(1:LENCHR(FLOWFILE))
      IF(ISOUT.LT.0) THEN
        WRITE(IO94) TET,NP
        WRITE(IO94) ((VEL(K,N),K=1,3),(VDOT(K,N),K=1,3),N=1,NP)
      ELSEIF(ISOUT.GT.0) THEN
        WRITE(IO94,2240) TET,NP
        WRITE(IO94,2241) (N,(VEL(K,N),K=1,3),(VDOT(K,N),K=1,3),N=1,NP)
      ENDIF
999   RETURN
      END
C
C
C
      SUBROUTINE RSRC
C
C---- RSRC ------------------------------------------------------------S
C
C  PURPOSE:
C    WRITES RESULTS OF CURRENT ITERATION TO THE RESTART/RECOVERY FILE.
C
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   LENCHR
      LOGICAL   EXIST
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(1H+,79(' ')/1H+,'Writing (',(A),')...')
2200  FORMAT(F10.3,2I10)
2201  FORMAT(I10,3F10.3,3G10.3)
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) RSRCFILE(1:LENCHR(RSRCFILE))
C
C---- Open and delete the file if it already exists
      INQUIRE(FILE=RSRCFILE,EXIST=EXIST)
      IF(EXIST) THEN
        OPEN(IO97,FILE=RSRCFILE)
        CLOSE(IO97,STATUS='DELETE')
      ENDIF
C
C---- Write to a formatted file if IRSRC > 0
      IF(IRSRC.GT.0) THEN
        OPEN(IO97,FILE=RSRCFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #  FORM='FORMATTED')
        WRITE(IO97,2200) TET,NP,NITSV
        WRITE(IO97,2201) (N,(VEL(K,N),K=1,3),(VDOT(K,N),K=1,3),N=1,NP)
C
C---- Write to an unformatted file if IRSRC < 0
      ELSEIF(IRSRC.LT.0) THEN
        OPEN(IO97,FILE=RSRCFILE,STATUS='NEW',ACCESS='SEQUENTIAL',
     #  FORM='UNFORMATTED')
        REWIND IO97
        WRITE(IO97) TET,NP,NITSV
        WRITE(IO97) ((VEL(K,N),K=1,3),(VDOT(K,N),K=1,3),N=1,NP)
      ENDIF
C
C---- Close and keep the file
      CLOSE(IO97,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE PRECMP(IER)
C
C---- PRECMP ---------------------------------------------------------S
C
C  PURPOSE:
C    PERFORMS PRELIMINARY COMPUTATIONS.
C
C  INPUT ARGUMENT(S):
C    IER - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C  SUBPROGRAM(S) CALLED:
C    MIDSID, TAUWND, FORDAR, HIGHOI, VNCINP, LONOFF, NPIDEN, BCODE1,
C    BCODE2, BANGLE, TBLOCK, LNODES, FORMFD
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   K,L,N,NCN,NFIX3
      REAL      CS,SIGMAX,SIGMAY
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
 2000 FORMAT(1H+,79(' ')/1H+,'Computing (preliminaries)...')
 2045 FORMAT(' ***ERROR #440 - SPECIFIED DEPTH < 0 AT NODE ',I5,'.')
 2050 FORMAT(' ***ERROR #450 - ASSEMBLY SEQUENCE',I5,' IS ASSIGNED ',
     # 'TO ELEMENTS ',I5,' AND ',I5,'.')
 2055 FORMAT(' ***ERROR #190 - NODES ',I5,' AND ',I5,' HAVE THE SAME',
     # ' COORDINATES.')
 2065 FORMAT(' ***ERROR #210 - INDETERMINANT BOUNDARY ANGLE AT NODE',
     # I5,'.')
 2070 FORMAT(' ***ERROR #445 - CEILING IS LESS THAN GROUND AT NODE ',
     # I5,' IN ELEMENT ',I5,'.')
 2080 FORMAT(//
     # 54X,'*** SYSTEM PARAMETERS ***'//
     # 46X,'MAXIMUM NODE NUMBER.........: ',I5/
     # 46X,'MAXIMUM ELEMENT NUMBER......: ',I5/
     # 46X,'MAXIMUM PROPERTY SET NUMBER.: ',I5/
     # 46X,'NUMBER OF FLOW CHECK LINES..: ',I5/
     # 46X,'NUMBER OF WEIR SEGMENTS.....: ',I5/
     # 46X,'NUMBER OF CULVERTS..........: ',I5/
     # 46X,'WATER DENSITY...............: ',G10.3/
     # 46X,'CORIOLIS PARAMETER..........: ',G10.3/
     # 46X,'AIR DENSITY.................: ',G10.3/
     # 46X,'WIND STRESS (TAU-X).........: ',G10.3/
     # 46X,'WIND STRESS (TAU-Y).........: ',G10.3)
2180  FORMAT(//
     # 27X,'*** SYSTEM PARAMETERS ***'//
     # 20X,'MAXIMUM NODE NUMBER.........: ',I5/
     # 20X,'MAXIMUM ELEMENT NUMBER......: ',I5/
     # 20X,'MAXIMUM PROPERTY SET NUMBER.: ',I5/
     # 20X,'NUMBER OF FLOW CHECK LINES..: ',I5/
     # 20X,'NUMBER OF WEIR SEGMENTS.....: ',I5/
     # 20X,'NUMBER OF CULVERTS..........: ',I5/
     # 20X,'WATER DENSITY...............: ',G10.3/
     # 20X,'CORIOLIS PARAMETER..........: ',G10.3/
     # 20X,'AIR DENSITY.................: ',G10.3/
     # 20X,'WIND STRESS (TAU-X).........: ',G10.3/
     # 20X,'WIND STRESS (TAU-Y).........: ',G10.3)
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000)
C
C---- Compute the Coriolis parameter (OMEGA > 0 IN NORTHERN HEMISPHERE)
      IF(OMEGA.GT.0. .OR. OMEGA.LT.0.) OMEGA=1.458E-04*SIN(DTOR*OMEGA)
C
C---- Compute general wind stresses
      IF(IWIND.GT.0) THEN
C------ Convert wind velocity from ft/s to m/s if U.S. units are used
        IF(IUNIT.EQ.0) WVEL=0.3048*WVEL
C------ Compute wind shear-stress coefficient.
C       Remember WVMIN is already in meters per second.
        IF(WVEL.LT.WVMIN) THEN
          CS=0.001*CSURF1
        ELSE
          CS=0.001*(CSURF1+CSURF2*(WVEL-WVMIN))
        ENDIF
C------ Compute wind shear stresses in x and y directions.
        SIGMAX=CS*ROAIR*COS(DTOR*WDIR)*WVEL*WVEL
        SIGMAY=CS*ROAIR*SIN(DTOR*WDIR)*WVEL*WVEL
      ELSE
        SIGMAX=0.
        SIGMAY=0.
      ENDIF
C
C---- Print general system specifications.
      IF(WIDE) THEN
        WRITE(IO2,2080) NP,NE,NMAT,NCL,NWER,NCUL,ROWAT,OMEGA,ROAIR,
     #  SIGMAX,SIGMAY
      ELSE
        WRITE(IO2,2180) NP,NE,NMAT,NCL,NWER,NCUL,ROWAT,OMEGA,ROAIR,
     #  SIGMAX,SIGMAY
      ENDIF
C
C---- Initialize arrays and set depth spec
      DO 10 N=1,NP
        NFIX3=MOD(NFIX(N)/100,10)
        IF(NFIX3.EQ.1 .OR. NFIX3.EQ.2) THEN
          SPEC(N,3)=SPEC(N,3)-GRND(N)
          IF(SPEC(N,3).LE.0.) THEN
            IER=IER+1
            WRITE(IO2,2045) N
          ENDIF
        ENDIF
10    CONTINUE
C
C---- COMPUTE UNASSIGNED MIDSIDE- AND CENTER-NODE VALUES
      CALL MIDSID
C
C---- CHECK CEILING-OPTION ELEMENTS
      DO 30 L=1,NE
        IF(IABS(LMAT(L)).GT.1000) THEN
          CALL LNODES(L,NCN)
          DO 20 K=1,NCN
            N=NODE(K)
            IF(CEIL(N).LE.GRND(N)) THEN
              IER=IER+1
              WRITE(IO2,2070) N,L
            ENDIF
20        CONTINUE
        ENDIF
30    CONTINUE
C
C---- COMPUTE WIND STRESSES
      CALL TAUWND(SIGMAX,SIGMAY)
C
C---- FILL FRONTAL ORDER ARRAY
      CALL FORDAR(IER)
C
C---- ASSIGN HIGHER INTEGRATION ORDER TO CURVE-SIDED ELEMENTS
      IF(IXINT.GT.0) CALL HIGHOI
C
C---- INITIALIZE VARIABLES NOT COVERED BY INPUT
      CALL VNCINP
      RETURN
      END
C
C
C
      SUBROUTINE PRELNP
C
C---- PRELNP ---------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS ELEMENT AND NODE DATA.
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER    J,JINT,K,L,N
      REAL       LAREA
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(1H+,79(' ')/1H+,'Writing (',(A),')...')
2010  FORMAT(//56X,'*** ELEMENT DATA ***'//
     # 1X,2('ELMT',5X,'NODES (COUNTERCLOCKWISE)',16X,'PROP',
     # 7X,'AREA '))
2011  FORMAT(1X,2(1X,'NO',3X,'1',4X,'2',4X,'3',4X,'4',4X,'5',4X,'6',
     # 4X,'7',4X,'8',4X,'9',2X,'CODE',6X,'(FT2) ')/)
2012  FORMAT(1X,2(1X,'NO',3X,'1',4X,'2',4X,'3',4X,'4',4X,'5',4X,'6',
     # 4X,'7',4X,'8',4X,'9',2X,'CODE',8X,'(M2)')/)
2013  FORMAT(1X,2(I4,9I5,I4,F12.2))
2020  FORMAT(//58X,'*** NODE DATA ***'//
     # 2(3X,'NODE',5X,'X-COORD',5X,'Y-COORD',6X,'GRND',6X,'CEIL',
     # 5X,'ANGLE'))
2021  FORMAT(2(4X,'NO.',8X,'(FT)',8X,'(FT)',6X,'(FT)',6X,'(FT)',
     # 7X,'DEG')/)
2022  FORMAT(2(4X,'NO.',9X,'(M)',9X,'(M)',7X,'(M)',7X,'(M)',
     # 7X,'DEG')/)
2023  FORMAT(2(1X,I6,2F12.2,3F10.2))
2110  FORMAT(//30X,'*** ELEMENT DATA ***'//
     # 7X,'ELMT',5X,'NODES (COUNTERCLOCKWISE)',16X,'PROP',
     # 7X,'AREA ')
2111  FORMAT(9X,'NO',3X,'1',4X,'2',4X,'3',4X,'4',4X,'5',4X,'6',4X,'7',
     # 4X,'8',4X,'9',1X,'CODE',6X,'(FT2) '/)
2112  FORMAT(9X,'NO',3X,'1',4X,'2',4X,'3',4X,'4',4X,'5',4X,'6',4X,'7',
     # 4X,'8',4X,'9',1X,'CODE',8X,'(M2)'/)
2113  FORMAT(7X,I4,9I5,I4,F12.2)
2120  FORMAT(//32X,'*** NODE DATA ***'//
     # 12X,'NODE',5X,'X-COORD',5X,'Y-COORD',6X,'GRND',6X,'CEIL',
     # 5X,'ANGLE')
2121  FORMAT(13X,'NO.',8X,'(FT)',8X,'(FT)',6X,'(FT)',6X,'(FT)',
     # 7X,'DEG'/)
2122  FORMAT(13X,'NO.',9X,'(M)',9X,'(M)',7X,'(M)',7X,'(M)',
     # 7X,'DEG'/)
2123  FORMAT(10X,I6,2F12.2,3F10.2)
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) PRNTFILE(1:LENCHR(PRNTFILE))
C
C---- PRINT ELEMENT DATA
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2010)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2011)
        ELSE
          WRITE(IO2,2012)
        ENDIF
        JINT=(NE+1)/2
        DO 10 J=1,JINT
          WRITE(IO2,2013) (L,(IABS(NOP(L,K)),K=1,9),LMAT(L),LAREA(L),
     #    L=J,NE,JINT)
10      CONTINUE
      ELSE
        WRITE(IO2,2110)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2111)
        ELSE
          WRITE(IO2,2112)
        ENDIF
        DO 20 L=1,NE
          WRITE(IO2,2113) L,(IABS(NOP(L,K)),K=1,9),LMAT(L),LAREA(L)
20      CONTINUE
      ENDIF
C
C---- NODE DATA
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2020)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2021)
        ELSE
          WRITE(IO2,2022)
        ENDIF
        JINT=(NP+1)/2
      ELSE
        WRITE(IO2,2120)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2121)
        ELSE
          WRITE(IO2,2122)
        ENDIF
        JINT=NP
      ENDIF
      DO 30 J=1,JINT
        IF(WIDE) THEN
          WRITE(IO2,2023) (N,(CORD(N,K),K=1,2),GRND(N),CEIL(N),
     #    RTOD*ALFA(1,N),N=J,NP,JINT)
        ELSE
          WRITE(IO2,2123) J,(CORD(J,K),K=1,2),GRND(J),CEIL(J),
     #    RTOD*ALFA(1,J)
        ENDIF
30    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE PRBDIC
C
C---- PRBDIC ---------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS BOUNDARY AND INITIAL CONDITION DATA
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER    J,JINT,K,L,N
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(1H+,79(' ')/1H+,'Writing (',(A),')...')
2030  FORMAT(//51X,'*** BOUNDARY SPECIFICATIONS ***'//
     # 1X,2(3X,'NODE',2X,'CODE',4X,'X-SPEC',4X,'Y-SPEC',4X,'Z-SPEC',
     # 5X,'TAUS-X',5X,'TAUS-Y'))
2031  FORMAT(1X,2(1X,2I6,3F10.3,2E11.3))
2040  FORMAT(//53X,'*** INITIAL CONDITIONS ***'//
     # 1X,2(3X,'NODE',3X,'X-VEL',3X,'Y-VEL',3X,'DEPTH',6X,'WSEL',
     # 3X,'U-DOT',3X,'V-DOT',3X,'H-DOT'))
2041  FORMAT(1X,2(4X,'NO.',2X,'(FT/S)',2X,'(FT/S)',4X,'(FT)',6X,'(FT)',
     # 1X,'(FT/S2)',1X,'(FT/S2)',2X,'(FT/S)')/)
2042  FORMAT(1X,2(4X,'NO.',3X,'(M/S)',3X,'(M/S)',5X,'(M)',7X,'(M)',
     # 2X,'(M/S2)',2X,'(M/S2)',3X,'(M/S)')/)
2043  FORMAT(1X,2(I7,3F8.3,F10.3,3F8.4))
2045  FORMAT(' ***ERROR #440 - SPECIFIED DEPTH < 0 AT NODE ',I5,'.')
2130  FORMAT(//25X,'*** BOUNDARY SPECIFICATIONS ***'//
     # 11X,'NODE',2X,'CODE',4X,'X-SPEC',4X,'Y-SPEC',4X,'Z-SPEC',
     # 4X,'TAUS-X',4X,'TAUS-Y'/)
2131  FORMAT(9X,2I6,3F10.3,2E10.3)
2140  FORMAT(//27X,'*** INITIAL CONDITIONS ***'//
     # 10X,'NODE',3X,'X-VEL',3X,'Y-VEL',3X,'DEPTH',6X,'WSEL',
     # 3X,'U-DOT',3X,'V-DOT',3X,'H-DOT')
2141  FORMAT(11X,'NO.',2X,'(FT/S)',2X,'(FT/S)',4X,'(FT)',6X,'(FT)',
     # 1X,'(FT/S2)',1X,'(FT/S2)',2X,'(FT/S)'/)
2142  FORMAT(11X,'NO.',3X,'(M/S)',3X,'(M/S)',5X,'(M)',7X,'(M)',
     # 2X,'(M/S2)',2X,'(M/S2)',3X,'(M/S)'/)
2143  FORMAT(8X,I6,3F8.3,F10.3,3F8.4)
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) PRNTFILE(1:LENCHR(PRNTFILE))
C
C---- BOUNDARY CONDITION AND WIND DATA
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2030)
        JINT=(NP+1)/2
      ELSE
        WRITE(IO2,2130)
        JINT=NP
      ENDIF
      DO 10 J=1,JINT
        IF(WIDE) THEN
          WRITE(IO2,2031) (N,NFIX(N),(SPEC(N,K),K=1,3),
     #    (SIGMA(N,K),K=1,2),N=J,NP,JINT)
        ELSE
          WRITE(IO2,2131) J,NFIX(J),(SPEC(J,K),K=1,3),
     #    (SIGMA(J,K),K=1,2)
        ENDIF
10    CONTINUE
C
C---- INITIAL VELOCITIES, DEPTHS, ELEVATIONS, AND TIME DERIVATIVES
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        JINT=(NP+1)/2
        WRITE(IO2,2040)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2041)
        ELSE
          WRITE(IO2,2042)
        ENDIF
      ELSE
        JINT=NP
        WRITE(IO2,2140)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2141)
        ELSE
          WRITE(IO2,2142)
        ENDIF
      ENDIF
      DO 20 J=1,JINT
        IF(WIDE) THEN
          WRITE(IO2,2043) (N,(VEL(K,N),K=1,3),VEL(3,N)+GRND(N),
     #    (VDOT(K,N),K=1,3),N=J,NP,JINT)
        ELSE
          WRITE(IO2,2143) J,(VEL(K,J),K=1,3),VEL(3,J)+GRND(J),
     #    (VDOT(K,J),K=1,3)
        ENDIF
20    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE PRLSEQ
C
C---- PRECMP ---------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS ELEMENT ASSEMBLY SEQUENCE.
C
C---------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   I,LENCHR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(1H+,79(' ')/1H+,'Writing (',(A),')...')
2090  FORMAT(//51X,'*** ELEMENT ASSEMBLY ORDER ***'//(22I6))
2190  FORMAT(//25X,'*** ELEMENT ASSEMBLY ORDER ***'//(16I5))
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) PRNTFILE(1:LENCHR(PRNTFILE))
C
C---- PRINT ELEMENT ASSEMBLY SEQUENCE
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2090) (ISEQ(I),I=1,NE)
      ELSE
        WRITE(IO2,2190) (ISEQ(I),I=1,NE)
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE QCHECK
C
C---- QCHECK ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES FLOW ACROSS A SPECIFIED SERIES OF ELEMENT SIDES. THIS
C    FLOW COMPUTATION MAY BE USED TO CHECK ON CONTINUITY OF THE MODEL
C    DURING STEADY-STATE SIMULATIONS.
C
C  SUBPROGRAM(S) CALLED:
C    FORMFD
C
C--------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   I,J,K,KHI,KK,KLO,LOFF,N,NFIX3,NR
      REAL      U,V,H,QREF,QX,QY,QTOT,QPCT,DXDT,DYDT,SFACT
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
 2000 FORMAT(1H+,79(' ')/1H+,'Computing (flow checks)...')
 2030 FORMAT(//57X,'*** FLOW CHECKS ***'//
     # 37X,'LINE',10X,'TOTAL',9X,'X-FLOW',9X,'Y-FLOW',3X,'PERCENT')
 2031 FORMAT(38X,'NO.',8X,'(FT3/S)',8X,'(FT3/S)',8X,'(FT3/S)'/)
 2032 FORMAT(38X,'NO.',9X,'(M3/S)',9X,'(M3/S)',9X,'(M3/S)'/)
 2035 FORMAT(36X,I5,1P3E15.3,0PF10.1)
 2130 FORMAT(//31X,'*** FLOW CHECKS ***'//
     # 11X,'LINE',10X,'TOTAL',9X,'X-FLOW',9X,'Y-FLOW',3X,'PERCENT')
 2131 FORMAT(12X,'NO.',8X,'(FT3/S)',8X,'(FT3/S)',8X,'(FT3/S)'/)
 2132 FORMAT(12X,'NO.',9X,'(M3/S)',9X,'(M3/S)',9X,'(M3/S)'/)
 2135 FORMAT(10X,I5,1P3E15.3,0PF10.1)
C
      IF(SCREEN) WRITE(6,2000)
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2030)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2031)
        ELSE
          WRITE(IO2,2032)
        ENDIF
      ELSE
        WRITE(IO2,2130)
        IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2131)
        ELSE
          WRITE(IO2,2132)
        ENDIF
      ENDIF
      KHI=0
C
C---- LOOP OVER FLUX LINES
      DO 110 J=1,NCL
        QX=0.
        QY=0.
        KLO=KHI+1
        DO 10 K=KLO,MAXLP
          KHI=K
          IF(LFLUXN(K).LE.0) GOTO 20
10      CONTINUE
C
C------ LOOP OVER ELEMENT SIDES
20      DO 100 KK=KLO,KHI,2
          IF(KHI-KK.LT.2) GOTO 100
          NODE(1)=LFLUXN(KK)
          NODE(2)=LFLUXN(KK+1)
          NODE(3)=IABS(LFLUXN(KK+2))
C
C-------- COMPUTE LOCAL COORDINATES
          NR=NODE(1)
          DO 30 K=2,3
            N=NODE(K)
            XL(K)=CORD(N,1)-CORD(NR,1)
            YL(K)=CORD(N,2)-CORD(NR,2)
30        CONTINUE
C
C-------- ASSIGN NODAL VARIABLES
          LOFF=0
          DO 40 K=1,3
            N=NODE(K)
            UL(K)=VEL(1,N)
            VL(K)=VEL(2,N)
            HL(K)=VEL(3,N)
            IF(HL(K).LT.ZIP) LOFF=LOFF+1
            NFIX3=MOD(NFIX(N)/100,10)
            IF(NFIX3.EQ.7 .AND. HL(K).GT.CEIL(N)-GRND(N)) THEN
              HL(K)=CEIL(N)-GRND(N)
            ENDIF
40        CONTINUE
C
C-------- CHECK FOR ELEMENT THAT IS TURNED OFF
          IF(LOFF.GT.0) GOTO 100
C-------- LOOP OVER INTEGRATION POINTS
          DO 90 I=1,4
            DYDT=0.0
            DXDT=0.0
            DO 50 K=2,3
              DYDT=DYDT+DNAL(K,I)*YL(K)
              DXDT=DXDT-DNAL(K,I)*XL(K)
50          CONTINUE
C---------- COMPUTE U, V, AND H
            U=0.0
            V=0.0
            H=0.0
            DO 60 K=1,3
              U=U+XNL(K,I)*UL(K)
              V=V+XNL(K,I)*VL(K)
60          CONTINUE
            DO 70 K=1,3,2
              H=H+XML(K,I)*HL(K)
70          CONTINUE
C---------- SUM QX AND QY
            QX=QX+WAITL(I)*U*H*DYDT
            QY=QY+WAITL(I)*V*H*DXDT
90        CONTINUE
100     CONTINUE
        QTOT=QX+QY
        IF(J.EQ.1) QREF=QTOT
        IF(ABS(QREF).LT.0.0001) QREF=1.0
        QPCT=100.0*QTOT/QREF
        IF(WIDE) THEN
          WRITE(IO2,2035) J,QTOT,QX,QY,QPCT
        ELSE
          WRITE(IO2,2135) J,QTOT,QX,QY,QPCT
        ENDIF
110   CONTINUE
999   RETURN
      END
C
C
C
      SUBROUTINE QCOEFS(
     # NCN,WEIGHT,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     # H,DHDX,DHDY,DHDT,DZBDX,DZBDY,DZCDX,DZCDY,
     # DFDH,UMEAN,TFRIC,TNU,BETA,PRESS)
C
C---- QCOEFS ----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES EQUATION COEFFICENTS AT THE ITH QUADRATURE POINT
C    AND ADDS THE WEIGHTED VALUES TO THE ELEMENT COEFFICIENT MATRIX.
C
C  INPUT ARGUMENT(S):
C    NCN    - NUMBER OF ELEMENT NODE POINTS.
C    WEIGHT - INTEGRATION WEIGHTING FACTOR.
C    U      - VELOCITY IN THE X-DIRECTION.
C    DUDX   - DERIVATIVE OF U WITH RESPECT TO X.
C    DUDY   - DERIVATIVE OF U WITH RESPECT TO Y.
C    DUDT   - DERIVATIVE OF U WITH RESPECT TO TIME.
C    V      - VELOCITY IN THE Y-DIRECTION.
C    DVDX   - DERIVATIVE OF V WITH RESPECT TO X.
C    DVDY   - DERIVATIVE OF V WITH RESPECT TO Y.
C    DVDT   - DERIVATIVE OF V WITH RESPECT TO TIME.
C    H      - WATER DEPTH.
C    DHDX   - DERIVATIVE OF H WITH RESPECT TO X.
C    DHDY   - DERIVATIVE OF H WITH RESPECT TO Y.
C    DHDT   - DERIVATIVE OF H WITH RESPECT TO TIME.
C    DZBDX  - DERIVATIVE OF ZB WITH RESPECT TO X.
C    DZBDY  - DERIVATIVE OF ZB WITH RESPECT TO Y.
C    DZCDX  - DERIVATIVE OF ZC WITH RESPECT TO X.
C    DZCDY  - DERIVATIVE OF ZC WITH RESPECT TO Y.
C    DFDH   - DERIVATIVE OF FFACT WITH RESPECT TO H.
C    UMEAN  - MEAN ABSOLUTE VELOCITY.
C    TFRIC  - FRICTION PARAMETER - FFACT/UMEAN.
C    TNU    - KINEMATIC EDDY VISCOSITY COEFFICIENT.
C    BETA   - MOMENTUM CORRECTION FACTOR.
C    PRESS  - PRESSURE FLOW IDICATOR
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NCN
      REAL      WEIGHT,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     #          H,DHDX,DHDY,DHDT,DZBDX,DZBDY,DZCDX,DZCDY,DFDH,
     #          UMEAN,TFRIC,TNU,BETA
      LOGICAL   PRESS
C
C---- Declarations (local)
      INTEGER   IH,IHU,IHV,IHH,IU,IUU,IUV,IUH,IV,IVU,IVV,IVH,J,K,NCM
      REAL      SFACT,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,
     #          DFUN,DFUNX,DFUNY,DFVN,DFVNX,DFVNY,DFHN,DFHNX,DFHNY,
     #          DFUM,DFVM,DFHM
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- COMPUTE BOTTOM SLOPE FACTOR
      SFACT=SQRT(1.+DZBDX*DZBDX+DZBDY*DZBDY)
      IF(PRESS) THEN
        SFACT=SFACT+SQRT(1.+DZCDX*DZCDX+DZCDY*DZCDY)
      ENDIF
C
C---- X-MOMENTUM-EQUATION COEFFICIENTS
      T1=TFRIC*(2.*U*U+V*V)*SFACT
      IF(ITDS.GT.0) T1=T1+(H*ALPHA)+DHDT
      T2=-2.*BETA*H*U
      T3=2.*TNU*H
      T4=-BETA*H*V
      T5=TNU*H
      T6=-OMEGA*H+TFRIC*U*V*SFACT
      T7=-BETA*H*U
      T8=TNU*H
      IF(PRESS) THEN
        T9=GRAV*(DZBDX-DZCDX)
        T10=-GRAV*H
        T11=0.
      ELSE
        T9=-OMEGA*V+GRAV*DZBDX+DFDH*U*UMEAN
        IF(ITDS.GT.0) T9=T9+DUDT+(ALPHA*U)
        T10=-BETA*U*U-GRAV*H+TNU*(DUDX+DUDX)
        T11=-BETA*U*V+TNU*(DUDY+DVDX)
      END IF
C---- VELOCITY TERMS
      IUU=1-NDF
      DO 20 K=1,NCN
        IUU=IUU+NDF
        IUV=IUU+1
        DFUN=XN(K)*T1
        DFUNX=XN(K)*T2+DNX(K)*T3
        DFUNY=XN(K)*T4+DNY(K)*T5
        DFVN=XN(K)*T6
        DFVNY=XN(K)*T7+DNX(K)*T8
        IU=1-NDF
        DO 10 J=1,NCN
          IU=IU+NDF
          ECOEFM(IU,IUU)=ECOEFM(IU,IUU)+WEIGHT*(XN(J)*DFUN+DNX(J)*DFUNX+
     #                   DNY(J)*DFUNY)
          ECOEFM(IU,IUV)=ECOEFM(IU,IUV)+WEIGHT*(XN(J)*DFVN+DNY(J)*DFVNY)
10      CONTINUE
20    CONTINUE
C---- DEPTH TERMS
      IUH=3-2*NDF
      NCM=NCN/2
      DO 40 K=1,NCM
        IUH=IUH+2*NDF
        DFHN=XM(K)*T9
        DFHNX=XM(K)*T10
        DFHNY=XM(K)*T11
        IU=1-NDF
        DO 30 J=1,NCN
          IU=IU+NDF
          ECOEFM(IU,IUH)=ECOEFM(IU,IUH)+WEIGHT*(XN(J)*DFHN+DNX(J)*DFHNX+
     #    DNY(J)*DFHNY)
30     CONTINUE
40    CONTINUE
C
C---- Y-MOMENTUM EQUATION COEFFICIENTS
      T1=OMEGA*H+TFRIC*U*V*SFACT
      T2=-BETA*H*V
      T3=TNU*H
      T4=TFRIC*(U*U+2.*V*V)*SFACT
      IF(ITDS.GT.0) T4=T4+H*ALPHA+DHDT
      T5=-BETA*H*U
      T6=TNU*H
      T7=-2.*BETA*H*V
      T8=2.*TNU*H
      IF(PRESS) THEN
        T9=GRAV*(DZBDY-DZCDY)
        T10=0.
        T11=-GRAV*H
      ELSE
        T9=OMEGA*U+GRAV*DZBDY+DFDH*V*UMEAN
        IF(ITDS.GT.0) T9=T9+DVDT+(ALPHA*V)
        T10=-BETA*V*U+TNU*(DUDY+DVDX)
        T11=-BETA*V*V-GRAV*H+TNU*(DVDY+DVDY)
      ENDIF
C---- VELOCITY TERMS
      IVU=1-NDF
      DO 60 K=1,NCN
        IVU=IVU+NDF
        IVV=IVU+1
        DFUN=XN(K)*T1
        DFUNX=XN(K)*T2+DNY(K)*T3
        DFVN=XN(K)*T4
        DFVNX=XN(K)*T5+DNX(K)*T6
        DFVNY=XN(K)*T7+DNY(K)*T8
        IV=2-NDF
        DO 50 J=1,NCN
          IV=IV+NDF
          ECOEFM(IV,IVU)=ECOEFM(IV,IVU)+WEIGHT*(XN(J)*DFUN+DNX(J)*DFUNX)
          ECOEFM(IV,IVV)=ECOEFM(IV,IVV)+WEIGHT*(XN(J)*DFVN+DNX(J)*DFVNX+
     #                   DNY(J)*DFVNY)
50      CONTINUE
60    CONTINUE
C---- DEPTH TERMS
      IVH=3-2*NDF
      DO 80 K=1,NCM
        IVH=IVH+2*NDF
        DFHN=XM(K)*T9
        DFHNX=XM(K)*T10
        DFHNY=XM(K)*T11
        IV=2-NDF
        DO 70 J=1,NCN
          IV=IV+NDF
          ECOEFM(IV,IVH)=ECOEFM(IV,IVH)+WEIGHT*(XN(J)*DFHN+DNX(J)*DFHNX+
     #                   DNY(J)*DFHNY)
70      CONTINUE
80    CONTINUE
C
C---- CONTINUITY EQUATION COEFFICIENTS
      T1=DHDX
      T2=H
      T3=DHDY
      T4=H
      IF(PRESS) THEN
        T5=0.
        T6=0.
        T7=0.
      ELSE
        T5=DUDX+DVDY
        IF(ITDS.GT.0) T5=T5+ALPHA
        T6=U
        T7=V
      ENDIF
C---- VELOCITY TERMS
      IHU=1-NDF
      DO 100 K=1,NCN
        IHU=IHU+NDF
        IHV=IHU+1
        DFUM=XN(K)*T1+DNX(K)*T2
        DFVM=XN(K)*T3+DNY(K)*T4
        IH=3-2*NDF
        DO 90 J=1,NCM
          IH=IH+2*NDF
          ECOEFM(IH,IHU)=ECOEFM(IH,IHU)+WEIGHT*XM(J)*DFUM
          ECOEFM(IH,IHV)=ECOEFM(IH,IHV)+WEIGHT*XM(J)*DFVM
90      CONTINUE
100   CONTINUE
C---- DEPTH TERMS
      IHH=3-2*NDF
      DO 120 K=1,NCM
        IHH=IHH+2*NDF
        DFHM=XM(K)*T5+DMX(K)*T6+DMY(K)*T7
        IH=3-2*NDF
        DO 110 J=1,NCM
          IH=IH+2*NDF
          ECOEFM(IH,IHH)=ECOEFM(IH,IHH)+WEIGHT*XM(J)*DFHM
110     CONTINUE
120   CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE QRESID(
     # NCN,WEIGHT,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     # H,DHDX,DHDY,DHDT,P,DZBDX,DZBDY,DZCDX,DZCDY,
     # TAUSX,TAUSY,FFACT,UMEAN,TNU,BETA,PRESS)
C
C---- QRESID ---------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES EQUATION RESIDUALS AT THE ITH QUADRATURE POINT AND
C    INSERTS THE WEIGHTED VALUES IN THE ELEMENT LOAD VECTOR.
C
C  INPUT ARGUMENT(S):
C    NCN    - NUMBER OF ELEMENT NODES.
C    WEIGHT - INTEGRATION WEIGHTING FACTOR.
C    U      - VELOCITY IN THE X-DIRECTION.
C    DUDX   - DERIVATIVE OF U WITH RESPECT TO X.
C    DUDY   - DERIVATIVE OF U WITH RESPECT TO Y.
C    DUDT   - DERIVATIVE OF U WITH RESPECT TO TIME.
C    V      - VELOCITY IN THE Y-DIRECTION.
C    DVDX   - DERIVATIVE OF V WITH RESPECT TO X.
C    DVDY   - DERIVATIVE OF V WITH RESPECT TO Y.
C    DVDT   - DERIVATIVE OF V WITH RESPECT TO TIME.
C    H      - DEPTH.
C    DHDX   - DERIVATIVE OF H WITH RESPECT TO X.
C    DHDY   - DERIVATIVE OF H WITH RESPECT TO Y.
C    DHDT   - DERIVATIVE OF H WITH RESPECT TO TIME.
C    DZBDX  - DERIVATIVE OF ZB WITH RESPECT TO X.
C    DZBDY  - DERIVATIVE OF ZB WITH RESPECT TO Y.
C    DZCDX  - DERIVATIVE OF ZC WITH RESPECT TO X.
C    DZCDY  - DERIVATIVE OF ZC WITH RESPECT TO Y.
C    TAUSX  - WIND SHEAR STRESS IN THE X-DIRECTION.
C    TAUSY  - WIND SHEAR STRESS IN THE Y-DIRECTION.
C    FFACT  - DIMENSIONLESS FRICTION FACTOR.
C    UMEAN  - MEAN ABSOLUTE VELOCITY.
C    TNU    - KINEMATIC EDDY VISCOSITY COEFFICIENT.
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NCN
      REAL      WEIGHT,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     #          H,DHDX,DHDY,DHDT,DZBDX,DZBDY,DZCDX,DZCDY,DFDH,
     #          UMEAN,TFRIC,TNU,BETA,TEMP
      LOGICAL   PRESS
C
C---- Declarations (local)
      INTEGER   IU,IV,IH,K,NCM
      REAL      SFACT,FUN,FUNX,FUNY,FVN,FVNX,FVNY,FHM
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- COMPUTE SLOPE FACTOR
      SFACT=SQRT(1.+DZBDX*DZBDX+DZBDY*DZBDY)
      IF(PRESS) THEN
        SFACT=SFACT+SQRT(1.+DZCDX*DZCDX+DZCDY*DZCDY)
      ENDIF
C
C---- MOMENTUM EQUATION RESIDUALS
      FUN=0.0
      FVN=0.0
      FUNX=0.0
      FUNY=0.0
      FVNX=0.0
      FVNY=0.0
C---- TIME TERMS
      IF(ITDS.GT.0) THEN
        FUN=H*DUDT+U*DHDT
        FVN=H*DVDT+V*DHDT
      ENDIF
C---- CONVECTIVE TERMS
      FUNX=FUNX-BETA*H*U*U
      FUNY=FUNY-BETA*H*U*V
      FVNX=FVNX-BETA*H*V*U
      FVNY=FVNY-BETA*H*V*V
C---- PRESSURE TERMS
      FUN=FUN+GRAV*(P*DZBDX-(P-H)*DZCDX)
      FVN=FVN+GRAV*(P*DZBDY-(P-H)*DZCDY)
      TEMP=GRAV*H*(P-0.5*H)
      FUNX=FUNX-TEMP
      FVNY=FVNY-TEMP
C---- CORIOLIS TERMS
      FUN=FUN-V*OMEGA*H
      FVN=FVN+U*OMEGA*H
C---- TURBULENT STRESS TERMS
      FUNX=FUNX+TNU*H*(DUDX+DUDX)
      FUNY=FUNY+TNU*H*(DUDY+DVDX)
      FVNX=FVNX+TNU*H*(DUDY+DVDX)
      FVNY=FVNY+TNU*H*(DVDY+DVDY)
C---- WIND STRESS TERMS
      FUN=FUN-TAUSX/ROWAT
      FVN=FVN-TAUSY/ROWAT
C---- BOTTOM FRICTION TERMS
      FUN=FUN+FFACT*U*UMEAN*SFACT
      FVN=FVN+FFACT*V*UMEAN*SFACT
C---- CALCULATE RESIDUALS
      IU=1-NDF
      DO 10 K=1,NCN
        IU=IU+NDF
        IV=IU+1
        F(IU)=F(IU)-WEIGHT*(XN(K)*FUN+DNX(K)*FUNX+DNY(K)*FUNY)
        F(IV)=F(IV)-WEIGHT*(XN(K)*FVN+DNX(K)*FVNX+DNY(K)*FVNY)
10    CONTINUE
C
C---- CONTINUITY EQUATION RESIDUAL
      FHM=0.0
C---- TIME TERM
      IF(ITDS.GT.0) FHM=DHDT
C---- FLOW TERMS
      FHM=FHM+U*DHDX+V*DHDY+H*(DUDX+DVDY)
C---- CALCULATE RESIDUAL
      IH=3-2*NDF
      NCM=NCN/2
      DO 20 K=1,NCM
        IH=IH+2*NDF
        F(IH)=F(IH)-WEIGHT*XM(K)*FHM
20    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE QVALUE(
     # L,NCN,U,DUDX,DUDY,DUDT,V,DVDX,DVDY,DVDT,
     # H,DHDX,DHDY,DHDT,P,DPDX,DPDY,DPDT,ZB,DZBDX,DZBDY,ZC,DZCDX,DZCDY,
     # TAUSX,TAUSY,FFACT,DFDH,UMEAN,TFRIC,USTAR,TNU,BETA,PRESS)
C
C---- QVALUE ---------------------------------------------------------S
C
C  PURPOSE:
C    CALCULATES VALUES OF VARIABLES AND OTHER PARAMETERS AT
C    QUADRATURE POINTS.
C
C  INPUT ARGUMENT(S):
C    L      - ELEMENT NUMBER.
C    NCN    - NUMBER OF ELEMENT NODES.
C
C  OUTPUT ARGUMENTS:
C    U      - VELOCITY IN THE X-DIRECTION.
C    DUDX   - DERIVATIVE OF U WITH RESPECT TO X.
C    DUDY   - DERIVATIVE OF U WITH RESPECT TO Y.
C    DUDT   - DERIVATIVE OF U WITH RESPECT TO TIME.
C    V      - VELOCITY IN THE Y-DIRECTION.
C    DVDX   - DERIVATIVE OF V WITH RESPECT TO X.
C    DVDY   - DERIVATIVE OF V WITH RESPECT TO Y.
C    DVDT   - DERIVATIVE OF V WITH RESPECT TO TIME.
C    H      - DEPTH.
C    DHDX   - DERIVATIVE OF H WITH RESPECT TO X.
C    DHDY   - DERIVATIVE OF H WITH RESPECT TO Y.
C    DHDT   - DERIVATIVE OF H WITH RESPECT TO TIME.
C    ZB     - BED ELEVATION.
C    DZBDX  - DERIVATIVE OF ZB WITH RESPECT TO X.
C    DZBDY  - DERIVATIVE OF ZB WITH RESPECT TO Y.
C    ZC     - CEILING ELEVATION.
C    DZCDX  - DERIVATIVE OF ZC WITH RESPECT TO X.
C    DZCDY  - DERIVATIVE OF ZC WITH RESPECT TO Y.
C    TAUSX  - WIND SHEAR STRESS IN THE X-DIRECTION.
C    TAUSY  - WIND SHEAR STRESS IN THE Y-DIRECTION.
C    FFACT  - DIMENSIONLESS FRICTION FACTOR.
C    DFDH   - DERIVATIVE OF FFACT WITH RESPECT TO H.
C    UMEAN  - MEAN ABSOLUTE VELOCITY.
C    TFRIC  - FRICTION PARAMETER - FFACT/UMEAN.
C    USTAR  - BED SHEAR VELOCITY.
C    TNU    - KINEMATIC EDDY VISCOSITY COEFFICIENT.
C
C  SUBPROGRAMS CALLED:
C    FRCFCT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      LOGICAL   PRESS
C
C---- Declarations (local)
      INTEGER   K,KC,L,N,NCM,M,ISW
C
C---- Common blocks
      COMMON/CISW/ISW
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2077  FORMAT(' ***ERROR #??? - DEPTH <= 0 IN ELEMENT ',I5,':'/
     # '     K  NODE     DEPTH')
2078  FORMAT(2I6,F10.3)
C
C---- INITIALIZE U, V, H AND THEIR DERIVATIVES
      U=0.
      DUDX=0.
      DUDY=0.
      V=0.
      DVDX=0.
      DVDY=0.
      H=0.
      DHDX=0.
      DHDY=0.
      DUDT=0.
      DVDT=0.
      DHDT=0.
      TAUSX=0.
      TAUSY=0.
      ZB=0.
      DZBDX=0.
      DZBDY=0.
      ZC=0.
      DZCDX=0.
      DZCDY=0.
      P=0.
      DPDX=0.
      DPDY=0.
      DPDT=0.
C
C---- COMPUTE VELOCITIES AND THEIR DERIVATIVES
      DO 10 K=1,NCN
        U=U+XN(K)*UL(K)
        DUDX=DUDX+DNX(K)*UL(K)
        DUDY=DUDY+DNY(K)*UL(K)
        V=V+XN(K)*VL(K)
        DVDX=DVDX+DNX(K)*VL(K)
        DVDY=DVDY+DNY(K)*VL(K)
        IF(ITDS.GT.0) THEN
          DUDT=DUDT+XN(K)*UDOTL(K)
          DVDT=DVDT+XN(K)*VDOTL(K)
        ENDIF
10    CONTINUE
C
C---- COMPUTE DEPTH AND SURFACE STRESS TERMS
      NCM=NCN/2
      DO 20 K=1,NCM
        KC=2*K-1
        H=H+XM(K)*HL(KC)
        DHDX=DHDX+DMX(K)*HL(KC)
        DHDY=DHDY+DMY(K)*HL(KC)
        P=P+XM(K)*PL(KC)
        DPDX=DPDX+DMX(K)*PL(KC)
        DPDY=DPDY+DMY(K)*PL(KC)
        ZB=ZB+XM(K)*GL(KC)
        DZBDX=DZBDX+DMX(K)*GL(KC)
        DZBDY=DZBDY+DMY(K)*GL(KC)
        ZC=ZC+XM(K)*CL(KC)
        DZCDX=DZCDX+DMX(K)*CL(KC)
        DZCDY=DZCDY+DMY(K)*CL(KC)
        IF(IWIND.GT.0) THEN
          N=IABS(NODE(KC))
          TAUSX=TAUSX+XM(K)*SIGMA(N,1)
          TAUSY=TAUSY+XM(K)*SIGMA(N,2)
        ENDIF
        IF(ITDS.GT.0) THEN
          DHDT=DHDT+XM(K)*HDOTL(K)
        ENDIF
20    CONTINUE
C
C---- CHECK FOR DEPTH <= 0
      IF(H.LT.ZIP) THEN
        IF (ISW.NE.L) THEN
        CALL LNODES(L,NCN)
        WRITE(IO2,2077) L
        ISW=L
        DO 30 K=1,NCN
          N=IABS(NODE(K))
          WRITE(IO2,2078) K,N,VEL(3,N)
30      CONTINUE
      ENDIF
      ENDIF
C
C---- CHECK FOR PRESSURE FLOW
      PRESS=.FALSE.
      IF(LMAT(L).GT.1000 .AND. P.GT.(ZC-ZB)) PRESS=.TRUE.
C
C---- COMPUTE COEFFICIENT OF BOTTOM FRICTION
      M=MOD(LMAT(L),100)
      CALL FRCFCT(H,M,FFACT,DFDH)
C
C---- COMPUTE FRICTION TERM AND VELOCITY
      UMEAN=SQRT(U*U+V*V)
      TFRIC=0.
      IF(UMEAN.GT.1.0E-06) TFRIC=FFACT/UMEAN
C
C---- DETERMINE KINEMATIC EDDY VISCOSITY
      USTAR=UMEAN*SQRT(FFACT)
      TNU=PROP(M,6)+PROP(M,7)*USTAR*H
C
C---- COMPUTE MOMENTUM CORRECTION COEFFICIENT
      BETA=BETA0+CBETA*FFACT
      RETURN
      END
C
C
C
      SUBROUTINE RDBOUN(IDS,IER)
C
C---- RDBOUN ---------------------------------------------------------S
C
C  PURPOSE:
C    READS BOUNDARY CONDITION (BOUN) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    NCHECK
C
C---------------------------------------------------------------------F
C
C---- Declarations  (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,N
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(I10,5X,I5,4E10.0)
C
C---- OUPUT FORMATS
2030  FORMAT(//51X,'*** BOUNDARY CONDITION DATA ***'//
     # 43X,'NODE',2X,'COND',6X,'X-SPEC',6X,'Y-SPEC',6X,'Z-SPEC'/)
2031  FORMAT(41X,2I6,4F12.3)
2130  FORMAT(//25X,'*** BOUNDARY CONDITION DATA ***'//
     # 17X,'NODE',2X,'COND',6X,'X-SPEC',6X,'Y-SPEC',6X,'Z-SPEC'/)
2131  FORMAT(15X,2I6,4F12.3)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2030)
        ELSE
          WRITE(IO2,2130)
        ENDIF
      ENDIF
10    READ(IO1,1000) N,INX(1),(XIN(K),K=1,4)
      IF(N.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2031) N,INX(1),(XIN(K),K=1,3)
        ELSE
          WRITE(IO2,2131) N,INX(1),(XIN(K),K=1,3)
        ENDIF
      ENDIF
      CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        NFIX(N)=INX(1)
        SPEC(N,1)=XIN(1)
        SPEC(N,2)=XIN(2)
        SPEC(N,3)=XIN(3)
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDCULV(IDS,IER)
C
C---- RDCULV ---------------------------------------------------------S
C
C  PURPOSE:
C    READS CULVERT (CULV) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C    IDS    - DATA SET IDENTIFICATION CODE
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations  (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,N1,N2
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(2I5,6E10.0)
C
C---- OUTPUT FORMATS
2050  FORMAT(//56X,'*** CULVERT DATA ***'//
     # 27X,'CULV',25X,'DISCH',6X,'AREA',4X,'HYDRAD',4X,'LENGTH',
     # 5X,'ROUGH',6X,'ELEV'/
     # 28X,'NO.',5X,'NODE1',5X,'NODE2',6X,'COEF',5X,'(FT2)',6X,'(FT)',
     # 6X,'(FT)',6X,'COEF',6X,'(FT)'/)
2051  FORMAT(21X,3I10,6F10.3)
2150  FORMAT(//30X,'*** CULVERT DATA ***'//
     # 1X,'CULV',25X,'DISCH',6X,'AREA',4X,'HYDRAD',4X,'LENGTH',
     # 5X,'ROUGH',6X,'ELEV'/
     # 2X,'NO.',5X,'NODE1',5X,'NODE2',6X,'COEF',5X,'(FT2)',6X,'(FT)',
     # 6X,'(FT)',6X,'COEF',6X,'(FT)'/)
2151  FORMAT(I5,2I10,6F10.3)
2340  FORMAT(' ***ERROR #340 - MAXIMUM ALLOWABLE NUMBER OF ',
     # 'CULVERTS HAS BEEN EXCEEDED.')
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2050)
        ELSE
          WRITE(IO2,2150)
        ENDIF
      ENDIF
10    READ(IO1,1000) (INX(K),K=1,2),(XIN(K),K=1,6)
      IF(INX(1).EQ.0) RETURN
      NCUL=NCUL+1
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2051) NCUL,(INX(K),K=1,2),(XIN(K),K=1,6)
        ELSE
          WRITE(IO2,2151) NCUL,(INX(K),K=1,2),(XIN(K),K=1,6)
        ENDIF
      ENDIF
      N1=IABS(INX(1))
      N2=IABS(INX(2))
      CALL NCHECK(N1,IDS,MAXP,IO2,IER,ERR)
      IF(N2.GT.0) CALL NCHECK(N2,IDS,MAXP,IO2,IER,ERR)
      IF(NCUL.GT.MAXC) THEN
        IER=IER+1
        WRITE(IO2,2340)
        RETURN
      ENDIF
      NOPC(NCUL,1)=INX(1)
      NOPC(NCUL,2)=INX(2)
      CCOF(NCUL)=XIN(1)
      CARE(NCUL)=XIN(2)
      CHYR(NCUL)=XIN(3)
      CLEN(NCUL)=XIN(4)
      CMAN(NCUL)=XIN(5)
      CELV(NCUL)=XIN(6)
      GOTO 10
      END
C
C
C
      SUBROUTINE RDELEM(IDS,IER)
C
C---- RDELEM ---------------------------------------------------------S
C
C  PURPOSE:
C    READS ELEMENT (ELEM) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C    IDS    - DATA SET IDENTIFICATION CODE
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations  (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,L,M,N
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
 2010 FORMAT(//
     # 56X,'*** ELEMENT DATA ***'//
     # 24X,'ELEMENT',6X,'NODAL CONNECTIVITY (COUNTER-CLOCKWISE)',
     # 16X,'PROP',5X,'FRONT'/25X,'NUMBER',7X,'1',5X,'2',
     # 5X,'3',5X,'4',5X,'5',5X,'6',5X,'7',5X,'8',5X,'9',4X,'CODE',
     # 5X,'ORDER'/)
 2011 FORMAT(26X,I5,2X,9I6,5X,I3,5X,I5)
 2110 FORMAT(//
     # 30X,'*** ELEMENT DATA ***'//
     # 1X,'ELMT',6X,'NODAL CONNECTIVITY (COUNTER-CLOCKWISE)',
     # 16X,'PROP',4X,'FRONT'/2X,'NO.',7X,'1',5X,'2',
     # 5X,'3',5X,'4',5X,'5',5X,'6',5X,'7',5X,'8',5X,'9',4X,'CODE',
     # 4X,'ORDER'/)
 2111 FORMAT(1X,I5,2X,9I6,5X,I3,3X,I5)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2010)
        ELSE
          WRITE(IO2,2110)
        ENDIF
      ENDIF
10    READ(IO1,'(16I5)') L,(INX(K),K=1,11)
      IF(L.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2011) L,(INX(K),K=1,11)
        ELSE
          WRITE(IO2,2111) L,(INX(K),K=1,11)
        ENDIF
      ENDIF
      CALL LCHECK(L,IDS,MAXE,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        DO 20 K=1,9
          N=INX(K)
          IF(N.GT.0) CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
          IF(.NOT.ERR) THEN
            NOP(L,K)=N
            IF(N.GT.NP) NP=N
          ENDIF
20      CONTINUE
        M=INX(10)
        IF(M.GT.0) CALL MCHECK(M,IDS,MAXM,IO2,IER,ERR)
        LMAT(L)=M
        LSEQ(L)=INX(11)
        IF(L.GT.NE) NE=L
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDFLUX(IDS,IER)
C
C---- RDFLUX ---------------------------------------------------------S
C
C  PURPOSE:
C    READ FLOW CHECK (FLUX) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations  (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,KHI,N
      LOGICAL   ERR,NEWLIN
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2040  FORMAT(//54X,'*** FLOW CHECK DATA ***'/)
2041  FORMAT(/26X,'LINE NO.',I4/
     # 26X,'NODES IN CONSECUTIVE ORDER:')
2042  FORMAT(26X,16I5)
2140  FORMAT(//28X,'*** FLOW CHECK DATA ***'/)
2141  FORMAT(/1X,'LINE NO.',I4/
     # 1X,'NODES IN CONSECUTIVE ORDER:')
2142  FORMAT(16I5)
2200  FORMAT(' ***ERROR #320 - MAXIMUM ALLOWABLE NUMBER OF FLUX-LINE ',
     # 'POINTS HAS BEEN EXCEEDED.')
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2040)
        ELSE
          WRITE(IO2,2140)
        ENDIF
      ENDIF
      KHI=0
      NEWLIN=.TRUE.
10    READ(IO1,'(16I5)') (INX(K),K=1,16)
      IF(INX(1).EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          IF(NEWLIN) WRITE(IO2,2041) NCL+1
          WRITE(IO2,2042) (INX(K),K=1,16)
        ELSE
          IF(NEWLIN) WRITE(IO2,2141) NCL+1
          WRITE(IO2,2142) (INX(K),K=1,16)
        ENDIF
      ENDIF
      NEWLIN=.FALSE.
      DO 20 K=1,16
        N=INX(K)
        IF(N.NE.0) THEN
          KHI=KHI+1
          LFLUXN(KHI)=N
          IF(KHI.GT.MAXLP) THEN
            IER=IER+1
            WRITE(IO2,2200)
            RETURN
          ENDIF
          IF(N.GT.0) THEN
            CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
          ELSE
            NCL=NCL+1
            NEWLIN=.TRUE.
            GOTO 10
          ENDIF
        ENDIF
20    CONTINUE
      GOTO 10
      END
C
C
C
      SUBROUTINE RDINIT(IDS,IER)
C
C---- RDINIT ---------------------------------------------------------S
C
C  PURPOSE:
C    READS INITIAL CONDITION (INIT) DATA.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations  (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,N
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
 2025 FORMAT(//51X,'*** INITIAL CONDITION DATA ***'//
     # 48X,'NODE',7X,'X-VEL',7X,'Y-VEL',7X,'DEPTH'/)
 2026 FORMAT(46X,I6,3F12.4)
 2125 FORMAT(//25X,'*** INITIAL CONDITION DATA ***'//
     # 22X,'NODE',7X,'X-VEL',7X,'Y-VEL',7X,'DEPTH'/)
 2126 FORMAT(20X,I6,3F12.4)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2025)
        ELSE
          WRITE(IO2,2125)
        ENDIF
      ENDIF
10    READ(IO1,'(I10,6E10.0)') N,(XIN(K),K=1,6)
      IF(N.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2026) N,(XIN(K),K=1,6)
        ELSE
          WRITE(IO2,2126) N,(XIN(K),K=1,6)
        ENDIF
      ENDIF
      CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        VEL(1,N)=XIN(1)
        VEL(2,N)=XIN(2)
        VEL(3,N)=XIN(3)
        VDOT(1,N)=XIN(4)
        VDOT(2,N)=XIN(5)
        VDOT(3,N)=XIN(6)
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDNODE(IDS,IER)
C
C---- RDNODE ---------------------------------------------------------S
C
C  PURPOSE:
C    READS NODE (NODE) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,N
      REAL      XFACT,YFACT,ZFACT,XZERO,YZERO,ZZERO
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1000  FORMAT(I10,7E10.0)
C
C---- Output formats
 2015 FORMAT(
     # 50X,'        *** NODE DATA ***'//
     # 50X,'CONVERSION FACTORS FOR:'/
     # 50X,'   X-COORDINATES.........:',F10.2/
     # 50X,'   Y-COORDINATES.........:',F10.2/
     # 50X,'   GROUND ELEVATIONS.....:',F10.2/
     # 50X,'BASE VALUES FOR:'/
     # 50X,'   X-COORDINATES.........:',F10.2/
     # 50X,'   Y-COORDINATES.........:',F10.2/
     # 50X,'   GROUND ELEVATIONS.....:',F10.2//
     # 35X,'NODE',13X,'X-',13X,'Y-',9X,'GROUND',8X,'CEILING'/
     # 36X,'NO.',10X,'COORD',10X,'COORD',11X,'ELEV',11X,'ELEV'/)
 2016 FORMAT(34X,I5,4F15.3)
 2115 FORMAT(
     # 24X,'        *** NODE DATA ***'//
     # 24X,'CONVERSION FACTORS FOR:'/
     # 24X,'   X-COORDINATES.........:',F10.2/
     # 24X,'   Y-COORDINATES.........:',F10.2/
     # 24X,'   GROUND ELEVATIONS.....:',F10.2/
     # 24X,'BASE VALUES FOR:'/
     # 24X,'   X-COORDINATES.........:',F10.2/
     # 24X,'   Y-COORDINATES.........:',F10.2/
     # 24X,'   GROUND ELEVATIONS.....:',F10.2//
     # 9X,'NODE',13X,'X-',13X,'Y-',9X,'GROUND',8X,'CEILING'/
     # 10X,'NO.',10X,'COORD',10X,'COORD',11X,'ELEV',11X,'ELEV'/)
 2116 FORMAT(8X,I5,4F15.3)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2015) (XIN(K),K=1,6)
        ELSE
          WRITE(IO2,2115) (XIN(K),K=1,6)
        ENDIF
      ENDIF
      XFACT=XIN(1)
      YFACT=XIN(2)
      ZFACT=XIN(3)
      XZERO=XIN(4)
      YZERO=XIN(5)
      ZZERO=XIN(6)
      IF(ABS(XFACT).LT.ZIP) XFACT=1.0
      IF(ABS(YFACT).LT.ZIP) YFACT=1.0
      IF(ABS(ZFACT).LT.ZIP) ZFACT=1.0
10    READ(IO1,1000) N,(XIN(K),K=1,4)
      IF(N.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2016) N,(XIN(K),K=1,4)
        ELSE
          WRITE(IO2,2116) N,(XIN(K),K=1,4)
        ENDIF
      ENDIF
      CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        CORD(N,1)=XIN(1)*XFACT+XZERO
        CORD(N,2)=XIN(2)*YFACT+YZERO
        GRND(N)=XIN(3)*ZFACT+ZZERO
        CEIL(N)=XIN(4)*ZFACT+ZZERO
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDPROP(IDS,IER)
C
C---- RDPROP ---------------------------------------------------------S
C
C  PURPOSE:
C    READS ELEMENT PROPERTY (PROP) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,M
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1000  FORMAT(I10,7E10.0)
C
C---- Output formats
2020  FORMAT(//53X,'*** PROPERTY SET DATA ***'//
     # 13X,'SET',6X,'MANNING N',10X,'DEPTH',6X,'MANNING N',10X,'DEPTH',
     # 8X,'CHEZY C',12X,'TNU',12X,'CMU'/)
2021  FORMAT(13X,I3,7E15.3)
2120  FORMAT(//27X,'*** PROPERTY SET DATA ***'//
     # 2X,'SET',1X,'MANNING N',5X,'DEPTH',1X,'MANNING N',5X,'DEPTH',
     # 3X,'CHEZY C',7X,'TNU',7X,'CMU'/)
2121  FORMAT(2X,I3,7E10.3)
2300  FORMAT(' ***ERROR #300 - CHEZY COEFFICIENT FOR ',
     # 'PROPERTY SET ',I3,' IS <= 0.')
2310  FORMAT(' ***ERROR #310 - MANNING COEFFICIENT FOR ',
     # 'PROPERTY SET ',I3,' IS <= 0.')
C
       IF(ECHO) THEN
         IF(WIDE) THEN
           WRITE(IO2,2020)
         ELSE
           WRITE(IO2,2120)
         ENDIF
      ENDIF
10    READ(IO1,1000) M,(XIN(K),K=1,7)
      IF(M.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2021) M,(XIN(K),K=1,7)
        ELSE
          WRITE(IO2,2121) M,(XIN(K),K=1,7)
        ENDIF
      ENDIF
      CALL MCHECK(M,IDS,MAXM,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        DO 20 K=1,7
          PROP(M,K)=XIN(K)
20      CONTINUE
        IF(IFRIC.EQ.0 .AND. PROP(M,1).LE.0.) THEN
          IER=IER+1
          WRITE(IO2,2310) M
        ENDIF
        IF(IFRIC.GT.0 .AND. PROP(M,5).LE.0.) THEN
          IER=IER+1
          WRITE(IO2,2300) M
        ENDIF
        IF(M.GT.NMAT) NMAT=M
        PROP(M,5)=PROP(M,5)*SQRT(GRAV)
        IF(PROP(M,6).LT.ZIP) THEN
          IF(IUNIT.EQ.0) PROP(M,6)=1.23E-05
          IF(IUNIT.GT.0) PROP(M,6)=1.14E-06
        ENDIF
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDQSEC(IDS,IER)
C
C---- RDQSEC ---------------------------------------------------------S
C
C  PURPOSE:
C    READ CROSS SECTION TOTAL FLOW (QSEC) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   IXSQ,J,K,KHI,N
      REAL      Q
      LOGICAL   ERR
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1000  FORMAT(8E10.0)
1010  FORMAT(16I5)
C
C---- Output formats
2010  FORMAT(//47X,'*** CROSS-SECTION TOTAL FLOW DATA ***')
2011  FORMAT(/26X,'SECTION NO. ',I3,':  TOTAL FLOW = ',F12.2,
     # ' CFS'/26X,'NODES IN CONSECUTIVE ORDER:')
2012  FORMAT(/26X,'SECTION NO. ',I3,':  TOTAL FLOW = ',F12.2,
     # ' CMS'/26X,'NODES IN CONSECUTIVE ORDER:')
2013  FORMAT(26X,16I5)
2110  FORMAT(//22X,'*** CROSS-SECTION TOTAL FLOW DATA ***')
2111  FORMAT(/1X,'SECTION NO. ',I3,':  TOTAL FLOW = ',F12.2,
     # ' CFS'/1X,'NODES IN CONSECUTIVE ORDER:')
2112  FORMAT(/1X,'SECTION NO. ',I3,':  TOTAL FLOW = ',F12.2,
     # ' CMS'/1X,'NODES IN CONSECUTIVE ORDER:')
2113  FORMAT(16I5)
2200  FORMAT(' ***ERROR #345 - SECTION NUMBER EXCEEDS 10.')
2210  FORMAT(' ***ERROR #346 - MAXIMUM ALLOWABLE NUMBER OF ',
     # 'CROSS-SECTION POINTS WAS EXCEEDED.')
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2010)
        ELSE
          WRITE(IO2,2110)
        ENDIF
      ENDIF
      KHI=0
10    READ(IO1,1000) (XIN(K),K=1,2)
      IF(XIN(1).LT.ZIP) RETURN
      IXSQ=INT(XIN(1))
      Q=XIN(2)
      IF(ECHO) THEN
        IF(WIDE) THEN
          IF(IUNIT.EQ.0) THEN
            WRITE(IO2,2011) IXSQ,Q
          ELSE
            WRITE(IO2,2012) IXSQ,Q
          ENDIF
        ELSE
          IF(IUNIT.EQ.0) THEN
            WRITE(IO2,2111) IXSQ,Q
          ELSE
            WRITE(IO2,2112) IXSQ,Q
          ENDIF
        ENDIF
      ENDIF
      IF(IXSQ.GT.MAXS) THEN
        WRITE(IO2,2200)
        IER=IER+1
        RETURN
      ELSE
        K=MAXSP*(IXSQ-1)
        NSECQ=NSECQ+1
        XSQ(IXSQ)=Q
      ENDIF
20    READ(IO1,1010) (INX(J),J=1,16)
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2013) (INX(J),J=1,16)
        ELSE
          WRITE(IO2,2113) (INX(J),J=1,16)
        ENDIF
      ENDIF
      DO 30 J=1,16
        N=INX(J)
        K=K+1
        IF(K.GT.MAXSN) THEN
          IER=IER+1
          WRITE(IO2,2210)
          RETURN
        ENDIF
        IF(N.LT.0 .AND. MOD(K,MAXSP).EQ.1) GOTO 10
        LXSQN(K)=N
        IF(N.LT.0) GOTO 10
        CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
30    CONTINUE
      GOTO 20
      END
C
C
C
      SUBROUTINE RDZSEC(IDS,IER)
C
C---- RDZSEC ---------------------------------------------------------S
C
C  PURPOSE:
C    Read Cross Section Water Surface Elevation (ZSEC) data set.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   IBC,IXSZ,J,K,KHI,N
      REAL      Z1,Z2
      LOGICAL   ERR
C
C---- Common blocks
C
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1000  FORMAT(8E10.0)
1010  FORMAT(16I5)
C
C---- Output formats
2010  FORMAT(//
     # 41X,'*** CROSS-SECTION WATER-SURFACE ELEVATION DATA ***'/)
2011  FORMAT(/20X,'SECTION NO. ',I2,': ELEVATION 1 = ',F9.3,
     # ' FT: ELEVATION 2 = ',F9.3,' FT: BOUNDARY CODE = ',I2/
     # 26X,'NODES IN CONSECUTIVE ORDER:')
2012  FORMAT(/20X,'SECTION NO. ',I2,': ELEVATION 1 = ',F9.3,
     # ' M: ELEVATION 2 = ',F9.3,' M: BOUNDARY CODE = ',I2/
     # 26X,'NODES IN CONSECUTIVE ORDER:')
2013  FORMAT(26X,16I5)
2110  FORMAT(//
     # 15X,'*** CROSS-SECTION WATER-SURFACE ELEVATION DATA ***'/)
2111  FORMAT(/1X,'SECTION NO. ',I2/
     # 1X,'ELEVATION 1 = ',F9.3,' FT: ELEVATION 2 = ',F9.3,
     # ' FT: BOUNDARY CODE = ',I2/
     # 1X,'NODES IN CONSECUTIVE ORDER:')
2112  FORMAT(/1X,'SECTION NO. ',I2/
     # 1X,'ELEVATION 1 = ',F9.3,' M: ELEVATION 2 = ',F9.3,
     # ' M: BOUNDARY CODE = ',I2/
     # 1X,'NODES IN CONSECUTIVE ORDER:')
2113  FORMAT(16I5)
2200  FORMAT(' ***ERROR #345 - SECTION NUMBER EXCEEDS 10.')
2210  FORMAT(' ***ERROR #346 - MAXIMUM ALLOWABLE NUMBER OF ',
     # 'CROSS-SECTION POINTS WAS EXCEEDED.')
2220  FORMAT(' ***ERROR #347 - BOUNDARY CODE FOR SECTION ',I2,
     # ' IS OUT OF ALLOWABLE RANGE.')
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2010)
        ELSE
          WRITE(IO2,2110)
        ENDIF
      ENDIF
      KHI=0
10    READ(IO1,1000) (XIN(K),K=1,4)
      IF(XIN(1).LT.ZIP) RETURN
      IXSZ=INT(XIN(1))
      Z1=XIN(2)
      Z2=XIN(3)
      IBC=INT(XIN(4))
      IF(ECHO) THEN
        IF(WIDE) THEN
          IF(IUNIT.EQ.0) THEN
            WRITE(IO2,2011) IXSZ,Z1,Z2,IBC
          ELSE
            WRITE(IO2,2012) IXSZ,Z1,Z1,IBC
          ENDIF
        ELSE
          IF(IUNIT.EQ.0) THEN
            WRITE(IO2,2111) IXSZ,Z1,Z2,IBC
          ELSE
            WRITE(IO2,2112) IXSZ,Z1,Z2,IBC
          ENDIF
        ENDIF
      ENDIF
      IF(IXSZ.GT.MAXS) THEN
        WRITE(IO2,2200)
        IER=IER+1
        RETURN
      ELSE
        K=MAXSP*(IXSZ-1)
        NXSZ=NXSZ+1
        XSZ1(IXSZ)=Z1
        IF(ABS(Z2).LT.ZIP) Z2=Z1
        XSZ2(IXSZ)=Z2
        IF(IBC.LT.0 .OR. IBC.GT.3) THEN
          WRITE(IO2,2220) IXSZ
          IER=IER+1
        ELSEIF(IBC.EQ.0) THEN
          IBCZN(IXSZ)=2
        ELSE
          IBCZN(IXSZ)=IBC
        ENDIF
      ENDIF
20    READ(IO1,1010) (INX(J),J=1,16)
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2013) (INX(J),J=1,16)
        ELSE
          WRITE(IO2,2113) (INX(J),J=1,16)
        ENDIF
      ENDIF
      DO 30 J=1,16
        N=INX(J)
        K=K+1
        IF(K.GT.MAXSN) THEN
          IER=IER+1
          WRITE(IO2,2210)
          RETURN
        ENDIF
        IF(N.LT.0 .AND. MOD(K,MAXSP).EQ.1) GOTO 10
        LXSZN(K)=N
        IF(N.LT.0) GOTO 10
        CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
30    CONTINUE
      GOTO 20
      END
C
C
C
      SUBROUTINE RDTIME(IDS,IER)
C
C---- RDTIME ---------------------------------------------------------S
C
C  PURPOSE:
C    READS TIME-DEPENDENT (TIME) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      REAL      TSOLD
C
C---- Common blocks
C
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1010  FORMAT(8E10.0)
C
C---- Output formats
2055  FORMAT(//45X,'*** NEXT DATA UPDATE AT ',F8.3,' HOURS ***')
2155  FORMAT(//19X,'*** NEXT DATA UPDATE AT ',F8.3,' HOURS ***')
2350  FORMAT(' ***ERROR #350 - TSNEW <= CURRENT SIMULATION TIME.')
C
10    TSOLD=TSNEW
      READ(IO1,1010) TSNEW
      IF(TSNEW.LE.TSOLD) THEN
        WRITE(IO2,2350)
        IER=IER+1
      ELSEIF(TSNEW.GT.TET) THEN
        IF(WIDE) THEN
          WRITE(IO2,2055) TSNEW
        ELSE
          WRITE(IO2,2155) TSNEW
        ENDIF
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE RDWEIR(IDS,IER)
C
C---- RDWEIR ---------------------------------------------------------S
C
C  PURPOSE:
C    READS WEIR (WEIR) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Declarations (local)
      INTEGER   K,N1,N2
      LOGICAL   ERR
C
C---- Common blocks
C
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1010  FORMAT(2I5,7E10.0)
C
C---- Output formats
2045  FORMAT(//58X,'*** WEIR DATA ***'//
     # 34X,'WEIR',31X,'DISCH',6X,'LENGTH',8X,'ELEV'/
     # 35X,'NO.',7X,'NODE1',7X,'NODE2',8X,'COEF',8X,'(FT)',8X,'(FT)'/)
2046  FORMAT(26X,3I12,3F12.3)
2145  FORMAT(//32X,'*** WEIR DATA ***'//
     # 8X,'WEIR',31X,'DISCH',6X,'LENGTH',8X,'ELEV'/
     # 9X,'NO.',7X,'NODE1',7X,'NODE2',8X,'COEF',8X,'(FT)',8X,'(FT)'/)
2146  FORMAT(3I12,3F12.3)
2330  FORMAT(' ***ERROR #330 - MAXIMUM ALLOWABLE NUMBER OF ',
     # 'WEIR SEGMENTS HAS BEEN EXCEEDED.')
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2045)
        ELSE
          WRITE(IO2,2145)
        ENDIF
      ENDIF
10    READ(IO1,1010) (INX(K),K=1,2),(XIN(K),K=1,3)
      IF(INX(1).EQ.0) RETURN
      NWER=NWER+1
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2046) NWER,(INX(K),K=1,2),(XIN(K),K=1,3)
        ELSE
          WRITE(IO2,2146) NWER,(INX(K),K=1,2),(XIN(K),K=1,3)
        ENDIF
      ENDIF
      N1=IABS(INX(1))
      N2=IABS(INX(2))
      CALL NCHECK(N1,IDS,MAXP,IO2,IER,ERR)
      IF(N2.GT.0) CALL NCHECK(N2,IDS,MAXP,IO2,IER,ERR)
      IF(NWER.GT.MAXW) THEN
        IER=IER+1
        WRITE(IO2,2330)
        RETURN
      ENDIF
      NOPW(NWER,1)=INX(1)
      NOPW(NWER,2)=INX(2)
      WCOF(NWER)=XIN(1)
      WLEN(NWER)=XIN(2)
      WCEL(NWER)=XIN(3)
      GOTO 10
      END
C
C
C
      SUBROUTINE RDWIND(IDS,IER)
C
C---- RDWIND ---------------------------------------------------------S
C
C  PURPOSE:
C    READS WIND (WIND) DATA SET.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
      LOGICAL   ERR
C
C---- Declarations (local)
      INTEGER   K,N
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C-----Output formats
 2035 FORMAT(//58X,'*** WIND DATA ***'/
     # 53X,'NODE',4X,'VELOCITY',3X,'DIRECTION'/
     # 54X,'NO.',5X,'(FPS)',7X,'(DEG)'/)
 2036 FORMAT(51X,I6,2F12.3)
 2135 FORMAT(//32X,'*** WIND DATA ***'/
     # 27X,'NODE',4X,'VELOCITY',3X,'DIRECTION'/
     # 28X,'NO.',5X,'(FPS)',7X,'(DEG)'/)
 2136 FORMAT(25X,I6,2F12.3)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2035)
        ELSE
          WRITE(IO2,2135)
        ENDIF
      ENDIF
10    READ(IO1,'(I10,2E10.0)') N,(XIN(K),K=1,2)
      IF(N.EQ.0) RETURN
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2036) N,(XIN(K),K=1,2)
        ELSE
          WRITE(IO2,2136) N,(XIN(K),K=1,2)
        ENDIF
      ENDIF
      CALL NCHECK(N,IDS,MAXP,IO2,IER,ERR)
      IF(.NOT.ERR) THEN
        SIGMA(N,1)=XIN(1)
        SIGMA(N,2)=XIN(2)
      ENDIF
      GOTO 10
      END
C
C
C
      SUBROUTINE RDLAST
C
C---- RDLAST ---------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS END OF DATA MESSAGE.
C
C---------------------------------------------------------------------F
C
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
 2060 FORMAT(//57X,'*** END OF DATA ***'/)
 2160 FORMAT(//31X,'*** END OF DATA ***'/)
C
      IF(ECHO) THEN
        IF(WIDE) THEN
          WRITE(IO2,2060)
        ELSE
          WRITE(IO2,2160)
        ENDIF
      ENDIF
      TSNEW=1.0E+35
      RETURN
      END
C
C
C
      SUBROUTINE RDSWMS(IDS,IER)
C
C---- RDSWMS ---------------------------------------------------------S
C
C  PURPOSE:
C    READS PROGRAM CONTROL DATA SET (SWMS) AND ASSIGNS DEFAULT VALUES.
C
C  INPUT ARGUMENT(S):
C    IDS    - DATA SET IDENTIFICATION CODE
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
      CHARACTER IDS*(*)
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Input formats
1000  FORMAT(A80)
1010  FORMAT(8E10.0)
1020  FORMAT(16I5)
C
C---- Output formats
2002  FORMAT(/
     # 50X,'JOB OPTION CODES FOR:'/
     # 50X,'  RESTART/RECOVERY RUN........: ',I3/
     # 50X,'  PRINTED OUTPUT..............: ',I3/
     # 50X,'  UNITS OF COMPUTATION........: ',I3/
     # 50X,'  WIND STRESS.................: ',I3/
     # 50X,'  BOTTOM FRICTION.............: ',I3/
     # 50X,'  SLIP CONDITION..............: ',I3/
     # 50X,'  EXTENDED INTEGRATION........: ',I3/
     # 50X,'  CONTINUITY NORM.............: ',I3/
     # 50X,'  ELEMENT ON/OFF CHECK........: ',I3/
     # 50X,'  SAVE COEF. MATRICES.........: ',I3/
     # 50X,'  MINIMUM PIVOTAL EQUATIONS...: ',I3)
2003  FORMAT(/
     # 50X,'INPUT/OUTPUT FILE CODES:'/
     # 50X,'  NETWORK DATA FILE...........: ',I3/
     # 50X,'  INITIAL CONDITION FILE......: ',I3/
     # 50X,'  SOLUTION OUTPUT FILE........: ',I3/
     # 50X,'  RESTART/RECOVERY FILE.......: ',I3/
     # 50X,'  BOUNDARY CONDITION FILE.....: ',I3/
     # 50X,'  WIND DATA FILE..............: ',I3/
     # 50X,'  LOWER COEF MATRIX FILE #....: ',I3/
     # 50X,'  UPPER COEF MATRIX FILE #....: ',I3)
2004  FORMAT(/
     # 50X,'ITERATION AND TIME PARAMETERS:'/
     # 50X,'  STEADY-STATE ITERATIONS.....: ',I8/
     # 50X,'  ITERATIONS/TIME-STEP........: ',I8/
     # 50X,'  UPDATE VECTORS (MAX=5)......: ',I8/
     # 50X,'  PRINTED ITERATIONS..........: ',I8/
     # 50X,'  STARTING TIME  (HOURS)......: ',F8.2/
     # 50X,'  TOTAL RUN TIME (HOURS)......: ',F8.2/
     # 50X,'  TIME STEP      (HOURS)......: ',F8.2/
     # 50X,'  TIME INTEGRATION FACTOR ....: ',F8.2)
2005  FORMAT(/
     # 50X,'GENERAL SYSTEM SPECIFICATIONS:'/
     # 50X,'  INITIAL WSEL IN FT (M)......: ',G10.3/
     # 50X,'  AVG LATITUDE IN DEG.........: ',G10.3/
     # 50X,'  WAT DEN IN SLUGS/FT3 (KG/M3): ',G10.3/
     # 50X,'  MOMENTUM CORRECTION COEF(1).: ',G10.3/
     # 50X,'  MOMENTUM CORRECTION COEF(2).: ',G10.3/
     # 50X,'  CONTINUITY NORM FLAG VALUE..: ',G10.3/
     # 50X,'  DEPTH TOLERANCE IN FT (M)...: ',G10.3/
     # 50X,'  RELAXATION FACTOR...........: ',G10.3)
2006  FORMAT(/
     # 50X,'GENERAL WIND STRESS PARAMETERS:'/
     # 50X,'  WIND VEL IN FT/SEC (M/SEC)..: ',G10.3/
     # 50X,'  WIND DIRECTION IN DEG.......: ',G10.3/
     # 50X,'  AIR DEN IN SLUGS/FT3 (KG/M3): ',G10.3/
     # 50X,'  WIND STRESS COEFFICIENT(1)..: ',G10.3/
     # 50X,'  WIND STRESS COEFFICIENT(2)..: ',G10.3/
     # 50X,'  MINIMUM WIND VEL IN M/SEC...: ',G10.3)
2102  FORMAT(/
     # 24X,'JOB OPTION CODES FOR:'/
     # 24X,'  RESTART/RECOVERY RUN........: ',I3/
     # 24X,'  PRINTED OUTPUT..............: ',I3/
     # 24X,'  UNITS OF COMPUTATION........: ',I3/
     # 24X,'  WIND STRESS.................: ',I3/
     # 24X,'  BOTTOM FRICTION.............: ',I3/
     # 24X,'  SLIP CONDITION..............: ',I3/
     # 24X,'  EXTENDED INTEGRATION........: ',I3/
     # 24X,'  CONTINUITY NORM.............: ',I3/
     # 24X,'  ELEMENT ON/OFF CHECK........: ',I3/
     # 24X,'  SAVE COEF. MATRICES.........: ',I3/
     # 24X,'  MINIMUM PIVOTAL EQUATIONS...: ',I3)
2103  FORMAT(/
     # 24X,'INPUT/OUTPUT FILE CODES:'/
     # 24X,'  NETWORK DATA FILE...........: ',I3/
     # 24X,'  INITIAL CONDITION FILE......: ',I3/
     # 24X,'  SOLUTION OUTPUT FILE........: ',I3/
     # 24X,'  RESTART/RECOVERY FILE.......: ',I3/
     # 24X,'  BOUNDARY CONDITION FILE.....: ',I3/
     # 24X,'  WIND DATA FILE..............: ',I3/
     # 24X,'  LOWER COEF MATRIX FILE #....: ',I3/
     # 24X,'  UPPER COEF MATRIX FILE #....: ',I3)
2104  FORMAT(/
     # 24X,'ITERATION AND TIME PARAMETERS:'/
     # 24X,'  STEADY-STATE ITERATIONS.....: ',I8/
     # 24X,'  ITERATIONS/TIME-STEP........: ',I8/
     # 24X,'  UPDATE VECTORS (MAX=5)......: ',I8/
     # 24X,'  PRINTED ITERATIONS..........: ',I8/
     # 24X,'  STARTING TIME  (HOURS)......: ',F8.2/
     # 24X,'  TOTAL RUN TIME (HOURS)......: ',F8.2/
     # 24X,'  TIME STEP      (HOURS)......: ',F8.2/
     # 24X,'  TIME INTEGRATION FACTOR ....: ',F8.2)
2105  FORMAT(/
     # 24X,'GENERAL SYSTEM SPECIFICATIONS:'/
     # 24X,'  INITIAL WSEL IN FT (M)......: ',G10.3/
     # 24X,'  AVG LATITUDE IN DEG.........: ',G10.3/
     # 24X,'  WAT DEN IN SLUGS/FT3 (KG/M3): ',G10.3/
     # 24X,'  MOMENTUM CORRECTION COEF(1).: ',G10.3/
     # 24X,'  MOMENTUM CORRECTION COEF(2).: ',G10.3/
     # 24X,'  CONTINUITY NORM FLAG VALUE..: ',G10.3/
     # 24X,'  DEPTH TOLERANCE IN FT (M)...: ',G10.3/
     # 24X,'  RELAXATION FACTOR...........: ',G10.3)
2106  FORMAT(/
     # 24X,'GENERAL WIND STRESS PARAMETERS:'/
     # 24X,'  WIND VEL IN FT/SEC (M/SEC)..: ',G10.3/
     # 24X,'  WIND DIRECTION IN DEG.......: ',G10.3/
     # 24X,'  AIR DEN IN SLUGS/FT3 (KG/M3): ',G10.3/
     # 24X,'  WIND STRESS COEFFICIENT(1)..: ',G10.3/
     # 24X,'  WIND STRESS COEFFICIENT(2)..: ',G10.3/
     # 24X,'  MINIMUM WIND VEL IN M/SEC...: ',G10.3)
2360  FORMAT(' ***ERROR #360 - NO ITERATIONS HAVE BEEN SPECIFIED.')
2370  FORMAT(' ***ERROR #370 - NUMBER OF STEADY-STATE ITERATIONS ',
     #  'EXCEEDS MAXIMUM.')
2380  FORMAT(' ***ERROR #380 - NUMBER OF ITERATIONS/TIME-STEP ',
     #  'EXCEEDS MAXIMUM.')
2390  FORMAT(' ***ERROR #390 - TIME INTEGRATION FACTOR IS OUT OF ',
     #  'ALLOWABLE RANGE.')
2391  FORMAT(' ***ERROR #391 - RELAXATION FACTOR IS OUT OF ',
     #  'ALLOWABLE RANGE.')
C
CARD 1 - JOB TITLE
      READ(IO1,1000) TITLE
      CALL CPRINT(TITLE,IO2,WIDE,1,1)
      CALL CPRINT('*** PROGRAM CONTROL DATA ***',IO2,WIDE,1,1)
C
CARD 2 - JOB OPTION CODES
      READ(IO1,1020) IDRUN,IPRNT,IUNIT,IWIND,IFRIC,ISLIP,IXINT,
     # INORM,IONOFF,ISAVE,NPVMIN
      IF(WIDE) THEN
        WRITE(IO2,2002)IDRUN,IPRNT,IUNIT,IWIND,IFRIC,ISLIP,IXINT,
     #  INORM,IONOFF,ISAVE,NPVMIN
      ELSE
        WRITE(IO2,2102)IDRUN,IPRNT,IUNIT,IWIND,IFRIC,ISLIP,IXINT,
     #  INORM,IONOFF,ISAVE,NPVMIN
      ENDIF
C
CARD 3 - INPUT/OUTPUT FILE CODES
      READ(IO1,1020) IGRID,INITC,ISOUT,IRSRC,IBCIN,IWDIN,
     # ILCOF,IUCOF
      IF(WIDE) THEN
        WRITE(IO2,2003)IGRID,INITC,ISOUT,IRSRC,IBCIN,IWDIN,
     #  ILCOF,IUCOF
      ELSE
        WRITE(IO2,2103)IGRID,INITC,ISOUT,IRSRC,IBCIN,IWDIN,
     #  ILCOF,IUCOF
      ENDIF
C
CARD 4 - ITERATION AND TIME PARAMETERS
      READ(IO1,'(4I10,4E10.0)') NITS,NITD,NUPV,NITP,TSTRT,TMAX,DELT,
     # THETA
      IF(WIDE) THEN
        WRITE(IO2,2004)NITS,NITD,NUPV,NITP,TSTRT,TMAX,DELT,THETA
      ELSE
        WRITE(IO2,2104)NITS,NITD,NUPV,NITP,TSTRT,TMAX,DELT,THETA
      ENDIF
C
CARD 5 - GENERAL SYSTEM SPECIFICATIONS
      READ(IO1,1010) WSEL,OMEGA,ROWAT,BETA0,CBETA,CFLAG,DEPTOL,
     # RELAX
      IF(WIDE) THEN
        WRITE(IO2,2005)WSEL,OMEGA,ROWAT,BETA0,CBETA,CFLAG,DEPTOL,RELAX
      ELSE
        WRITE(IO2,2105)WSEL,OMEGA,ROWAT,BETA0,CBETA,CFLAG,DEPTOL,RELAX
      ENDIF
C
CARD 6 - GENERAL WIND STRESS PARAMETERS
      READ(IO1,1010) WVEL,WDIR,ROAIR,CSURF1,CSURF2,WVMIN
      IF(WIDE) THEN
        WRITE(IO2,2006) WVEL,WDIR,ROAIR,CSURF1,CSURF2,WVMIN
      ELSE
        WRITE(IO2,2106) WVEL,WDIR,ROAIR,CSURF1,CSURF2,WVMIN
      ENDIF
C
C---- CHECK FOR SPECIFIED ITERATIONS
      IF(NITS.LE.0  .AND.  NITD.LE.0) THEN
        IER=IER+1
        WRITE(IO2,2360)
      ENDIF
C
C---- COMPUTE CONSTANTS AND SET DEFAULTS
      IF(IUNIT.EQ.0) THEN
        IF(DEPTOL.LT.ZIP) DEPTOL=0.5
        IF(ROAIR.LT.ZIP) ROAIR=0.00237
        IF(ROWAT.LT.ZIP) ROWAT=1.937
      ELSE
        IF(DEPTOL.LT.ZIP) DEPTOL=0.15
        IF(ROAIR.LT.ZIP) ROAIR=1.225
        IF(ROWAT.LT.ZIP) ROWAT=999.0
      ENDIF
      IF(RELAX.LT.ZIP) RELAX=1.0
      IF(CSURF1.LT.ZIP) CSURF1=1.0
      IF(NUPV.LT.0) NUPV=0
      IF(NUPV.GT.5) NUPV=5
      IF(THETA.LT.ZIP) THETA=0.5
      IF(BETA0.LT.0) BETA0=0.
      IF(CFLAG.LT.ZIP) CFLAG=1.0E+38
      IF(ILCOF.EQ.0) ILCOF=IO98
      IF(IUCOF.EQ.0) IUCOF=IO99
      IF(ISAVE.NE.0) ISAVE=1
      IF(MOD(IPRNT,2).EQ.1) ECHO=.TRUE.
      IF(NPVMIN.EQ.0) NPVMIN=1
      IF(IUNIT.EQ.1) GRAV=9.806
      TET=TSTRT
      NINITS=MOD(NITS,100)
      NQNITS=MOD(NITS/100,100)
      NANITS=NITS/10000
      NITS=NINITS+NQNITS+NANITS*(1+NQNITS)
      NINITD=MOD(NITD,100)
      NQNITD=MOD(NITD/100,100)
      NANITD=NITD/10000
      NITD=NINITD+NQNITD+NANITD*(1+NQNITD)
      NITPS=MOD(NITP,100)+1
      NITPDT=MOD(NITP/100,100)+1
      NITPDI=NITP/10000+1
C
C---- CHECK FOR VALID INPUT VALUES
      IF(NITS.GT.99) THEN
        IER=IER+1
        WRITE(IO2,2370)
      ENDIF
      IF(NITD.GT.99) THEN
        IER=IER+1
        WRITE(IO2,2380)
      ENDIF
      IF(THETA.LT.0.5 .OR. THETA.GT.1.0) THEN
        IER=IER+1
        WRITE(IO2,2390)
      ENDIF
      IF(RELAX.LT.0. .OR. RELAX.GT.2.) THEN
        IER=IER+1
        WRITE(IO2,2391)
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE RFBOUN(IER)
C
C---- RFBOUN ----------------------------------------------------------S
C
C  PURPOSE:
C    READS BOUNDARY CONDITION FILE. THIS FILE MAY BE EITHER FORMATTED
C    OR UNFORMATTED.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,LENCHR,N,NPX
      REAL      TIME
      LOGICAL   EXISTS
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(E10.0,I10)
1010  FORMAT(I10,3E10.0)
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #1000 - ERROR OCCURRED WHILE READING THE ',
     # 'FILE ',(A))
2010  FORMAT(' ***ERROR #1010 - UNEXPECTED END-OF-FILE ENCOUNTERED ',
     # 'WHILE READING ',(A))
2020  FORMAT(' ***ERROR #404 - BOUNDARY CONDITION FILE ',(A),
     # ' CANNOT BE FOUND.')
2100  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2100) BOUNFILE(1:LENCHR(BOUNFILE))
C
      IF(EXISTS(BOUNFILE)) THEN
C
C------ TRY READING A FORMATTED FILE FIRST
        OPEN(IO95,FILE=BOUNFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     #  FORM='FORMATTED')
10      READ(IO95,1000,ERR=20,END=50) TIME,NPX
        READ(IO95,1010,ERR=40,END=50) (N,NFIX(N),(SPEC(N,K),K=1,3),
     #  I=1,NPX)
        IF(TIME.LT.TET) GOTO 10
        GOTO 60
C
C------ TRY READING AN UNFORMATTED FILE IF AN ERROR WAS FOUND
20      CLOSE(IO95,STATUS='KEEP')
        OPEN(IO95,FILE=BOUNFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     #  FORM='UNFORMATTED')
30      READ(IO95,ERR=40,END=50) TIME,NPX
        READ(IO95,ERR=40,END=50) (N,NFIX(N),(SPEC(N,K),K=1,3),
     #  I=1,NPX)
        IF(TIME.LT.TET) GOTO 30
      ELSE
C
C------ PRINT AN ERROR MESSAGE AND RETURN
        WRITE(IO2,2020) BOUNFILE(1:LENCHR(BOUNFILE))
        IER=IER+1
        RETURN
      ENDIF
      GOTO 60
C
C---- ERROR READING THE FILE
40    WRITE(IO2,2000) BOUNFILE(1:LENCHR(BOUNFILE))
      IER=IER+1
      GOTO 60
C
C---- UNEXPECTED END-OF-FILE ENCOUNTERED
50    WRITE(IO2,2010) BOUNFILE(1:LENCHR(BOUNFILE))
      IER=IER+1
60    IF(NITD.EQ.0) CLOSE(IO95,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE RFGRID(IER)
C
C---- RFGRID ----------------------------------------------------------S
C
C  PURPOSE:
C    READS NETWORK DATA FILE. THIS FILE MAY BE EITHER FORMATTED OR
C    UNFORMATTED.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,L,LENCHR,N
      LOGICAL   EXIST
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(2I10)
1010  FORMAT(I10,4E10.0)
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #1000 - ERROR OCCURRED WHILE READING THE ',
     # 'FILE ',(A))
2010  FORMAT(' ***ERROR #1010 - UNEXPECTED END-OF-FILE ENCOUNTERED ',
     # 'WHILE READING ',(A))
2030  FORMAT(' ***ERROR #401 - NETWORK DATA FILE ',(A),
     # ' CANNOT BE FOUND.')
2100  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2100) GRIDFILE(1:LENCHR(GRIDFILE))
C
C---- DOES A NETWORK DATA FILE EXIST?
      INQUIRE(FILE=GRIDFILE,EXIST=EXIST)
      IF(.NOT.EXIST) THEN
        WRITE(IO2,2030) GRIDFILE(1:LENCHR(GRIDFILE))
        IER=IER+1
        RETURN
      ENDIF
C
C---- TRY READING A FORMATTED FILE FIRST
      OPEN(IO93,FILE=GRIDFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='FORMATTED')
      READ(IO93,1000,ERR=10,END=30) NP,NE
      READ(IO93,1010,ERR=20,END=30) (N,(CORD(N,K),K=1,2),
     # GRND(N),CEIL(N),I=1,NP)
      READ(IO93,'(12I5)',ERR=20,END=30) (L,(NOP(L,K),K=1,9),
     # LMAT(L),LSEQ(L),I=1,NE)
      GOTO 50
C
C---- TRY READING AN UNFORMATTED FILE IF AN ERROR WAS FOUND
10    CLOSE(IO93,STATUS='KEEP')
      OPEN(IO93,FILE=GRIDFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
      READ(IO93,ERR=20,END=30) NP,NE
      READ(IO93,ERR=20,END=30) ((CORD(N,K),K=1,2),GRND(N),
     # CEIL(N),N=1,NP)
      READ(IO93,ERR=20,END=30) ((NOP(L,K),K=1,9),LMAT(L),
     # LSEQ(L),L=1,NE)
      GOTO 50
C
C---- ERROR READING THE FILE
20    WRITE(IO2,2000) GRIDFILE(1:LENCHR(GRIDFILE))
      IER=IER+1
      GOTO 50
C
C---- UNEXPECTED END-OF-FILE ENCOUNTERED
30    WRITE(IO2,2010) GRIDFILE(1:LENCHR(GRIDFILE))
      IER=IER+1
C
C---- CLOSE NETWORK DATA FILE
50    CLOSE(IO93,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE RFINIT(IER)
C
C---- RFINIT ----------------------------------------------------------S
C
C  PURPOSE:
C    READS INITIAL CONDITION FILE. THIS FILE MAY BE EITHER FORMATTED
C    OR UNFORMATTED.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,LENCHR,N,NPX
      REAL      TIME
      LOGICAL   EXIST
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INPUT FORMATS
1000  FORMAT(E10.0,I10)
1010  FORMAT(I10,6E10.0)
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #1000 - ERROR OCCURRED WHILE READING THE ',
     # 'FILE ',(A))
2010  FORMAT(' ***ERROR #1010 - UNEXPECTED END-OF-FILE ENCOUNTERED ',
     # 'WHILE READING ',(A))
2020  FORMAT(' ***ERROR #403 - INITIAL CONDITION FILE ',(A),
     # ' CANNOT BE FOUND.')
2100  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2100) INITFILE(1:LENCHR(INITFILE))
C
C---- DOES AN INITIAL CONDITION FILE EXIST?
      INQUIRE(FILE=INITFILE,EXIST=EXIST)
      IF(.NOT.EXIST) THEN
        WRITE(IO2,2020) INITFILE(1:LENCHR(INITFILE))
        IER=IER+1
        RETURN
      ENDIF
C
C---- TRY READING A FORMATTED FILE FIRST
      OPEN(IO94,FILE=INITFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='FORMATTED')
10    READ(IO94,1000,ERR=20,END=50) TIME,NPX
      DO 15 I=1,NPX
      READ(IO94,1010,ERR=40,END=50) N,(VEL(K,N),K=1,3),(VDOT(K,N),K=1,3)
15    CONTINUE
      IF(TIME.LT.TET) GOTO 10
      GOTO 60
C
C---- TRY READING AN UNFORMATTED FILE IF AN ERROR WAS FOUND
20    CLOSE(IO94,STATUS='KEEP')
      OPEN(IO94,FILE=INITFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
30    READ(IO94,ERR=40,END=50) TIME,NPX
      READ(IO94,ERR=40,END=50) ((VEL(K,N),K=1,3),(VDOT(K,N),
     # K=1,3),N=1,NPX)
      IF(TIME.LT.TET) GOTO 30
      GOTO 60
C
C---- ERROR READING THE FILE
40    WRITE(IO2,2000) INITFILE(1:LENCHR(INITFILE))
      IER=IER+1
      GOTO 60
C
C---- UNEXPECTED END-OF-FILE ENCOUNTERED
50    WRITE(IO2,2010) INITFILE(1:LENCHR(INITFILE))
      IER=IER+1
C
C---- CLOSE INITIAL CONDITION FILE
60    CLOSE(IO94,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE RFRSRC(IER)
C
C---- RFRSRC ----------------------------------------------------------S
C
C  PURPOSE:
C    READS RESTART/RECOVERY FILE. THIS FILE MAY BE EITHER FORMATTED
C    OR UNFORMATTED.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,LENCHR,N,NPX
      LOGICAL   EXIST
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
1000  FORMAT(' ***ERROR #1000 - ERROR OCCURRED WHILE READING THE ',
     # 'FILE ',(A))
1010  FORMAT(' ***ERROR #1010 - UNEXPECTED END-OF-FILE ENCOUNTERED ',
     # 'WHILE READING ',(A))
2020  FORMAT(' ***ERROR #402 - RESTART/RECOVERY FILE ',(A),
     # ' CANNOT BE FOUND.')
2100  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2100) RSRCFILE(1:LENCHR(RSRCFILE))
C
C---- DOES A RESTART/RECOVERY FILE EXIST?
      INQUIRE(FILE=RSRCFILE,EXIST=EXIST)
      IF(.NOT.EXIST) THEN
        WRITE(IO2,2020) RSRCFILE(1:LENCHR(RSRCFILE))
        IER=IER+1
        RETURN
      ENDIF
C
C---- TRY READING A FORMATTED FILE FIRST
      OPEN(IO97,FILE=RSRCFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='FORMATTED')
      READ(IO97,'(E10.0,2I10)',ERR=10,END=30) TET,NPX,NITSV
      READ(IO97,'(I10,6E10.0)',ERR=20,END=30) (N,(VEL(K,N),
     # K=1,3),(VDOT(K,N),K=1,3),I=1,NPX)
      GOTO 60
C
C---- TRY READING AN UNFORMATTED IF AN ERROR WAS FOUND
10    CLOSE(IO97,STATUS='KEEP')
      READ(IO97,ERR=20,END=30) TET,NPX,NITSV
      READ(IO97,ERR=20,END=30) ((VEL(K,N),K=1,3),(VDOT(K,N),
     # K=1,3),N=1,NPX)
      GOTO 60
C
C---- ERROR READING THE FILE
20    WRITE(IO2,1000) RSRCFILE(1:LENCHR(RSRCFILE))
      IER=IER+1
      GOTO 60
C
C---- UNEXPECTED END-OF-FILE ENCOUNTERED
30    WRITE(IO2,1000) RSRCFILE(1:LENCHR(RSRCFILE))
      IER=IER+1
C
C---- CLOSE RESTART/RECOVERY FILE
60    CLOSE(IO97,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE RFWIND(IER)
C
C---- RFWIND ----------------------------------------------------------S
C
C  PURPOSE:
C    READS WIND DATA FILE. THIS FILE MAY BE EITHER FORMATTED OR
C    UNFORMATTED.
C
C  INPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   I,K,LENCHR,N,NPX
      REAL      TIME
      LOGICAL   EXIST
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***ERROR #1000 - ERROR OCCURRED WHILE READING THE ',
     # 'FILE ',(A))
2010  FORMAT(' ***ERROR #1010 - UNEXPECTED END-OF-FILE ENCOUNTERED ',
     # 'WHILE READING ',(A))
2030  FORMAT(' ***ERROR #405 - WIND DATA FILE  ',(A),
     # ' CANNOT BE FOUND.')
2100  FORMAT(1H+,79(' ')/1H+,'Reading (',(A),')...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2100) WINDFILE(1:LENCHR(WINDFILE))
C
C---- DOES WIND DATA FILE EXIST?
      INQUIRE(FILE=WINDFILE,EXIST=EXIST)
      IF(.NOT.EXIST) THEN
        WRITE(IO2,2030) WINDFILE(1:LENCHR(WINDFILE))
        IER=IER+1
        RETURN
      ENDIF
C
C---- TRY READING A FORMATTED FILE FIRST
      OPEN(IO96,FILE=WINDFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='FORMATTED')
10    READ(IO96,'(E10.0,I10)',ERR=20,END=50) TIME,NPX
      READ(IO96,'(I10,2E10.0)',ERR=40,END=50) (N,SIGMA(N,1),
     # SIGMA(N,2),I=1,NPX)
      IF(TIME.LT.TET) GOTO 10
      GOTO 60
C
C---- TRY READING AN UNFORMATTED FILE IF AN ERROR OCCURRED
20    CLOSE(IO96,STATUS='KEEP')
      OPEN(IO96,FILE=WINDFILE,STATUS='OLD',ACCESS='SEQUENTIAL',
     # FORM='UNFORMATTED')
30    READ(IO96,ERR=40,END=50) TIME,NPX
      READ(IO96,ERR=40,END=50) (N,SIGMA(N,1),SIGMA(N,2),I=1,NPX)
      IF(TIME.LT.TET) GOTO 30
      GOTO 60
C
C---- ERROR READING THE FILE
40    WRITE(IO2,2000) WINDFILE(1:LENCHR(WINDFILE))
      IER=IER+1
      GOTO 60
C
C---- UNEXPECTED END-OF-FILE ENCOUNTERED
50    WRITE(IO2,2010) WINDFILE(1:LENCHR(WINDFILE))
      IER=IER+1
C
C---- CLOSE WIND DATA FILE
60    IF(NITD.EQ.0) CLOSE(IO96,STATUS='KEEP')
      RETURN
      END
C
C
C
      SUBROUTINE SFLOAD(L,NCN,NGP,KODE)
C
C---- SFLOAD ----------------------------------------------------------S
C
C   PURPOSE:
C     COPIES THE PROPER WEIGHTING FUNCTIONS INTO THE WORKING ARRAYS
C     WAITX, DA, DB, CA, AND CB.
C
C   INPUT ARGUMENT(S):
C     L - ELEMENT NUMBER.
C
C   OUTPUT ARGUMENT(S):
C     NCN - NUMBER OF ELEMENT NODE POINTS.
C     NGP - NUMBER OF ELEMENT INTEGRATION POINTS.
C     KODE - INTEGRATION ORDER CODE.  KODE = 0 RESULTS IN INTEGRATION
C     OF O(H**5) WHILE KODE = 1 RESULTS IN INTEGRATION OF O(H**7).
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   L,NCN,NGP,KODE
C
C---- Declarations (local)
      INTEGER J,K
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      NCN=6
      IF(NOP(L,8).NE.0) NCN=8
      IF(NOP(L,9).NE.0) NCN=9
C
C---- TRIANGULAR ELEMENT (LOW-ORDER)
      IF(NCN.EQ.6 .AND. KODE.EQ.0) THEN
        NGP=7
        DO 20 J=1,NGP
          WAITX(J)=WAITT(J)
          DO 10 K=1,NCN
            XNX(K,J)=XNT(K,J)
            DA(K,J)=DNAT(K,J)
            DB(K,J)=DNBT(K,J)
            XMX(K,J)=XMT(K,J)
            CA(K,J)=DMAT(K,J)
            CB(K,J)=DMBT(K,J)
10        CONTINUE
20      CONTINUE
C
C---- TRIANGULAR ELEMENT (HIGH-ORDER)
      ELSEIF(NCN.EQ.6 .AND. KODE.EQ.1) THEN
        NGP=16
        DO 40 J=1,NGP
          WAITX(J)=WAITTH(J)
          DO 30 K=1,NCN
            XNX(K,J)=XNTH(K,J)
            DA(K,J)=DNATH(K,J)
            DB(K,J)=DNBTH(K,J)
            XMX(K,J)=XMTH(K,J)
            CA(K,J)=DMATH(K,J)
            CB(K,J)=DMBTH(K,J)
30        CONTINUE
40      CONTINUE
C
C---- EIGHT-NODE QUADRILATERAL ELEMENT (LOW-ORDER)
      ELSEIF(NCN.EQ.8 .AND. KODE.EQ.0) THEN
        NGP=9
        DO 60 J=1,NGP
          WAITX(J)=WAITR(J)
          DO 50 K=1,NCN
            XNX(K,J)=XNR(K,J)
            DA(K,J)=DNAR(K,J)
            DB(K,J)=DNBR(K,J)
            XMX(K,J)=XMR(K,J)
            CA(K,J)=DMAR(K,J)
            CB(K,J)=DMBR(K,J)
50        CONTINUE
60      CONTINUE
C
C---- EIGHT-NODE QUADRILATERAL ELEMENT (HIGH-ORDER)
      ELSEIF(NCN.EQ.8 .AND. KODE.EQ.1) THEN
        NGP=16
        DO 80 J=1,NGP
          WAITX(J)=WAITRH(J)
          DO 70 K=1,NCN
            XNX(K,J)=XNRH(K,J)
            DA(K,J)=DNARH(K,J)
            DB(K,J)=DNBRH(K,J)
            XMX(K,J)=XMRH(K,J)
            CA(K,J)=DMARH(K,J)
            CB(K,J)=DMBRH(K,J)
70        CONTINUE
80      CONTINUE
C
C---- NINE-NODE QUADRILATERAL ELEMENT (LOW-ORDER)
      ELSEIF(NCN.EQ.9 .AND. KODE.EQ.0) THEN
        NGP=9
        DO 100 J=1,NGP
          WAITX(J)=WAITR(J)
          DO 90 K=1,NCN
            XNX(K,J)=XNQ(K,J)
            DA(K,J)=DNAQ(K,J)
            DB(K,J)=DNBQ(K,J)
            XMX(K,J)=XMR(K,J)
            CA(K,J)=DMAR(K,J)
            CB(K,J)=DMBR(K,J)
90        CONTINUE
100     CONTINUE
C
C---- NINE-NODE QUADRILATERAL ELEMENT (HIGH-ORDER)
      ELSEIF(NCN.EQ.9 .AND. KODE.EQ.1) THEN
        NGP=16
        DO 120 J=1,NGP
          WAITX(J)=WAITRH(J)
          DO 110 K=1,NCN
            XNX(K,J)=XNQH(K,J)
            DA(K,J)=DNAQH(K,J)
            DB(K,J)=DNBQH(K,J)
            XMX(K,J)=XMRH(K,J)
            CA(K,J)=DMARH(K,J)
            CB(K,J)=DMBRH(K,J)
110       CONTINUE
120     CONTINUE
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE SHPFNC(NCN,I,DETJ)
C
C---- SHPFNC ----------------------------------------------------------S
C
C  PURPOSE:
C    CALCULATES THE DETERMINANT OF THE JACOBIAN MATRIX AND ASSIGNS
C    SHAPE FUNCTIONS AND THEIR DERIVATIVES AT THE ITH QUADRATURE POINT.
C
C  INPUT ARGUMENT(S):
C    NCN - NUMBER OF ELEMENT NODES.
C    I - QUADRATURE POINT.
C
C  OUTPUT ARGUMENT(S):
C    DETJ - DETERMINANT OF THE JACOBIAN MATRIX AT THE ITH GAUSS POINT.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   NCN,I
      REAL      DETJ
C
C---- Declarations (local)
      INTEGER    K,KK,NCM
      REAL       J11,J12,J21,J22
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- FORM THE JACOBIAN AT THE ITH GAUSS POINT
      J11=0.
      J12=0.
      J21=0.
      J22=0.
      DO 10 K=2,NCN
        J11=J11+DA(K,I)*XL(K)
        J12=J12+DA(K,I)*YL(K)
        J21=J21+DB(K,I)*XL(K)
        J22=J22+DB(K,I)*YL(K)
10    CONTINUE
      DETJ=J11*J22-J12*J21
C
C---- COMPUTE QUADRATIC SHAPE FUNCTION AND DERIVATIVES
      DO 20 K=1,NCN
        XN(K)=XNX(K,I)
        DNX(K)=(J22*DA(K,I)-J12*DB(K,I))/DETJ
        DNY(K)=(J11*DB(K,I)-J21*DA(K,I))/DETJ
20    CONTINUE
C
C---- COMPUTE LINEAR SHAPE FUNCTION AND DERIVATIVES
      NCM=NCN/2
      KK=-1
      DO 30 K=1,NCM
        KK=KK+2
        XM(K)=XMX(KK,I)
        DMX(K)=(J22*CA(KK,I)-J12*CB(KK,I))/DETJ
        DMY(K)=(J11*CB(KK,I)-J21*CA(KK,I))/DETJ
30    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE SOLCOR(IDP)
C
C---- SOLCOR ----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES THE CORRECTED VALUE OF EACH FLOW VARIABLE AFTER EVERY
C    NEWTON ITERATION.  VALUES OF DEPTH ARE LINEARLY INTERPOLATED
C    AT EACH MIDSIDE NODE. CONVERGENCE PARAMETERS ARE COMPUTED AND
C    PRINTED TO PROVIDE A MEANS OF DETERMINING IF THE SOLUTION IS
C    CONVERGING.
C
C
C  INPUT ARGUMENT(S):
C    IDP    - OUTPUT CODE: CONVERGENCE PARAMETERS ARE PRINTED IF
C             IDP > 1
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IDP
C
C---- Declarations (local)
      INTEGER   I,J,K,KNT,L,N,NCN,NEQN,NN,NMAX,N1,N2,N3
      REAL      EX,AEX,EAVG,EMAX,H,TEMP
      CHARACTER DOF*12,UNIT0*5,UNIT1*5,UNIT*5
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Dimensions
      DIMENSION EAVG(5),EMAX(5),NMAX(5),DOF(5),UNIT0(5),UNIT1(5)
C---- Data
      DATA DOF/'  X-VELOCITY','  Y-VELOCITY','       DEPTH',
     # '    SALINITY',' TEMPERATURE'/
      DATA UNIT0/' FT/S',' FT/S','   FT',' MG/L','DEG F'/
      DATA UNIT1/'  M/S','  M/S','    M',' MG/L','DEG C'/
C
C---- OUTPUT FORMATS
2000  FORMAT(/27X,'*** RESULTS AT THE END OF ',F8.3,' HRS:',
     # 1X,'TIME STEP NO. ',I3,': ITERATION NO. ',I3,' ***'/)
2001  FORMAT(/55X,'CONVERGENCE PARAMETERS:'/
     # 41X,'  VARIABLE  UNITS  AVG CHANGE  MAX CHANGE   NODE'/
     # 41X,'==========  =====  ==========  ==========  =====')
2002  FORMAT(39X,A12,2X,A5,2F12.4,I7)
2090  FORMAT(/ 47X,'*** NODAL VARIABLES AND CORRECTIONS ***'/)
2091  FORMAT(I10,10F12.6)
2100  FORMAT(/1X,'*** RESULTS AT THE END OF ',F8.3,' HRS:',
     # ' TIME STEP NO. ',I3,': ITERATION NO. ',I2,' ***'/)
2101  FORMAT(/29X,'CONVERGENCE PARAMETERS:'/
     # 15X,'  VARIABLE  UNITS  AVG CHANGE  MAX CHANGE   NODE'/
     # 15X,'==========  =====  ==========  ==========  =====')
2102  FORMAT(13X,A12,2X,A5,2F12.4,I7)
2500  FORMAT(1H+,79(' ')/1H+,'Computing solution corrections ...')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2500)
C
C---- PRINT SOLUTION CORRECTIONS IF MOD(IDEBUG,64) > 31
      IF(MOD(IDEBUG,64).GT.31) THEN
        WRITE(IO2,2090)
        DO 20 N=1,NP
          DO 10 K=1,NDF
            EAVG(K)=0.
            NEQN=NBC(N,K)
            IF(NEQN.GT.0) EAVG(K)=B(NEQN)
10        CONTINUE
          WRITE(IO2,2091) N,(VEL(K,N),K=1,NDF),(EAVG(K),K=1,NDF)
20      CONTINUE
      ENDIF
C
C---- CORRECT SOLUTION VARIABLES
      DO 40 K=1,NDF
        EMAX(K)=0.
        NMAX(K)=0
        KNT=0
        TEMP=0.
        DO 30 N=1,NP
          IF(NBC(N,K).GT.0) THEN
            KNT=KNT+1
            J=NBC(N,K)
C
C---------- Multiply full correction by over/under relaxation factor
            EX=B(J)*RELAX
            VEL(K,N)=VEL(K,N)+EX
            AEX=ABS(EX)
            TEMP=TEMP+AEX
            IF(AEX.GT.ABS(EMAX(K))) THEN
              EMAX(K)=EX
              NMAX(K)=N
            ENDIF
          ENDIF
30      CONTINUE
        EAVG(K)=TEMP/REAL(KNT)
40    CONTINUE
C
C---- COMPUTE LINEAR VALUES AT NON-CORNER NODES
      DO 70 L=1,NE
        IF(LMAT(L).LT.1) GOTO 70
        NCN=6
        IF(NOP(L,7).NE.0) NCN=8
C
C------ MIDSIDE NODES
        DO 50 K=2,NCN,2
          N1=IABS(NOP(L,K-1))
          N2=IABS(NOP(L,K))
          N3=MOD(K+1,NCN)
          N3=IABS(NOP(L,N3))
          VEL(3,N2)=0.5*(VEL(3,N1)+VEL(3,N3))
50      CONTINUE
C
C------ CENTER NODE FOR LAGRANGIAN QUADRILATERAL
        IF(NOP(L,9).NE.0) THEN
          N=IABS(NOP(L,9))
          H=0.0
          DO 60 K=1,7,2
            NN=IABS(NOP(L,K))
            H=H+VEL(3,NN)
60        CONTINUE
          VEL(3,N)=0.25*H
        ENDIF
70    CONTINUE
      IF(IDP.EQ.1) GOTO 999
C
C---- PRINT CONVERGENCE PARAMETERS
      CALL CPRINT(TITLE,IO2,WIDE,1,0)
      IF(WIDE) THEN
        WRITE(IO2,2000) TET,ITDS,NITSV
        WRITE(IO2,2001)
      ELSE
        WRITE(IO2,2100) TET,ITDS,NITSV
        WRITE(IO2,2101)
      ENDIF
      DO 80 I=1,NDF
        IF(IUNIT.EQ.0) THEN
          UNIT=UNIT0(I)
        ELSE
          UNIT=UNIT1(I)
        ENDIF
        IF(WIDE) THEN
          WRITE(IO2,2002) DOF(I),UNIT,EAVG(I),EMAX(I),NMAX(I)
        ELSE
          WRITE(IO2,2102) DOF(I),UNIT,EAVG(I),EMAX(I),NMAX(I)
        ENDIF
80    CONTINUE
999   RETURN
      END
C
C
C
      SUBROUTINE CPRINT(INSTR,IUNIT,WIDE,NSKIP1,NSKIP2)
C
C---- CPRINT ----------------------------------------------------------S
C
C  PURPOSE:
C    PRINTS A STRING CENTERED ON A LINE.
C
C  INPUT ARGUMENT(S):
C    INSTR  - STRING TO BE PRINTED.
C    IUNIT  - UNIT NUMBER OF OUTPUT FILE.
C    WIDE   - OUTPUT WIDTH CODE.
C    NSKIP1 - NUMBER OF LINES TO SKIP BEFORE INSTR IS PRINTED.
C    NSKIP2 - NUMBER OF LINES TO SKIP AFTER INSTR IS PRINTED.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IUNIT,NSKIP1,NSKIP2
      CHARACTER INSTR*(*)
      LOGICAL   WIDE
C
C---- Declarations (local)
      INTEGER I,LENGTH,LENCHR,WIDTH
      CHARACTER OUTSTR*132,BLANK*1
C
C---- Data
      DATA BLANK/' '/
C
C---- Output formats
2000  FORMAT(A)
C
      WIDTH=80
      IF(WIDE) WIDTH=132
      LENGTH=LENCHR(INSTR)
      IF(LENGTH.GT.WIDTH) LENGTH=132
      I=(WIDTH-LENGTH)/2
      OUTSTR=' '
      OUTSTR(I:I+LENGTH-1)=INSTR(1:LENGTH)
      IF(NSKIP1.GT.0) WRITE(IUNIT,2000) (BLANK,I=1,NSKIP1)
      WRITE(IUNIT,2000) OUTSTR(1:LENCHR(OUTSTR))
      IF(NSKIP2.GT.0) WRITE(IUNIT,2000) (BLANK,I=1,NSKIP2)
      RETURN
      END
C
C
C
      SUBROUTINE TAUWND(SIGMAX,SIGMAY)
C
C---- TAUWND ----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES AND ASSIGNS WIND STRESSES TO NODE POINTS.  IF WIND
C    PARAMETERS HAVE NOT BEEN PRESCRIBED FOR A PARTICULAR NODE THEN
C    GENERAL WIND STRESSES ARE ASSIGNED.
C
C  INPUT ARGUMENT(S):
C    SIGMAX - GENERAL WIND STRESS IN THE X-DIRECTION.
C    SIGMAY - GENERAL WIND STRESS IN THE Y-DIRECTION.
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      REAL      SIGMAX,SIGMAY
C
C---- Declarations (local)
      INTEGER   N
      REAL      TEMP,T1,T2,CS,CS1,CS2
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      CS1=0.001*CSURF1
      CS2=0.001*CSURF2
      DO 10 N=1,NP
        IF(SIGMA(N,1).GT.VOID) THEN
          T1=SIGMA(N,1)
          T2=SIGMA(N,2)
          IF(IUNIT.EQ.0) T1=0.3048*T1
          IF(T1.LT.WVMIN) THEN
            CS=CS1
          ELSE
            CS=CS1+CS2*(T1-WVMIN)
          ENDIF
          TEMP=CS*T1*T1
          SIGMA(N,1)=TEMP*COS(DTOR*T2)
          SIGMA(N,2)=TEMP*SIN(DTOR*T2)
        ELSE
          SIGMA(N,1)=SIGMAX
          SIGMA(N,2)=SIGMAY
        ENDIF
10    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE UPDATE(ITER,IER)
C
C---- UPDATE ----------------------------------------------------------S
C
C  PURPOSE:
C    UPDATES THE COEFFICIENT MATRIX USING BROYDEN'S METHOD AND SOLVES.
C
C  INPUT ARGUMENT(S):
C    ITER - UPDATE ITERATION NUMBER.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C  SUBPROGRAM(S) CALLED:
C    ZFRONT
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   ITER,IER
C
C---- Declarations (local)
      INTEGER   I,IHI,II,J,JHI,K
      REAL      FAC,SUM
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      II=MIN0(ITER,NUPV)
C
C---- PREPARE TO UPDATE SEARCH DIRECTION IF NUPV > 0
      IF(NUPV.GT.0) THEN
C
C------ SHIFT RESIDUAL VECTORS IF ITER > NUPV
        IF(ITER.GT.NUPV) THEN
          REWIND IO90
          REWIND IO92
          DO 10 I=1,NUPV
            READ(IO92) (DEL(K),K=1,NSZB)
            IF(I.GT.1) WRITE(IO90) (DEL(K),K=1,NSZB)
10        CONTINUE
          REWIND IO90
          REWIND IO92
          IHI=NUPV-1
          DO 20 I=1,IHI
            RHO(I)=RHO(I+1)
            READ(IO90) (DEL(K),K=1,NSZB)
            WRITE(IO92) (DEL(K),K=1,NSZB)
20        CONTINUE
          REWIND IO90
          REWIND IO91
          DO 30 I=1,NUPV
            READ(IO91) (RES(K),K=1,NSZB)
            IF(I.GT.1) WRITE(IO90) (RES(K),K=1,NSZB)
30        CONTINUE
          REWIND IO90
          REWIND IO91
          DO 40 I=1,IHI
            READ(IO90) (RES(K),K=1,NSZB)
            WRITE(IO91) (RES(K),K=1,NSZB)
40        CONTINUE
        ENDIF
C
C------ ADD NEW DEL VECTOR
        REWIND IO92
        JHI=II-1
        DO 50 J=1,JHI
          READ(IO92) (DEL(K),K=1,NSZB)
50      CONTINUE
        WRITE(IO92) (B(K),K=1,NSZB)
      ENDIF
C
C---- MODIFIED-NEWTON ITERATION TO OBTAIN FIRST SEARCH DIRECTION
      CALL ZFRONT(3,IER)
C
C---- UPDATE SEARCH DIRECTION IF NUPV > 0
      IF(NUPV.GT.0) THEN
        REWIND IO91
        REWIND IO92
        JHI=II-1
        DO 90,J=1,JHI
          FAC=0.
          READ(IO91) (RES(K),K=1,NSZB)
          READ(IO92) (DEL(K),K=1,NSZB)
          DO 70 K=1,NSZB
            FAC=FAC+DEL(K)*B(K)
70        CONTINUE
          FAC=FAC*RHO(J)
          DO 80 K=1,NSZB
            B(K)=B(K)-FAC*(DEL(K)+RES(K))
80        CONTINUE
90      CONTINUE
C
C------ FORM RES AND RHO
        FAC=0.
        SUM=0.
        READ(IO92) (DEL(K),K=1,NSZB)
        DO 100 K=1,NSZB
          RES(K)=B(K)-DEL(K)
          FAC=FAC+DEL(K)*B(K)
          SUM=SUM+DEL(K)*RES(K)
100     CONTINUE
        RHO(II)=1./SUM
        FAC=FAC*RHO(II)
        WRITE(IO91) (RES(K),K=1,NSZB)
C
C------ COMPUTE NEW SEARCH DIRECTION
        DO 110 K=1,NSZB
          B(K)=B(K)-FAC*(DEL(K)+RES(K))
110     CONTINUE
      ENDIF
  999 RETURN
      END
C
C
C
      SUBROUTINE VNCINP
C
C---- VNCINP ----------------------------------------------------------S
C
C  PURPOSE:
C    INITIALIZES VARIABLES NOT COVERED BY INPUT.
C
C----------------------------------------------------------------------F
C
C---- Declarations (local)
      INTEGER   N
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
      DO 10 N=1,NP
C------ NODE POINT COORDINATES
        IF(CORD(N,1).LT.VDX) CORD(N,1)=0.0
        IF(CORD(N,2).LT.VDX) CORD(N,2)=0.0
C------ VELOCITIES
        IF(VEL(1,N).LT.VDX) VEL(1,N)=0.0
        IF(VEL(2,N).LT.VDX) VEL(2,N)=0.0
        IF(VDOT(1,N).LT.VDX) VDOT(1,N)=0.0
        IF(VDOT(2,N).LT.VDX) VDOT(2,N)=0.0
C------ DEPTH OF FLOW
        IF(VEL(3,N).LT.VDX) VEL(3,N)=WSEL-GRND(N)
        IF(VDOT(3,N).LT.VDX) VDOT(3,N)=0.0
C------ BOUNDARY SPECIFICATIONS
        IF(SPEC(N,1).LT.VDX) SPEC(N,1)=0.0
        IF(SPEC(N,2).LT.VDX) SPEC(N,2)=0.0
        IF(SPEC(N,3).LT.VDX) SPEC(N,3)=0.0
10    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE WEIRQ(J,WS1,WS2,Q,DQDH,CSUB)
C
C---- WEIRQ -----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES FLOW OVER A WEIR SEGMENT DEFINED BY A DISCHARGE
C    COEFFICIENT, A CREST LENGTH, AND A CREST ELEVATION.
C    FLOW MAY PASS BETWEEN TWO NODE POINTS OR LEAVE THE SYSTEM FROM A
C    SINGLE NODE POINT.  THE DISCHARGE IS MODIFIED DUE TO THE EFFECT OF
C    SUBMERGENCE USING THE DISCHARGE REDUCTION FACTOR PRESENTED BY THE
C    FEDERAL HIGHWAY ADMINISTRATION IN "HYDRAULICS OF BRIDGE WATERWAYS"
C    FOR FLOW OVER ROADWAY EMBANKMENTS.
C
C  INPUT ARGUMENT(S):
C    J - WEIR SEGMENT NUMBER
C    WS1 - WATER SURFACE ELEVATION AT NODE1
C    WS2 - WATER SURFACE ELEVATION AT NODE2
C
C  OUTPUT ARGUMENT(S):
C    Q - COMPUTED DISCHARGE
C    DQDH - DERIVATIVE OF Q WITH RESPECT TO DEPTH
C    CSUB - SUBMERGENCE FACTOR
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   J
      REAL      WS1,WS2,Q,DQDH,CSUB
C
C---- Declarations (local)
      INTEGER   K,KR
      REAL      H,H1,H2,WEXP,RATIO,KW,TEMP
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- INITIALIZE VALUES AND DETERMINE HEAD
      Q=0.0
      DQDH=0.0
      CSUB=1.0
      WEXP=1.5
      H1=WS1-WCEL(J)
      H2=0.0
      IF(NOPW(J,2).GT.0) H2=WS2-WCEL(J)
      H=H1
      IF(H2.GT.H) H=H2
      IF(H.GT.0.) THEN
        IF(H2.GT.H1) H2=H1
        RATIO=H2/H
C
C------ COMPUTE SUBMERGENCE FACTOR
        IF(RATIO.GT.0.75) THEN
          DO 10 K=2,12
            KR=K
            IF(RATIO.LT.SR(K)) GOTO 20
10        CONTINUE
20        TEMP=(RATIO-SR(KR-1))/(SR(KR)-SR(KR-1))
          CSUB=(1.0-TEMP)*SF(KR-1)+TEMP*SF(KR)
        ENDIF
C
C------ COMPUTE DISCHARGE AND DERIVATIVE
        KW=CSUB*WCOF(J)*WLEN(J)*SQRT(GRAV)
        Q=KW*H**WEXP
        IF(H.LT.H1 .OR. H.GT.H1) Q=-Q
        DQDH=WEXP*Q/H
      ENDIF
      RETURN
      END
C
C
C
      SUBROUTINE XSECQ(IER)
C
C---- XSECQ -----------------------------------------------------------S
C
C  PURPOSE:
C    ASSIGNS A PORTION OF THE SPECIFIED DISCHARGE PASSING THROUGH A
C    CROSS SECTION TO NODE N BASED ON CONVEYANCE. THE PORTION OF THE
C    TOTAL CROSS SECTION DISCHARGE ASSIGNED TO NODE N IS STORED IN
C    SPEC(N,1).
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    FORMFD
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   ISECQ,J,K,KLO,KHI,L,LPREV,M,N,N1,N2,N3,N3LAST
      REAL      AREA,D,DFDH,DH,DSUM,DX,DY,DZ,FFACT,HAVG,HYDR,H1,H3,QK,
     #          SUMQK,TEMP,WETP,ZAVG,Z1,Z3
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Assigning cross-section flows...                ')
2030  FORMAT((8(I6,F10.2)))
2130  FORMAT((5(I6,F10.2)))
2040  FORMAT(8('  NODE      FLOW'))
2140  FORMAT(5('  NODE      FLOW'))
2010  FORMAT(44X,'*** CROSS SECTION TOTAL FLOW COMPUTATION ***')
2110  FORMAT(18X,'*** CROSS SECTION TOTAL FLOW COMPUTATION ***')
2011  FORMAT(' ***ERROR #??? - ELEMENT SIDE ',I5,',',I5,',',I5,
     # ' IN SECTION ',I2,' WAS NOT FOUND.')
2012  FORMAT(' ***ERROR #??? - DEPTH < 0 AT NODE ',I5,
     # ': DEPTH = ',F10.3,': GRND = ',F10.3)
2020  FORMAT(/' SECTION NO. ',I2,': TOTAL FLOW = ',F10.2,
     # ' FT3/S: AVERAGE ELEVATION = ',F8.2,' FT')
2021  FORMAT(/' SECTION NO. ',I2,': TOTAL FLOW = ',F10.2,
     # ' M3/S: AVERAGE ELEVATION = ',F8.2,' M')
C
C---- SCREEN ACKNOWLEDGEMENT
      IF(SCREEN) WRITE(6,2000) ITDS,NITSV
C
C---- Print heading for XSECQ computation
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2010)
      ELSE
        WRITE(IO2,2110)
      ENDIF
C---- Set SPEC(N,1) (tangential flow) to zero for all XSECQ nodes
      DO 10 K=1,MAXSN
        N=LXSQN(K)
        IF(N.GT.0) SPEC(N,1)=0.
10    CONTINUE
C
C---- BEGIN XSEC LOOP
      KHI=0
      DO 140 ISECQ=1,MAXS
        KLO=KHI+1
        KHI=KHI+MAXSP
        IF(LXSQN(KLO).LT.1) GOTO 140
        SUMQK=0.
C
C------ Compute average water-surface elevation and assign boundary
C------ condition codes to nodes in the cross section.
        ZAVG=0.
        DSUM=0.
        LPREV=0
        N3LAST=0
        DO 50 K=KLO,KHI,2
          N2=LXSQN(K+1)
          IF(N2.LT.1) GOTO 60
          N1=LXSQN(K)
          N3=LXSQN(K+2)
          NFIX(N1)=0
          NFIX(N2)=0
          NFIX(N3)=0
C
C-------- FIND THE ELEMENT WITH SIDE N1,N2,N3
          CALL FINDLS(N1,N2,N3,L)
          IF(L.EQ.0) THEN
            WRITE(IO2,2011) N1,N2,N3,ISECQ
            IER=IER+1
            GOTO 999
          ENDIF
C
C-------- Do not include element if it is turned off
40        IF(LMAT(L).LT.0) THEN
            IF(N3LAST.GT.0) NFIX(N3LAST)=55000
            N3LAST=0
            LPREV=0
            GOTO 50
          ENDIF
C
C-------- Assign boundary condition codes
          NFIX(N1)=56000
          NFIX(N2)=56000
          NFIX(N3)=56000
          IF(LPREV.EQ.0) NFIX(N1)=55000
          LPREV=1
          N3LAST=N3
C
C-------- Add contribution for the side
          Z1=VEL(3,N1)+GRND(N1)
          Z3=VEL(3,N3)+GRND(N3)
          DX=CORD(N3,1)-CORD(N1,1)
          DY=CORD(N3,2)-CORD(N1,2)
          D=SQRT(DX*DX+DY*DY)
          DSUM=DSUM+D
          ZAVG=ZAVG+0.5*(Z1+Z3)*D
50      CONTINUE
60      IF(N3LAST.GT.0) NFIX(N3LAST)=55000
        ZAVG=ZAVG/DSUM
C
C------ Compute cross section conveyance using average water-surface
        DO 100 K=KLO,KHI,2
          N2=LXSQN(K+1)
          IF(N2.LT.1) GOTO 110
          N1=LXSQN(K)
          N3=LXSQN(K+2)
C
C-------- FIND THE ELEMENT WITH SIDE N1,N2,N3
          CALL FINDLS(N1,N2,N3,L)
C
C-------- IF ELEMENT IS TURNED OFF DO NOT COMPUTE CONVEYANCE
90        IF(LMAT(L).LT.0) GOTO 100
C
C-------- COMPUTE CONVEYANCE THROUGH THE ELEMENT SIDE
          H1=ZAVG-GRND(N1)
          IF(H1.LT.0.) THEN
            IER=IER+1
            WRITE(IO2,2012) N1,VEL(3,N1),GRND(N1)
          ENDIF
          H3=ZAVG-GRND(N3)
          IF(H3.LT.0.) THEN
            IER=IER+1
            WRITE(IO2,2012) N3,VEL(3,N3),GRND(N3)
          ENDIF
          IF(IER.GT.0) GOTO 999
          HAVG=0.5*(H1+H3)
          DH=H3-H1
          DX=CORD(N3,1)-CORD(N1,1)
          DY=CORD(N3,2)-CORD(N1,2)
          DZ=GRND(N3)-GRND(N1)
          TEMP=SQRT(DX*DX+DY*DY)
          AREA=TEMP*HAVG
          WETP=SQRT(TEMP*TEMP+DZ*DZ)
          HYDR=AREA/WETP
          M=MOD(LMAT(L),100)
          CALL FRCFCT(HAVG,M,FFACT,DFDH)
          QK=AREA*SQRT(GRAV*HYDR/FFACT)
C
C-------- ASSIGN CONVEYANCE TO NODES
          TEMP=5.*DH/HAVG/12.
          SPEC(N1,1)=SPEC(N1,1)+QK*(1.-TEMP)/6.
          SPEC(N2,1)=SPEC(N2,1)+2.*QK/3.
          SPEC(N3,1)=SPEC(N3,1)+QK*(1.+TEMP)/6.
C
C-------- SUM CONVEYANCE FOR XSEC
          SUMQK=SUMQK+QK
100     CONTINUE
C
C------ APPORTION TOTAL FLOW TO NODES BASED ON CONVEYANCE
110      DO 120 K=KLO,KHI
          N=LXSQN(K)
          IF(N.LT.1) GOTO 130
          SPEC(N,1)=XSQ(ISECQ)*SPEC(N,1)/SUMQK
120     CONTINUE
C
C------ PRINT COMPUTED FLOWS AT NODES
130     IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2020) ISECQ,XSQ(ISECQ),ZAVG
        ELSE
          WRITE(IO2,2021) ISECQ,XSQ(ISECQ),ZAVG
        ENDIF
        IF(WIDE) THEN
          WRITE(IO2,2040)
          WRITE(IO2,2030) (LXSQN(J),SPEC(LXSQN(J),1),J=KLO,K-1)
        ELSE
          WRITE(IO2,2140)
          WRITE(IO2,2130) (LXSQN(J),SPEC(LXSQN(J),1),J=KLO,K-1)
        ENDIF
140   CONTINUE
999   RETURN
      END
C
C
C
      SUBROUTINE XSECZ(IER)
C
C---- XSECZ -----------------------------------------------------------S
C
C  PURPOSE:
C    COMPUTES WATER-SURFACE ELEVATIONS AT NODES POINTS THAT COMPTISE
C    AN OPEN-BOUNDARY CROSS SECTION.  A CONSTANT OR SLOPING WATER-
C    SURFACE CAN BE ASSIGNED.  ELEVATIONS OF A SLOPED WATER-SURFACE
C    ARE INTERPOLATED ON THE BASIS OF DISTANCE BETWEEN NODE POINTS
C    THAT DEFINE THE ENDS (LEFT AND RIGHT) OF THE CROSS SECTION.
C
C  OUTPUT ARGUMENT(S):
C    IER    - ERROR COUNT
C
C  SUBPROGRAM(S) CALLED:
C    FORMFD
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   IER
C
C---- Declarations (local)
      INTEGER   IBC,IFIX,IFIXE,IXSZ,K,KLO,KHI,KMAX,L,LPREV,N,N1,N2,N3,
     #          N3LAST
      REAL      DSUM,DTOT,DX,DY,FACT,Z,Z1,Z2
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- Output formats
2000  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Assigning cross-section elevations...           ')
2030  FORMAT((8(I6,F10.2)))
2130  FORMAT((5(I6,F10.2)))
2040  FORMAT(8('  NODE ELEVATION'))
2140  FORMAT(5('  NODE ELEVATION'))
2010  FORMAT(
     # 37X,'*** CROSS SECTION WATER-SURFACE ELEVATION COMPUTATION ***')
2110  FORMAT(
     # 11X,'*** CROSS SECTION WATER-SURFACE ELEVATION COMPUTATION ***')
2011  FORMAT(' ***ERROR #??? - ELEMENT SIDE ',I5,',',I5,',',I5,
     # ' IN SECTION ',I2,' WAS NOT FOUND.')
2012  FORMAT(' ***ERROR #??? - DEPTH < 0 AT NODE ',I5,
     # ': DEPTH = ',F10.3,': GRND = ',F10.3)
2020  FORMAT(/' SECTION NO. ',I2,': ELEVATION 1 = ',F10.3,
     # ' FT: ELEVATION 2 = ',F10.3,' FT')
2021  FORMAT(/' SECTION NO. ',I2,': ELEVATION 1 = ',F10.3,
     # ' M: ELEVATION 2 = ',F10.3,' M')
C
C---- Screen acknowledgement
      IF(SCREEN) WRITE(6,2000) ITDS,NITSV
C
C---- Print heading for cross-section computation
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2010)
      ELSE
        WRITE(IO2,2110)
      ENDIF
C
C---- Assign elevations to each cross section
      DO 90 IXSZ=1,MAXS
        KLO=1+MAXSP*(IXSZ-1)
        KMAX=KLO+MAXSP-1
        IF(LXSZN(KLO).LT.1) GOTO 90
C
C------ Assign endpoint elevations
        Z1=XSZ1(IXSZ)
        Z2=XSZ2(IXSZ)
C
C------ Assign boundary condition codes
        IBC=IBCZN(IXSZ)
        IFIX=100*IBC
        IFIXE=5000+IFIX
C------ Compute total length of cross section and assign boundary
C       condition codes
        DTOT=0.
        LPREV=0
        N3LAST=0
        DO 20 K=KLO,KMAX,2
          KHI=K
          N2=LXSZN(K+1)
          IF(N2.LT.1) GOTO 30
          N1=LXSZN(K)
          N3=LXSZN(K+2)
          NFIX(N1)=0
          NFIX(N2)=0
          NFIX(N3)=0
C
C-------- Find the element with side N1-N2-N3
          CALL FINDLS(N1,N2,N3,L)
          IF(L.GT.0) THEN
            DX=CORD(N3,1)-CORD(N1,1)
            DY=CORD(N3,2)-CORD(N1,2)
            DTOT=DTOT+SQRT(DX*DX+DY*DY)
          ELSE
            IER=IER+1
            WRITE(IO2,2011) N1,N2,N3,IXSZ
          ENDIF
C
C-------- Set LPREV and N3LAST if the element is turned off
          IF(LMAT(L).LT.0) THEN
            IF(N3LAST.GT.0) NFIX(N3LAST)=IFIXE
            N3LAST=0
            LPREV=0
          ENDIF
C
C-------- Assign boundary condition codes
          NFIX(N1)=IFIX
          NFIX(N2)=IFIX
          NFIX(N3)=IFIX
          IF(LPREV.EQ.0) NFIX(N1)=IFIXE
          LPREV=1
          N3LAST=N3
20      CONTINUE
30      IF(N3LAST.GT.0) NFIX(N3LAST)=IFIXE
C
C------ Interpolate water-surface elevation at node points
        DSUM=0.
        N=LXSZN(KLO)
        SPEC(N,3)=Z1-GRND(N)
        IF(IBC.EQ.1) VEL(3,N)=SPEC(N,3)
        DO 40 K=KLO,KHI,2
          N2=LXSZN(K+1)
          IF(N2.LT.1) GOTO 50
          N1=LXSZN(K)
          N3=LXSZN(K+2)
          DX=CORD(N3,1)-CORD(N1,1)
          DY=CORD(N3,2)-CORD(N1,2)
          DSUM=DSUM+SQRT(DX*DX+DY*DY)
          FACT=DSUM/DTOT
          Z=(1.-FACT)*Z1+FACT*Z2
          SPEC(N3,3)=Z-GRND(N3)
          SPEC(N2,3)=0.5*(SPEC(N1,3)+SPEC(N3,3))
C-------- Assign essential conditions if IBC = 1
          IF(IBC.EQ.1) THEN
            VEL(3,N3)=SPEC(N3,3)
            VEL(3,N2)=SPEC(N2,3)
          ENDIF
40      CONTINUE
C
C------ Print elevations computed for each node
50      IF(IUNIT.EQ.0) THEN
          WRITE(IO2,2020) IXSZ,XSZ1(IXSZ),XSZ2(IXSZ)
        ELSE
          WRITE(IO2,2021) IXSZ,XSZ1(IXSZ),XSZ2(IXSZ)
        ENDIF
        IF(WIDE) THEN
          WRITE(IO2,2040)
          WRITE(IO2,2030) (LXSZN(K),SPEC(LXSZN(K),3)+GRND(LXSZN(K)),
     #    K=KLO,KHI)
        ELSE
          WRITE(IO2,2140)
          WRITE(IO2,2130) (LXSZN(K),SPEC(LXSZN(K),3)+GRND(LXSZN(K)),
     #    K=KLO,KHI)
        ENDIF
88    continue
90    CONTINUE
999   RETURN
      END
C
C
C
      SUBROUTINE FINDLS(N1,N2,N3,LFOUND)
C
C---- FINDLS ----------------------------------------------------------S
C
C  PURPOSE:
C    FINDS AN ELEMENT WITH THE SIDE N1-N2-N3.
C
C  INPUT ARGUMENT(S):
C    N1     - ONE CORNER NODE.
C    N2     - THE MIDSIDE NODE.
C    N3     - THE OTHER CORNER NODE.
C
C  OUTPUT ARGUMENT(S):
C    LFOUND - ELEMENT THAT CONTAINS THE SIDE (0 IF NONE FOUND).
C
C----------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   LFOUND,N1,N2,N3
C
C---- Declarations (local)
      INTEGER   J1,J2,J3,L,NCN
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
      LFOUND=0
      DO 30 L=1,NE
        IF(LMAT(L).EQ.0) GOTO 30
        CALL LNODES(L,NCN)
        IF(NCN.EQ.9) NCN=8
        DO 20 J2=2,NCN,2
          IF(IABS(NODE(J2)).EQ.N2) THEN
            J1=J2-1
            J3=MOD(J2+1,NCN)
            IF(N1.EQ.IABS(NODE(J1)) .AND. N3.EQ.IABS(NODE(J3))) THEN
              LFOUND=L
              GOTO 40
            ELSEIF(N1.EQ.IABS(NODE(J3)) .AND. N3.EQ.IABS(NODE(J1))) THEN
              LFOUND=L
              GOTO 40
            ENDIF
          ENDIF
20      CONTINUE
30    CONTINUE
40    RETURN
      END
C
C
C
      SUBROUTINE ZFRONT(KEY,IER)
C
C---- ZFRONT ---------------------------------------------------------S
C
C  PURPOSE:
C    ASSEMBLES AND REDUCES THE SYSTEM OF FINITE ELEMENT EQUATIONS
C    USING THE FRONTAL METHOD.
C
C  INPUT ARGUMENT(S):
C    KEY - OPERATIONAL CONTROL KEY
C
C                  0 - SOLVE AX=B, DO NOT STORE LU DECOMPOSITION OF A.
C                  1 - SOLVE AX=B, STORE LU DECOMPOSITION OF A.
C                  2 - PERFORM LU DECOMPOSITION OF A ONLY AND STORE.
C          KEY  =  3 - REDUCE LOAD VECTOR B AND SOLVE FOR X USING
C                      STORED LU DECOMPOSITION; SAVE LOWER AND UPPER
C                      MATRICES.
C                  4 - REDUCE LOAD VECTOR B AND SOLVE FOR X USING
C                      STORED LU DECOMPOSITION; DELETE LOWER AND UPPER
C                      MATRICES.
C
C  OUTPUT ARGUMENT(S):
C    IER - ERROR COUNT.
C
C  SUBPROGRAM(S) CALLED:
C    LNODES, LCOEFS, FORMFD
C
C---------------------------------------------------------------------F
C
C---- Declarations (arguments)
      INTEGER   KEY,IER
C
C---- Declarations (local)
      INTEGER   I,IBUFF,IC,ICO,ICOL,IHI,II,ILO,IPIVCO,IR,IRW,
     #          J,JHI,JJ,JLO,K,L,LB,LBUFF,LENGTH,LENCHR,LL,
     #          LOCPIV,LOCPLO,LOCPUP,MAXFP,
     #          N,NBN,NCN,NEC,NEQN,NN,NPV
      REAL      FAC,PIVA,PIVOT,PIVOTI,RHS,TEMP
      LOGICAL   EXISTS
C
C---- Common blocks
      INCLUDE 'FLOMOD.INC'
C
C---- OUTPUT FORMATS
2000  FORMAT(' ***WARNING #150 - MAXF*MAXF > MAVAIL.  ',
     # 'AVAILABLE MEMORY MAY BE EXCEEDED.')
2001  FORMAT(' ***ERROR #530 - LOWER COEFFICIENT MATRIX FILE ',
     # (A),' DOES NOT EXIST.')
2002  FORMAT(' ***ERROR #540 - UPPER COEFFICIENT MATRIX FILE ',
     # (A),' DOES NOT EXIST.')
2003  FORMAT(' ***ERROR #510 - AVAILABLE MEMORY TOO SMALL TO ALLOW ',
     # 'ASSEMBLY OF ELEMENT ',I4,'.')
2004  FORMAT(' ***ERROR #??? - ERROR READING UPPER COEFFICIENT MATRIX',
     # ' FILE (IBUFF = ',I3,').')
2030  FORMAT(' ***ERROR #520 - SINGULAR MATRIX.'/
     # ' ZERO PIVOT IN EQUATION ',I5,' (NODE ',I5,', DOF ',I2,')'/
     # ' ICOL = ',I3,', CONTENTS OF IHED ARE:'/(26I5))
2031  FORMAT(' ***ERROR #520 - SINGULAR MATRIX.'/
     # ' ZERO PIVOT IN EQUATION ',I5,' (NODE ',I5,', DOF ',I2,')'/
     # ' ICOL = ',I3,' AND CONTENTS OF IHED ARE:'/(16I5))
2010  FORMAT(/37X,'***',I5,' BUFFER BLOCK(S) WRITTEN DURING FRONTAL ',
     # 'SOLUTION ***')
2130  FORMAT(/10X,' *** SOLUTIONS ***'//
     # 7X,'ICO',6X,'ICOL',8X,'LB',9X,'B(ICO)'/)
2050  FORMAT(' ***ERROR #515 - UNEXPECTED END-OF-BUFFER ENCOUNTERED ',
     # '(IBUFF = ',I5,')')
2060  FORMAT(' ***ERROR #516 - UNEXPECTED END-OF-BUFFER ENCOUNTERED ',
     # '(IBUFF = ',I4,').'/' CONTENTS OF BUFFER FOLLOW ...')
2061  FORMAT(' ***ERROR #517 - FAULTY PIVOT IN BUFFER # ',I4,
     # ' (LOCPIV = ',I4,').'/' CONTENTS OF BUFFER FOLLOW ...')
2062  FORMAT(6(I5,I5,F10.5))
2162  FORMAT(4(I5,I5,F10.5))
2163  FORMAT(/' CONTENTS OF BUFFER # ',I4,' FOLLOW ...')
2090  FORMAT(' ***ERROR #518 - ALL FULLY ASSEMBLED EQUATIONS HAVE ',
     # 'A ZERO DIAGONAL COEFFICIENT.'/
     # ' CONTENTS OF IHED ARE:'/(1X,20I6))
2190  FORMAT(' ***ERROR #518 - ALL FULLY ASSEMBLED EQUATIONS HAVE ',
     # 'A ZERO DIAGONAL COEFFICIENT.'/
     # ' CONTENTS OF IHED ARE:'/(1X,13I6))
2110  FORMAT(/10X,'***',I5,' BUFFER BLOCK(S) WRITTEN DURING FRONTAL ',
     # 'SOLUTION ***')
2501  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Assembling element ',I4,' (',I4,')...               ')
2502  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Assembling element ',I4,' (',I4,'):  Eliminating... ')
2503  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Backsubstituting (block ',I4,')...                ')
2504  FORMAT(1H+,'Time step ',I3,':  Iteration ',I2,
     # ':  Writing buffer block ',I4,'...                    ')
C
C---- Check on available storage
      MEMORY=MAXF*MAXF
      IF(MEMORY.GT.MAVAIL) WRITE(IO2,2000)
C
C---- Open coefficient matrix files.  Delete existing files if KEY < 3.
      LENGTH=2*(1+2*LBMAX)
      IF(KEY.EQ.3) THEN
        IF(EXISTS(LOWER)) THEN
Cx        OPEN(ILCOF,FILE=LOWER,STATUS='OLD',ACCESS='SEQUENTIAL',
Cx   #    FORM='UNFORMATTED')
          OPEN(ILCOF,FILE=LOWER,STATUS='OLD',ACCESS='DIRECT',
     #    FORM='UNFORMATTED',RECL=108004)
C          open(ilcof,file=lower,status='old',access='direct',
C     #    form='unformatted',recl=108004)
        ELSE
          WRITE(IO2,2001) LOWER(1:LENCHR(LOWER))
          IER=IER+1
        ENDIF
      ELSE
        IF(EXISTS(LOWER)) THEN
          OPEN(ILCOF,FILE=LOWER)
          CLOSE(ILCOF,STATUS='DELETE')
        ENDIF
Cx      OPEN(ILCOF,FILE=LOWER,STATUS='NEW',ACCESS='SEQUENTIAL',
Cx   #  FORM='UNFORMATTED')
          OPEN(ILCOF,FILE=LOWER,STATUS='NEW',ACCESS='DIRECT',
     #    FORM='UNFORMATTED',RECL=108004)
C          open(ilcof,file=lower,status='new',access='direct',
C     #    form='unformatted',recl=108004)
      ENDIF
      IF(KEY.EQ.3) THEN
        IF(EXISTS(UPPER)) THEN
Cx        OPEN(IUCOF,FILE=UPPER,STATUS='OLD',ACCESS='SEQUENTIAL',
Cx   #    FORM='UNFORMATTED')
          OPEN(IUCOF,FILE=UPPER,STATUS='OLD',ACCESS='DIRECT',
     #    FORM='UNFORMATTED',RECL=108004)
C          open(iucof,file=upper,status='old',access='direct',
C     #    form='unformatted',recl=108004)
        ELSE
          WRITE(IO2,2002) UPPER(1:LENCHR(UPPER))
          IER=IER+1
        ENDIF
      ELSE
        IF(EXISTS(UPPER)) THEN
          OPEN(IUCOF,FILE=UPPER)
          CLOSE(IUCOF,STATUS='DELETE')
        ENDIF
Cx      OPEN(IUCOF,FILE=UPPER,STATUS='NEW',ACCESS='SEQUENTIAL',
Cx   #  FORM='UNFORMATTED')
        OPEN(IUCOF,FILE=UPPER,STATUS='NEW',ACCESS='DIRECT',
     #  FORM='UNFORMATTED',RECL=108004)
C        open(iucof,file=upper,status='new',access='direct',
C     #  form='unformatted',recl=108004)
      ENDIF
C
C---- IF ERRORS DETECTED STOP EXECUTION
      IF(IER.GT.0) GOTO 999
C
C---- INITIALIZE VARIABLES
      NEC=0
      IBUFF=0
      LB=0
      LBUFF=0
      ICOL=0
      DO 10 I=1,LBMAX
        IHEDUP(I)=0
        IHEDLO(I)=0
        BUFFUP(I)=0.
        BUFFLO(I)=0.
10    CONTINUE
      DO 20 I=1,NSZB
        B(I)=0.
20    CONTINUE
      DO 30 I=1,MAVAIL
        A(I)=0.
30    CONTINUE
C
C---- LOOP AROUND ELEMENTS IN ORDER
      DO 300 LL=1,NE
      L=ISEQ(LL)
      IF(L.LT.1) THEN
        IF(KEY.EQ.3) GOTO 300
        IF(ICOL.LT.1) GOTO 300
        GOTO 140
      ENDIF
      IF(LMAT(L).LT.1) GOTO 300
      CALL LNODES(L,NCN)
      NBN=NCN*NDF
      IF(SCREEN) WRITE(6,2501) ITDS,NITSV,L,LL
C
C---- FILL ELEMENT LIAISON VECTOR
      I=0
      DO 70 K=1,NCN
        NN=NODE(K)
        N=IABS(NN)
        DO 60 J=1,NDF
          I=I+1
          IF(NN.GT.0) THEN
            LM(I)=NBC(N,J)
          ELSEIF(NN.LT.0) THEN
            LM(I)=-NBC(N,J)
          ELSE
            LM(I)=0
          ENDIF
60      CONTINUE
70    CONTINUE
C
C---- COMPUTE ELEMENT COEFFICIENT MATRIX AND LOAD VECTOR
      CALL LCOEFS(L,KEY)
C
C---- ASSEMBLE GLOBAL LOAD VECTOR
      DO 80 I=1,NBN
        II=IABS(LM(I))
        IF(II.GT.0) B(II)=B(II)+F(I)
80    CONTINUE
C
C---- IF KEY = 3 THEN ONLY FORM RESIDUAL LOAD VECTOR B
      IF(KEY.EQ.3) GOTO 300
C
C---- SET UP DESTINATION AND HEADING VECTORS
      DO 110 I=1,NBN
        IF(LM(I).NE.0) THEN
          DO 90 J=1,ICOL
            II=J
            IF(IABS(LM(I)).EQ.IABS(IHED(J))) GOTO 100
90        CONTINUE
          ICOL=ICOL+1
          II=ICOL
100       IDEST(I)=II
          IHED(II)=LM(I)
        ENDIF
110   CONTINUE
C
C---- CHECK ON NUMBER OF PARTIALLY ASSEMBLED EQUATIONS
      IF(ICOL.GT.MAXF) THEN
        IER=IER+1
        WRITE(IO2,2003) L
        GOTO 998
      ENDIF
C
C---- ASSEMBLE GLOBAL COEFFICIENT MATRIX
      DO 130 J=1,NBN
        IF(LM(J).NE.0) THEN
          JJ=(IDEST(J)-1)*MAXF
          DO 120 I=1,NBN
            IF(LM(I).NE.0) THEN
              II=IDEST(I)+JJ
              A(II)=A(II)+ECOEFM(I,J)
            ENDIF
120       CONTINUE
        ENDIF
130   CONTINUE
C
C---- FIND OUT WHICH EQUATIONS ARE COMPLETE
      IF(SCREEN) WRITE(6,2502) ITDS,NITSV,L,LL
140   IPIVCO=0
      PIVOT=0.
      NPV=0
      DO 150 I=1,ICOL
        IF(IHED(I).LT.0) THEN
          NPV=NPV+1
          II=I+MAXF*(I-1)
          PIVA=A(II)
          IF(ABS(PIVA).GT.ABS(PIVOT)) THEN
            PIVOT=PIVA
            IPIVCO=I
          ENDIF
        ENDIF
150   CONTINUE
C
C---- ASSEMBLE NEXT ELEMENT IF NOT ENOUGH EQUATIONS ARE COMPLETE
155   IF(NPV.LT.NPVMIN .AND. LL.LT.NE) GOTO 300
C
C---- CHECK FOR ALL ZERO DIAGONALS
      IF(IPIVCO.LT.1) THEN
        IER=IER+1
        IF(WIDE) THEN
          WRITE(IO2,2090) IPIVCO,(IHED(I),I=1,ICOL)
        ELSE
          WRITE(IO2,2190) IPIVCO,(IHED(I),I=1,ICOL)
        ENDIF
        GOTO 998
      ENDIF
      ICO=IABS(IHED(IPIVCO))
      IF(ABS(PIVOT).LT.1.0E-08) THEN
        DO 158 N=1,NP
          DO 157 K=1,NDF
            IF(NBC(N,K).EQ.ICO) GOTO 159
157       CONTINUE
158     CONTINUE
C------ ERROR TRAP
        N=0
        K=0
C------ PRINT ERROR MESSAGE
159     IER=IER+1
        IF(WIDE) THEN
          WRITE(IO2,2030) ICO,N,K,ICOL,(IHED(I),I=1,ICOL)
        ELSE
          WRITE(IO2,2031) ICO,N,K,ICOL,(IHED(I),I=1,ICOL)
        ENDIF
        GOTO 998
      ENDIF
C
C---- EXTRACT AND NORMALIZE PIVOTAL ROW
      II=IPIVCO-MAXF
      PIVOTI=1./PIVOT
      DO 160 I=1,ICOL
        II=II+MAXF
        PVROW(I)=PIVOTI*A(II)
160   CONTINUE
      RHS=PIVOTI*B(ICO)
      B(ICO)=RHS
      PVCOL(IPIVCO)=PIVOT
C
C---- ELIMINATE, THEN DELETE PIVOTAL ROW AND COLUMN
      II=MAXF*(IPIVCO-1)
      MAXFP=MAXF+1
C---- ROWS ABOVE PIVOTAL ROW
      IHI=IPIVCO-1
      DO 200 I=1,IHI
        II=II+1
        FAC=A(II)
        IRW=IABS(IHED(I))
        PVCOL(I)=FAC
        IF(FAC.LT.0. .OR. FAC.GT.0.) THEN
C-------- NON-ZERO FACTOR
          B(IRW)=B(IRW)-FAC*RHS
          JJ=I-MAXF
          JHI=IPIVCO-1
          DO 170 J=1,JHI
            JJ=JJ+MAXF
            A(JJ)=A(JJ)-FAC*PVROW(J)
170       CONTINUE
          JLO=IPIVCO+1
          DO 180 J=JLO,ICOL
            JJ=JJ+MAXF
            A(JJ)=A(JJ+MAXF)-FAC*PVROW(J)
180       CONTINUE
        ELSE
C-------- ZERO FACTOR
          JJ=MAXF*(IPIVCO-2)+I
          JLO=IPIVCO+1
          DO 190 J=JLO,ICOL
            JJ=JJ+MAXF
            A(JJ)=A(JJ+MAXF)
190       CONTINUE
        ENDIF
200   CONTINUE
C---- ROWS BELOW PIVOTAL ROW
      II=II+1
      ILO=IPIVCO+1
      DO 250 I=ILO,ICOL
        II=II+1
        FAC=A(II)
        IRW=IABS(IHED(I))
        PVCOL(I)=FAC
        IF(FAC.LT.0. .OR. FAC.GT.0.) THEN
C-------- NON-ZERO FACTOR
          B(IRW)=B(IRW)-FAC*RHS
          JJ=I-MAXF-1
          JHI=IPIVCO-1
          DO 210 J=1,JHI
            JJ=JJ+MAXF
            A(JJ)=A(JJ+1)-FAC*PVROW(J)
210       CONTINUE
          JLO=IPIVCO+1
          DO 220 J=JLO,ICOL
            JJ=JJ+MAXF
            A(JJ)=A(JJ+MAXFP)-FAC*PVROW(J)
220       CONTINUE
        ELSE
C-------- ZERO FACTOR
          JJ=I-MAXF-1
          JHI=IPIVCO-1
          DO 230 J=1,JHI
            JJ=JJ+MAXF
            A(JJ)=A(JJ+1)
230       CONTINUE
          JLO=IPIVCO+1
          DO 240 J=JLO,ICOL
            JJ=JJ+MAXF
            A(JJ)=A(JJ+MAXFP)
240       CONTINUE
        ENDIF
250   CONTINUE
C
C---- WRITE PIVOTAL ROW AND COLUMN TO BUFFERS
      NEC=NEC+1
      LOCPUP=LB+IPIVCO+1
      LOCPLO=LB+IPIVCO
      IHEDUP(LB+1)=0
      BUFFUP(LB+1)=0.
      DO 260 I=1,ICOL
        LB=LB+1
        IHEDUP(LB+1)=IABS(IHED(I))
        BUFFUP(LB+1)=PVROW(I)
        IHEDLO(LB)=IABS(IHED(I))
        BUFFLO(LB)=PVCOL(I)
260   CONTINUE
      IHEDLO(LB+1)=0
      BUFFLO(LB+1)=0.
      IHEDLO(LOCPLO)=-IHEDLO(LOCPLO)
      IHEDUP(LOCPUP)=-IHEDUP(LOCPUP)
      LB=LB+1
C
C---- PRINT COMPLETED EQUATION IF MOD(IDEBUG,16) > 7
      IF(MOD(IDEBUG,16).GT.7) THEN
        NEQN=IABS(IHED(IPIVCO))
        WRITE(IO2,'(/4I10,F15.5)') NEC,NEQN,ICOL,LB,B(NEQN)
        WRITE(IO2,'(/(1X,8(F10.5,''('',I4,'')'')))')
     #  (PVROW(I),IHED(I),I=1,ICOL)
      ENDIF
C
C---- DUMP BUFFERS IF FULL OR NEARLY FULL
      IF(LB.GT.LBMAX-MAXF-1) THEN
        IBUFF=IBUFF+1
        IF(SCREEN) WRITE(6,2504) ITDS,NITSV,IBUFF
        IF(KEY.GT.0) WRITE(ILCOF,REC=IBUFF) LB,(IHEDLO(I),I=1,LB),
     #  (BUFFLO(I),I=1,LB)
        WRITE(IUCOF,REC=IBUFF) LB,(IHEDUP(I),I=1,LB),(BUFFUP(I),I=1,LB)
        LB=0
      ENDIF
C
C---- ZERO OUT LAST COLUMN AND ROW
      IC=MAXF*(ICOL-1)
      IR=ICOL-MAXF
      DO 270 I=1,ICOL
        IC=IC+1
        IR=IR+MAXF
        A(IC)=0.0
        A(IR)=0.0
270   CONTINUE
C
C---- REARRANGE HEADING VECTORS
      ICOL=ICOL-1
      DO 280 I=IPIVCO,ICOL
        IHED(I)=IHED(I+1)
280   CONTINUE
      IF(ICOL.GT.0) GOTO 140
300   CONTINUE
C
C---- DUMP FINAL BUFFERS
      IBUFF=IBUFF+1
      IF(SCREEN) WRITE(6,2504) ITDS,NITSV,IBUFF
      IF(KEY.GT.0) WRITE(ILCOF,REC=IBUFF) LB,(IHEDLO(I),I=1,LB),
     # (BUFFLO(I),I=1,LB)
      WRITE(IUCOF,REC=IBUFF) LB,(IHEDUP(I),I=1,LB),(BUFFUP(I),I=1,LB)
      NBUFF=IBUFF
      LB=0
C
C---- PRINT NUMBER OF BUFFERS WRITTEN DURING FRONTAL SOLUTION
      CALL FORMFD(IO2)
      IF(WIDE) THEN
        WRITE(IO2,2010) IBUFF
      ELSE
        WRITE(IO2,2110) IBUFF
      ENDIF
C
C---- REDUCE B ONLY IF KEY = 3
      IF(KEY.EQ.3) THEN
        LB=0
        LBUFF=0
        NBUFF=0
C
C------ FILL BUFFER IF NECESSARY
305     IF(LB.EQ.LBUFF) THEN
          NBUFF=NBUFF+1
          READ(ILCOF,REC=NBUFF,ERR=340) LBUFF,(IHEDLO(I),I=1,LBUFF),
     #    (BUFFLO(I),I=1,LBUFF)
          LB=0
        ENDIF
C
C------ DETERMINE EQUATION LENGTH AND LOCATION OF PIVOT
        ILO=LB+1
        DO 310 I=ILO,LBUFF
          IF(IHEDLO(I).LT.0) LOCPIV=I
          IF(IHEDLO(I).EQ.0) GOTO 320
310     CONTINUE
C
C------ ERROR TRAP
        IER=IER+1
        WRITE(IO2,2050) NBUFF
        GOTO 998
C
C------ FORWARD ELIMINATION
320     IHI=I-1
        ICO=-IHEDLO(LOCPIV)
        IHEDLO(LOCPIV)=ICO
        RHS=B(ICO)/BUFFLO(LOCPIV)
        DO 330 I=ILO,IHI
          II=IHEDLO(I)
          B(II)=B(II)-BUFFLO(I)*RHS
330     CONTINUE
        B(ICO)=RHS
        LB=IHI+1
        GOTO 305
340     LB=0
      ENDIF
C
C---- BEGIN BACKSUBSTITUTION
      IF(MOD(IDEBUG,128).GT.63) WRITE(IO2,2130)
      DO 400 IBUFF=NBUFF,1,-1
        IF(SCREEN) WRITE(6,2503) ITDS,NITSV,IBUFF
C
C------ FILL BUFFER
        READ(IUCOF,REC=IBUFF,ERR=500)
     #  LB,(IHEDUP(I),I=1,LB),(BUFFUP(I),I=1,LB)
C
C------ DETERMINE EQUATION LENGTH AND LOCATION OF PIVOT
349     LOCPIV=0
        DO 350 I=LB,1,-1
          IF(IHEDUP(I).LT.0) LOCPIV=I
          IF(IHEDUP(I).EQ.0) THEN
            ILO=I+1
            GOTO 360
          ENDIF
350     CONTINUE
C
C------ ERROR TRAP
        IER=IER+1
        WRITE(IO2,2060) IBUFF
C
C------ CHECK FOR MISSING PIVOT
360     IF(LOCPIV.LT.1 .OR. LOCPIV.GT.LBMAX) THEN
          WRITE(IO2,2061) IBUFF,LOCPIV
          IER=IER+1
        ENDIF
C
C------ PRINT BUFFER IF ERROR ENCOUNTERED
        IF(IER.GT.0) THEN
          IF(WIDE) THEN
            WRITE(IO2,2062) (K,IHEDUP(K),BUFFUP(K),K=1,LB)
          ELSE
            WRITE(IO2,2162) (K,IHEDUP(K),BUFFUP(K),K=1,LB)
          ENDIF
          GOTO 998
        ENDIF
C
C------ PRINT HEADER AND COEFFICIENT ARRAYS IF MOD(IDEBUG,256) > 127
        IF(MOD(IDEBUG,256).GT.127) THEN
          IF(WIDE) THEN
            WRITE(IO2,2062) (K,IHEDUP(K),BUFFUP(K),K=1,LB)
          ELSE
            WRITE(IO2,2162) (K,IHEDUP(K),BUFFUP(K),K=1,LB)
          ENDIF
        ENDIF
C
C------ BACKSUBSTITUTE
        ICO=-IHEDUP(LOCPIV)
        IHEDUP(LOCPIV)=ICO
        BUFFUP(LOCPIV)=0.
        TEMP=0.
        DO 380 I=ILO,LB
          II=IHEDUP(I)
          TEMP=TEMP-BUFFUP(I)*B(II)
380     CONTINUE
        B(ICO)=B(ICO)+TEMP
C
C------ PRINT SOLUTION IF MOD(IDEBUG,128) > 63
        ICOL=LB-ILO+1
        IF(MOD(IDEBUG,128).GT.63) WRITE(IO2,'(3I10,F15.5)')
     #  ICO,ICOL,LB,B(ICO)
C
C------ ADJUST LB AND DECIDE WHAT TO DO
        LB=ILO-2
        IF(LB.GT.0) GOTO 349
400   CONTINUE
      GOTO 998
C
C---- ERROR READING UPPER COEFFICIENT MATRIX FILE
500   WRITE(IO2,2004) IBUFF
CD    IER=IER+1
C
C---- CLOSE COEFFICIENT MATRIX FILES
998   IF(KEY.EQ.0 .OR. KEY.EQ.4) THEN
        CLOSE(ILCOF,STATUS='DELETE')
        CLOSE(IUCOF,STATUS='DELETE')
      ELSE
        CLOSE(ILCOF,STATUS='KEEP')
        CLOSE(IUCOF,STATUS='KEEP')
      ENDIF
999   RETURN
      END
